/*
 *  vsccompletionvisitor.vala - Vala symbol completion library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#include <vsccompletionvisitor.h>
#include <vala/valacodenode.h>
#include <vala/valamember.h>
#include <vala/valaconstructor.h>
#include <vala/valadestructor.h>
#include <gee/collection.h>
#include <gee/iterable.h>
#include <gee/iterator.h>
#include <vala/valaobjecttypesymbol.h>
#include <vala/valaunresolvedtype.h>
#include <vala/valascope.h>
#include <vala/valausingdirective.h>
#include <vala/valaunresolvedsymbol.h>
#include <vala/valaobjecttype.h>
#include <vala/valatypeparameter.h>
#include <vsctypefindvisitor.h>
#include <vscsymbolcompletion.h>
#include <vscsymbolcompletionitem.h>




struct _VscCompletionVisitorPrivate {
	VscSymbolCompletionResult* _results;
	VscSymbolCompletionFilterOptions* _options;
	ValaCodeContext* _context;
	ValaSourceFile* _current_file;
	gboolean _parent_type_already_visited;
	char* _searched_typename;
};

#define VSC_COMPLETION_VISITOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VSC_TYPE_COMPLETION_VISITOR, VscCompletionVisitorPrivate))
enum  {
	VSC_COMPLETION_VISITOR_DUMMY_PROPERTY
};
static void vsc_completion_visitor_real_visit_namespace (ValaCodeVisitor* base, ValaNamespace* ns);
static void vsc_completion_visitor_real_visit_class (ValaCodeVisitor* base, ValaClass* cl);
static void vsc_completion_visitor_real_visit_interface (ValaCodeVisitor* base, ValaInterface* iface);
static void vsc_completion_visitor_real_visit_struct (ValaCodeVisitor* base, ValaStruct* st);
static void vsc_completion_visitor_real_visit_enum (ValaCodeVisitor* base, ValaEnum* en);
static void vsc_completion_visitor_real_visit_data_type (ValaCodeVisitor* base, ValaDataType* data_type);
static void vsc_completion_visitor_real_visit_method (ValaCodeVisitor* base, ValaMethod* m);
static void vsc_completion_visitor_real_visit_enum_value (ValaCodeVisitor* base, ValaEnumValue* e);
static void vsc_completion_visitor_real_visit_creation_method (ValaCodeVisitor* base, ValaCreationMethod* m);
static void vsc_completion_visitor_real_visit_delegate (ValaCodeVisitor* base, ValaDelegate* d);
static void vsc_completion_visitor_real_visit_signal (ValaCodeVisitor* base, ValaSignal* s);
static void vsc_completion_visitor_real_visit_field (ValaCodeVisitor* base, ValaField* f);
static void vsc_completion_visitor_real_visit_constant (ValaCodeVisitor* base, ValaConstant* c);
static void vsc_completion_visitor_real_visit_property (ValaCodeVisitor* base, ValaProperty* p);
static void vsc_completion_visitor_real_visit_error_domain (ValaCodeVisitor* base, ValaErrorDomain* ed);
static void vsc_completion_visitor_real_visit_error_code (ValaCodeVisitor* base, ValaErrorCode* ec);
static gboolean vsc_completion_visitor_test_symbol (VscCompletionVisitor* self, VscSymbolCompletionFilterOptions* options, ValaSymbol* symbol);
static gboolean vsc_completion_visitor_symbol_has_constructors (VscCompletionVisitor* self, ValaSymbol* symbol);
static gboolean vsc_completion_visitor_symbol_is_static (VscCompletionVisitor* self, ValaSymbol* symbol, gboolean instanziable_types);
static gboolean vsc_completion_visitor_class_is_static (VscCompletionVisitor* self, ValaClass* cl);
static ValaSymbol* vsc_completion_visitor_resolve_type (VscCompletionVisitor* self, ValaUnresolvedType* unresolved_type);
static char* vsc_completion_visitor_get_qualified_namespace_name (VscCompletionVisitor* self, ValaSymbol* namespace_symbol);
static gpointer vsc_completion_visitor_parent_class = NULL;
static void vsc_completion_visitor_finalize (ValaCodeVisitor* obj);



VscCompletionVisitor* vsc_completion_visitor_construct (GType object_type, VscSymbolCompletionFilterOptions* options, VscSymbolCompletionResult* results, ValaSourceFile* source, ValaCodeContext* context) {
	VscCompletionVisitor* self;
	VscSymbolCompletionResult* _tmp1;
	VscSymbolCompletionResult* _tmp0;
	VscSymbolCompletionFilterOptions* _tmp3;
	VscSymbolCompletionFilterOptions* _tmp2;
	ValaCodeContext* _tmp5;
	ValaCodeContext* _tmp4;
	ValaSourceFile* _tmp7;
	ValaSourceFile* _tmp6;
	g_return_val_if_fail (options != NULL, NULL);
	g_return_val_if_fail (results != NULL, NULL);
	self = (VscCompletionVisitor*) g_type_create_instance (object_type);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->priv->_results = (_tmp1 = (_tmp0 = results, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (self->priv->_results == NULL) ? NULL : (self->priv->_results = (g_object_unref (self->priv->_results), NULL)), _tmp1);
	_tmp3 = NULL;
	_tmp2 = NULL;
	self->priv->_options = (_tmp3 = (_tmp2 = options, (_tmp2 == NULL) ? NULL : vsc_symbol_completion_filter_options_ref (_tmp2)), (self->priv->_options == NULL) ? NULL : (self->priv->_options = (vsc_symbol_completion_filter_options_unref (self->priv->_options), NULL)), _tmp3);
	_tmp5 = NULL;
	_tmp4 = NULL;
	self->priv->_context = (_tmp5 = (_tmp4 = context, (_tmp4 == NULL) ? NULL : vala_code_context_ref (_tmp4)), (self->priv->_context == NULL) ? NULL : (self->priv->_context = (vala_code_context_unref (self->priv->_context), NULL)), _tmp5);
	_tmp7 = NULL;
	_tmp6 = NULL;
	self->priv->_current_file = (_tmp7 = (_tmp6 = source, (_tmp6 == NULL) ? NULL : vala_source_file_ref (_tmp6)), (self->priv->_current_file == NULL) ? NULL : (self->priv->_current_file = (vala_source_file_unref (self->priv->_current_file), NULL)), _tmp7);
	return self;
}


VscCompletionVisitor* vsc_completion_visitor_new (VscSymbolCompletionFilterOptions* options, VscSymbolCompletionResult* results, ValaSourceFile* source, ValaCodeContext* context) {
	return vsc_completion_visitor_construct (VSC_TYPE_COMPLETION_VISITOR, options, results, source, context);
}


void vsc_completion_visitor_integrate_completion (VscCompletionVisitor* self, ValaSymbol* symbol) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (symbol != NULL);
	self->priv->_parent_type_already_visited = FALSE;
	vala_code_node_accept ((ValaCodeNode*) symbol, (ValaCodeVisitor*) self);
}


static void vsc_completion_visitor_real_visit_namespace (ValaCodeVisitor* base, ValaNamespace* ns) {
	VscCompletionVisitor * self;
	self = (VscCompletionVisitor*) base;
	g_return_if_fail (ns != NULL);
	if (self->priv->_parent_type_already_visited) {
		if (vsc_completion_visitor_test_symbol (self, self->priv->_options, (ValaSymbol*) ns)) {
			if (!vsc_symbol_completion_result_namespaces_contain (self->priv->_results, vala_symbol_get_name ((ValaSymbol*) ns))) {
				VscSymbolCompletionItem* _tmp0;
				_tmp0 = NULL;
				gee_collection_add ((GeeCollection*) self->priv->_results->namespaces, _tmp0 = vsc_symbol_completion_item_new (vala_symbol_get_name ((ValaSymbol*) ns)));
				(_tmp0 == NULL) ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL));
			}
		}
	} else {
		self->priv->_parent_type_already_visited = TRUE;
		vala_code_node_accept_children ((ValaCodeNode*) ns, (ValaCodeVisitor*) self);
	}
}


static void vsc_completion_visitor_real_visit_class (ValaCodeVisitor* base, ValaClass* cl) {
	VscCompletionVisitor * self;
	self = (VscCompletionVisitor*) base;
	g_return_if_fail (cl != NULL);
	if (self->priv->_parent_type_already_visited) {
		gboolean _tmp0;
		_tmp0 = FALSE;
		if (!vsc_symbol_completion_result_classes_contain (self->priv->_results, vala_symbol_get_name ((ValaSymbol*) cl))) {
			_tmp0 = vsc_completion_visitor_test_symbol (self, self->priv->_options, (ValaSymbol*) cl);
		} else {
			_tmp0 = FALSE;
		}
		if (_tmp0) {
			VscSymbolCompletionItem* item;
			item = vsc_symbol_completion_item_new_with_class (cl);
			gee_collection_add ((GeeCollection*) self->priv->_results->classes, item);
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		}
	} else {
		gboolean tmp;
		tmp = self->priv->_parent_type_already_visited;
		{
			GeeList* _tmp1;
			GeeIterator* _tmp2;
			GeeIterator* _type_it;
			_tmp1 = NULL;
			_tmp2 = NULL;
			_type_it = (_tmp2 = gee_iterable_iterator ((GeeIterable*) (_tmp1 = vala_class_get_base_types (cl))), (_tmp1 == NULL) ? NULL : (_tmp1 = (gee_collection_object_unref (_tmp1), NULL)), _tmp2);
			while (gee_iterator_next (_type_it)) {
				ValaDataType* type;
				type = (ValaDataType*) gee_iterator_get (_type_it);
				if (VALA_IS_OBJECT_TYPE (type)) {
					ValaObjectType* _tmp3;
					ValaObjectType* obj;
					_tmp3 = NULL;
					obj = (_tmp3 = VALA_OBJECT_TYPE (type), (_tmp3 == NULL) ? NULL : vala_code_node_ref (_tmp3));
					vala_code_node_accept ((ValaCodeNode*) vala_object_type_get_type_symbol (obj), (ValaCodeVisitor*) self);
					(obj == NULL) ? NULL : (obj = (vala_code_node_unref (obj), NULL));
				} else {
					vala_code_node_accept ((ValaCodeNode*) type, (ValaCodeVisitor*) self);
				}
				self->priv->_parent_type_already_visited = tmp;
				(type == NULL) ? NULL : (type = (vala_code_node_unref (type), NULL));
			}
			(_type_it == NULL) ? NULL : (_type_it = (gee_collection_object_unref (_type_it), NULL));
		}
		self->priv->_parent_type_already_visited = TRUE;
		{
			GeeList* _tmp4;
			GeeIterator* _tmp5;
			GeeIterator* _p_it;
			_tmp4 = NULL;
			_tmp5 = NULL;
			_p_it = (_tmp5 = gee_iterable_iterator ((GeeIterable*) (_tmp4 = vala_object_type_symbol_get_type_parameters ((ValaObjectTypeSymbol*) cl))), (_tmp4 == NULL) ? NULL : (_tmp4 = (gee_collection_object_unref (_tmp4), NULL)), _tmp5);
			while (gee_iterator_next (_p_it)) {
				ValaTypeParameter* p;
				p = (ValaTypeParameter*) gee_iterator_get (_p_it);
				vala_code_node_accept ((ValaCodeNode*) p, (ValaCodeVisitor*) self);
				(p == NULL) ? NULL : (p = (vala_code_node_unref (p), NULL));
			}
			(_p_it == NULL) ? NULL : (_p_it = (gee_collection_object_unref (_p_it), NULL));
		}
		/* process enums first to avoid order problems in C code */
		{
			GeeList* _tmp6;
			GeeIterator* _tmp7;
			GeeIterator* _en_it;
			/* process enums first to avoid order problems in C code */
			_tmp6 = NULL;
			_tmp7 = NULL;
			_en_it = (_tmp7 = gee_iterable_iterator ((GeeIterable*) (_tmp6 = vala_class_get_enums (cl))), (_tmp6 == NULL) ? NULL : (_tmp6 = (gee_collection_object_unref (_tmp6), NULL)), _tmp7);
			/* process enums first to avoid order problems in C code */
			while (gee_iterator_next (_en_it)) {
				ValaEnum* en;
				/* process enums first to avoid order problems in C code */
				en = (ValaEnum*) gee_iterator_get (_en_it);
				vala_code_node_accept ((ValaCodeNode*) en, (ValaCodeVisitor*) self);
				(en == NULL) ? NULL : (en = (vala_code_node_unref (en), NULL));
			}
			(_en_it == NULL) ? NULL : (_en_it = (gee_collection_object_unref (_en_it), NULL));
		}
		{
			GeeList* _tmp8;
			GeeIterator* _tmp9;
			GeeIterator* _f_it;
			_tmp8 = NULL;
			_tmp9 = NULL;
			_f_it = (_tmp9 = gee_iterable_iterator ((GeeIterable*) (_tmp8 = vala_class_get_fields (cl))), (_tmp8 == NULL) ? NULL : (_tmp8 = (gee_collection_object_unref (_tmp8), NULL)), _tmp9);
			while (gee_iterator_next (_f_it)) {
				ValaField* f;
				f = (ValaField*) gee_iterator_get (_f_it);
				vala_code_node_accept ((ValaCodeNode*) f, (ValaCodeVisitor*) self);
				(f == NULL) ? NULL : (f = (vala_code_node_unref (f), NULL));
			}
			(_f_it == NULL) ? NULL : (_f_it = (gee_collection_object_unref (_f_it), NULL));
		}
		{
			GeeList* _tmp10;
			GeeIterator* _tmp11;
			GeeIterator* _c_it;
			_tmp10 = NULL;
			_tmp11 = NULL;
			_c_it = (_tmp11 = gee_iterable_iterator ((GeeIterable*) (_tmp10 = vala_class_get_constants (cl))), (_tmp10 == NULL) ? NULL : (_tmp10 = (gee_collection_object_unref (_tmp10), NULL)), _tmp11);
			while (gee_iterator_next (_c_it)) {
				ValaConstant* c;
				c = (ValaConstant*) gee_iterator_get (_c_it);
				vala_code_node_accept ((ValaCodeNode*) c, (ValaCodeVisitor*) self);
				(c == NULL) ? NULL : (c = (vala_code_node_unref (c), NULL));
			}
			(_c_it == NULL) ? NULL : (_c_it = (gee_collection_object_unref (_c_it), NULL));
		}
		{
			GeeList* _tmp12;
			GeeIterator* _tmp13;
			GeeIterator* _m_it;
			_tmp12 = NULL;
			_tmp13 = NULL;
			_m_it = (_tmp13 = gee_iterable_iterator ((GeeIterable*) (_tmp12 = vala_object_type_symbol_get_methods ((ValaObjectTypeSymbol*) cl))), (_tmp12 == NULL) ? NULL : (_tmp12 = (gee_collection_object_unref (_tmp12), NULL)), _tmp13);
			while (gee_iterator_next (_m_it)) {
				ValaMethod* m;
				m = (ValaMethod*) gee_iterator_get (_m_it);
				vala_code_node_accept ((ValaCodeNode*) m, (ValaCodeVisitor*) self);
				(m == NULL) ? NULL : (m = (vala_code_node_unref (m), NULL));
			}
			(_m_it == NULL) ? NULL : (_m_it = (gee_collection_object_unref (_m_it), NULL));
		}
		{
			GeeList* _tmp14;
			GeeIterator* _tmp15;
			GeeIterator* _prop_it;
			_tmp14 = NULL;
			_tmp15 = NULL;
			_prop_it = (_tmp15 = gee_iterable_iterator ((GeeIterable*) (_tmp14 = vala_object_type_symbol_get_properties ((ValaObjectTypeSymbol*) cl))), (_tmp14 == NULL) ? NULL : (_tmp14 = (gee_collection_object_unref (_tmp14), NULL)), _tmp15);
			while (gee_iterator_next (_prop_it)) {
				ValaProperty* prop;
				prop = (ValaProperty*) gee_iterator_get (_prop_it);
				vala_code_node_accept ((ValaCodeNode*) prop, (ValaCodeVisitor*) self);
				(prop == NULL) ? NULL : (prop = (vala_code_node_unref (prop), NULL));
			}
			(_prop_it == NULL) ? NULL : (_prop_it = (gee_collection_object_unref (_prop_it), NULL));
		}
		{
			GeeList* _tmp16;
			GeeIterator* _tmp17;
			GeeIterator* _sig_it;
			_tmp16 = NULL;
			_tmp17 = NULL;
			_sig_it = (_tmp17 = gee_iterable_iterator ((GeeIterable*) (_tmp16 = vala_object_type_symbol_get_signals ((ValaObjectTypeSymbol*) cl))), (_tmp16 == NULL) ? NULL : (_tmp16 = (gee_collection_object_unref (_tmp16), NULL)), _tmp17);
			while (gee_iterator_next (_sig_it)) {
				ValaSignal* sig;
				sig = (ValaSignal*) gee_iterator_get (_sig_it);
				vala_code_node_accept ((ValaCodeNode*) sig, (ValaCodeVisitor*) self);
				(sig == NULL) ? NULL : (sig = (vala_code_node_unref (sig), NULL));
			}
			(_sig_it == NULL) ? NULL : (_sig_it = (gee_collection_object_unref (_sig_it), NULL));
		}
		{
			GeeList* _tmp18;
			GeeIterator* _tmp19;
			GeeIterator* _subcl_it;
			_tmp18 = NULL;
			_tmp19 = NULL;
			_subcl_it = (_tmp19 = gee_iterable_iterator ((GeeIterable*) (_tmp18 = vala_class_get_classes (cl))), (_tmp18 == NULL) ? NULL : (_tmp18 = (gee_collection_object_unref (_tmp18), NULL)), _tmp19);
			while (gee_iterator_next (_subcl_it)) {
				ValaClass* subcl;
				subcl = (ValaClass*) gee_iterator_get (_subcl_it);
				vala_code_node_accept ((ValaCodeNode*) subcl, (ValaCodeVisitor*) self);
				(subcl == NULL) ? NULL : (subcl = (vala_code_node_unref (subcl), NULL));
			}
			(_subcl_it == NULL) ? NULL : (_subcl_it = (gee_collection_object_unref (_subcl_it), NULL));
		}
		{
			GeeList* _tmp20;
			GeeIterator* _tmp21;
			GeeIterator* _st_it;
			_tmp20 = NULL;
			_tmp21 = NULL;
			_st_it = (_tmp21 = gee_iterable_iterator ((GeeIterable*) (_tmp20 = vala_class_get_structs (cl))), (_tmp20 == NULL) ? NULL : (_tmp20 = (gee_collection_object_unref (_tmp20), NULL)), _tmp21);
			while (gee_iterator_next (_st_it)) {
				ValaStruct* st;
				st = (ValaStruct*) gee_iterator_get (_st_it);
				vala_code_node_accept ((ValaCodeNode*) st, (ValaCodeVisitor*) self);
				(st == NULL) ? NULL : (st = (vala_code_node_unref (st), NULL));
			}
			(_st_it == NULL) ? NULL : (_st_it = (gee_collection_object_unref (_st_it), NULL));
		}
		{
			GeeList* _tmp22;
			GeeIterator* _tmp23;
			GeeIterator* _d_it;
			_tmp22 = NULL;
			_tmp23 = NULL;
			_d_it = (_tmp23 = gee_iterable_iterator ((GeeIterable*) (_tmp22 = vala_class_get_delegates (cl))), (_tmp22 == NULL) ? NULL : (_tmp22 = (gee_collection_object_unref (_tmp22), NULL)), _tmp23);
			while (gee_iterator_next (_d_it)) {
				ValaDelegate* d;
				d = (ValaDelegate*) gee_iterator_get (_d_it);
				vala_code_node_accept ((ValaCodeNode*) d, (ValaCodeVisitor*) self);
				(d == NULL) ? NULL : (d = (vala_code_node_unref (d), NULL));
			}
			(_d_it == NULL) ? NULL : (_d_it = (gee_collection_object_unref (_d_it), NULL));
		}
	}
}


static void vsc_completion_visitor_real_visit_interface (ValaCodeVisitor* base, ValaInterface* iface) {
	VscCompletionVisitor * self;
	self = (VscCompletionVisitor*) base;
	g_return_if_fail (iface != NULL);
	{
		GeeList* _tmp0;
		GeeIterator* _tmp1;
		GeeIterator* _type_it;
		_tmp0 = NULL;
		_tmp1 = NULL;
		_type_it = (_tmp1 = gee_iterable_iterator ((GeeIterable*) (_tmp0 = vala_interface_get_prerequisites (iface))), (_tmp0 == NULL) ? NULL : (_tmp0 = (gee_collection_object_unref (_tmp0), NULL)), _tmp1);
		while (gee_iterator_next (_type_it)) {
			ValaDataType* type;
			type = (ValaDataType*) gee_iterator_get (_type_it);
			if (vala_data_type_get_data_type (type) != NULL) {
				vala_code_node_accept ((ValaCodeNode*) vala_data_type_get_data_type (type), (ValaCodeVisitor*) self);
			} else {
				vala_code_node_accept ((ValaCodeNode*) type, (ValaCodeVisitor*) self);
			}
			(type == NULL) ? NULL : (type = (vala_code_node_unref (type), NULL));
		}
		(_type_it == NULL) ? NULL : (_type_it = (gee_collection_object_unref (_type_it), NULL));
	}
	self->priv->_parent_type_already_visited = TRUE;
	{
		GeeList* _tmp2;
		GeeIterator* _tmp3;
		GeeIterator* _p_it;
		_tmp2 = NULL;
		_tmp3 = NULL;
		_p_it = (_tmp3 = gee_iterable_iterator ((GeeIterable*) (_tmp2 = vala_object_type_symbol_get_type_parameters ((ValaObjectTypeSymbol*) iface))), (_tmp2 == NULL) ? NULL : (_tmp2 = (gee_collection_object_unref (_tmp2), NULL)), _tmp3);
		while (gee_iterator_next (_p_it)) {
			ValaTypeParameter* p;
			p = (ValaTypeParameter*) gee_iterator_get (_p_it);
			vala_code_node_accept ((ValaCodeNode*) p, (ValaCodeVisitor*) self);
			(p == NULL) ? NULL : (p = (vala_code_node_unref (p), NULL));
		}
		(_p_it == NULL) ? NULL : (_p_it = (gee_collection_object_unref (_p_it), NULL));
	}
	/* process enums first to avoid order problems in C code */
	{
		GeeList* _tmp4;
		GeeIterator* _tmp5;
		GeeIterator* _en_it;
		/* process enums first to avoid order problems in C code */
		_tmp4 = NULL;
		_tmp5 = NULL;
		_en_it = (_tmp5 = gee_iterable_iterator ((GeeIterable*) (_tmp4 = vala_interface_get_enums (iface))), (_tmp4 == NULL) ? NULL : (_tmp4 = (gee_collection_object_unref (_tmp4), NULL)), _tmp5);
		/* process enums first to avoid order problems in C code */
		while (gee_iterator_next (_en_it)) {
			ValaEnum* en;
			/* process enums first to avoid order problems in C code */
			en = (ValaEnum*) gee_iterator_get (_en_it);
			vala_code_node_accept ((ValaCodeNode*) en, (ValaCodeVisitor*) self);
			(en == NULL) ? NULL : (en = (vala_code_node_unref (en), NULL));
		}
		(_en_it == NULL) ? NULL : (_en_it = (gee_collection_object_unref (_en_it), NULL));
	}
	{
		GeeList* _tmp6;
		GeeIterator* _tmp7;
		GeeIterator* _f_it;
		_tmp6 = NULL;
		_tmp7 = NULL;
		_f_it = (_tmp7 = gee_iterable_iterator ((GeeIterable*) (_tmp6 = vala_interface_get_fields (iface))), (_tmp6 == NULL) ? NULL : (_tmp6 = (gee_collection_object_unref (_tmp6), NULL)), _tmp7);
		while (gee_iterator_next (_f_it)) {
			ValaField* f;
			f = (ValaField*) gee_iterator_get (_f_it);
			vala_code_node_accept ((ValaCodeNode*) f, (ValaCodeVisitor*) self);
			(f == NULL) ? NULL : (f = (vala_code_node_unref (f), NULL));
		}
		(_f_it == NULL) ? NULL : (_f_it = (gee_collection_object_unref (_f_it), NULL));
	}
	{
		GeeList* _tmp8;
		GeeIterator* _tmp9;
		GeeIterator* _m_it;
		_tmp8 = NULL;
		_tmp9 = NULL;
		_m_it = (_tmp9 = gee_iterable_iterator ((GeeIterable*) (_tmp8 = vala_object_type_symbol_get_methods ((ValaObjectTypeSymbol*) iface))), (_tmp8 == NULL) ? NULL : (_tmp8 = (gee_collection_object_unref (_tmp8), NULL)), _tmp9);
		while (gee_iterator_next (_m_it)) {
			ValaMethod* m;
			m = (ValaMethod*) gee_iterator_get (_m_it);
			vala_code_node_accept ((ValaCodeNode*) m, (ValaCodeVisitor*) self);
			(m == NULL) ? NULL : (m = (vala_code_node_unref (m), NULL));
		}
		(_m_it == NULL) ? NULL : (_m_it = (gee_collection_object_unref (_m_it), NULL));
	}
	{
		GeeList* _tmp10;
		GeeIterator* _tmp11;
		GeeIterator* _prop_it;
		_tmp10 = NULL;
		_tmp11 = NULL;
		_prop_it = (_tmp11 = gee_iterable_iterator ((GeeIterable*) (_tmp10 = vala_object_type_symbol_get_properties ((ValaObjectTypeSymbol*) iface))), (_tmp10 == NULL) ? NULL : (_tmp10 = (gee_collection_object_unref (_tmp10), NULL)), _tmp11);
		while (gee_iterator_next (_prop_it)) {
			ValaProperty* prop;
			prop = (ValaProperty*) gee_iterator_get (_prop_it);
			vala_code_node_accept ((ValaCodeNode*) prop, (ValaCodeVisitor*) self);
			(prop == NULL) ? NULL : (prop = (vala_code_node_unref (prop), NULL));
		}
		(_prop_it == NULL) ? NULL : (_prop_it = (gee_collection_object_unref (_prop_it), NULL));
	}
	{
		GeeList* _tmp12;
		GeeIterator* _tmp13;
		GeeIterator* _sig_it;
		_tmp12 = NULL;
		_tmp13 = NULL;
		_sig_it = (_tmp13 = gee_iterable_iterator ((GeeIterable*) (_tmp12 = vala_object_type_symbol_get_signals ((ValaObjectTypeSymbol*) iface))), (_tmp12 == NULL) ? NULL : (_tmp12 = (gee_collection_object_unref (_tmp12), NULL)), _tmp13);
		while (gee_iterator_next (_sig_it)) {
			ValaSignal* sig;
			sig = (ValaSignal*) gee_iterator_get (_sig_it);
			vala_code_node_accept ((ValaCodeNode*) sig, (ValaCodeVisitor*) self);
			(sig == NULL) ? NULL : (sig = (vala_code_node_unref (sig), NULL));
		}
		(_sig_it == NULL) ? NULL : (_sig_it = (gee_collection_object_unref (_sig_it), NULL));
	}
	{
		GeeList* _tmp14;
		GeeIterator* _tmp15;
		GeeIterator* _cl_it;
		_tmp14 = NULL;
		_tmp15 = NULL;
		_cl_it = (_tmp15 = gee_iterable_iterator ((GeeIterable*) (_tmp14 = vala_interface_get_classes (iface))), (_tmp14 == NULL) ? NULL : (_tmp14 = (gee_collection_object_unref (_tmp14), NULL)), _tmp15);
		while (gee_iterator_next (_cl_it)) {
			ValaClass* cl;
			cl = (ValaClass*) gee_iterator_get (_cl_it);
			vala_code_node_accept ((ValaCodeNode*) cl, (ValaCodeVisitor*) self);
			(cl == NULL) ? NULL : (cl = (vala_code_node_unref (cl), NULL));
		}
		(_cl_it == NULL) ? NULL : (_cl_it = (gee_collection_object_unref (_cl_it), NULL));
	}
	{
		GeeList* _tmp16;
		GeeIterator* _tmp17;
		GeeIterator* _st_it;
		_tmp16 = NULL;
		_tmp17 = NULL;
		_st_it = (_tmp17 = gee_iterable_iterator ((GeeIterable*) (_tmp16 = vala_interface_get_structs (iface))), (_tmp16 == NULL) ? NULL : (_tmp16 = (gee_collection_object_unref (_tmp16), NULL)), _tmp17);
		while (gee_iterator_next (_st_it)) {
			ValaStruct* st;
			st = (ValaStruct*) gee_iterator_get (_st_it);
			vala_code_node_accept ((ValaCodeNode*) st, (ValaCodeVisitor*) self);
			(st == NULL) ? NULL : (st = (vala_code_node_unref (st), NULL));
		}
		(_st_it == NULL) ? NULL : (_st_it = (gee_collection_object_unref (_st_it), NULL));
	}
	{
		GeeList* _tmp18;
		GeeIterator* _tmp19;
		GeeIterator* _d_it;
		_tmp18 = NULL;
		_tmp19 = NULL;
		_d_it = (_tmp19 = gee_iterable_iterator ((GeeIterable*) (_tmp18 = vala_interface_get_delegates (iface))), (_tmp18 == NULL) ? NULL : (_tmp18 = (gee_collection_object_unref (_tmp18), NULL)), _tmp19);
		while (gee_iterator_next (_d_it)) {
			ValaDelegate* d;
			d = (ValaDelegate*) gee_iterator_get (_d_it);
			vala_code_node_accept ((ValaCodeNode*) d, (ValaCodeVisitor*) self);
			(d == NULL) ? NULL : (d = (vala_code_node_unref (d), NULL));
		}
		(_d_it == NULL) ? NULL : (_d_it = (gee_collection_object_unref (_d_it), NULL));
	}
}


static void vsc_completion_visitor_real_visit_struct (ValaCodeVisitor* base, ValaStruct* st) {
	VscCompletionVisitor * self;
	self = (VscCompletionVisitor*) base;
	g_return_if_fail (st != NULL);
	if (self->priv->_parent_type_already_visited) {
		gboolean _tmp0;
		_tmp0 = FALSE;
		if (!vsc_symbol_completion_result_structs_contain (self->priv->_results, vala_symbol_get_name ((ValaSymbol*) st))) {
			_tmp0 = vsc_completion_visitor_test_symbol (self, self->priv->_options, (ValaSymbol*) st);
		} else {
			_tmp0 = FALSE;
		}
		if (_tmp0) {
			VscSymbolCompletionItem* _tmp1;
			_tmp1 = NULL;
			gee_collection_add ((GeeCollection*) self->priv->_results->structs, _tmp1 = vsc_symbol_completion_item_new_with_struct (st));
			(_tmp1 == NULL) ? NULL : (_tmp1 = (g_object_unref (_tmp1), NULL));
		}
		if (vala_struct_get_base_type (st) != NULL) {
			vala_code_node_accept ((ValaCodeNode*) vala_struct_get_base_type (st), (ValaCodeVisitor*) self);
		}
	} else {
		self->priv->_parent_type_already_visited = TRUE;
		if (vala_struct_get_base_type (st) != NULL) {
			vala_code_node_accept ((ValaCodeNode*) vala_struct_get_base_type (st), (ValaCodeVisitor*) self);
		}
		{
			GeeList* _tmp2;
			GeeIterator* _tmp3;
			GeeIterator* _p_it;
			_tmp2 = NULL;
			_tmp3 = NULL;
			_p_it = (_tmp3 = gee_iterable_iterator ((GeeIterable*) (_tmp2 = vala_struct_get_type_parameters (st))), (_tmp2 == NULL) ? NULL : (_tmp2 = (gee_collection_object_unref (_tmp2), NULL)), _tmp3);
			while (gee_iterator_next (_p_it)) {
				ValaTypeParameter* p;
				p = (ValaTypeParameter*) gee_iterator_get (_p_it);
				vala_code_node_accept ((ValaCodeNode*) p, (ValaCodeVisitor*) self);
				(p == NULL) ? NULL : (p = (vala_code_node_unref (p), NULL));
			}
			(_p_it == NULL) ? NULL : (_p_it = (gee_collection_object_unref (_p_it), NULL));
		}
		{
			GeeList* _tmp4;
			GeeIterator* _tmp5;
			GeeIterator* _f_it;
			_tmp4 = NULL;
			_tmp5 = NULL;
			_f_it = (_tmp5 = gee_iterable_iterator ((GeeIterable*) (_tmp4 = vala_struct_get_fields (st))), (_tmp4 == NULL) ? NULL : (_tmp4 = (gee_collection_object_unref (_tmp4), NULL)), _tmp5);
			while (gee_iterator_next (_f_it)) {
				ValaField* f;
				f = (ValaField*) gee_iterator_get (_f_it);
				vala_code_node_accept ((ValaCodeNode*) f, (ValaCodeVisitor*) self);
				(f == NULL) ? NULL : (f = (vala_code_node_unref (f), NULL));
			}
			(_f_it == NULL) ? NULL : (_f_it = (gee_collection_object_unref (_f_it), NULL));
		}
		{
			GeeList* _tmp6;
			GeeIterator* _tmp7;
			GeeIterator* _c_it;
			_tmp6 = NULL;
			_tmp7 = NULL;
			_c_it = (_tmp7 = gee_iterable_iterator ((GeeIterable*) (_tmp6 = vala_struct_get_constants (st))), (_tmp6 == NULL) ? NULL : (_tmp6 = (gee_collection_object_unref (_tmp6), NULL)), _tmp7);
			while (gee_iterator_next (_c_it)) {
				ValaConstant* c;
				c = (ValaConstant*) gee_iterator_get (_c_it);
				vala_code_node_accept ((ValaCodeNode*) c, (ValaCodeVisitor*) self);
				(c == NULL) ? NULL : (c = (vala_code_node_unref (c), NULL));
			}
			(_c_it == NULL) ? NULL : (_c_it = (gee_collection_object_unref (_c_it), NULL));
		}
		{
			GeeList* _tmp8;
			GeeIterator* _tmp9;
			GeeIterator* _m_it;
			_tmp8 = NULL;
			_tmp9 = NULL;
			_m_it = (_tmp9 = gee_iterable_iterator ((GeeIterable*) (_tmp8 = vala_struct_get_methods (st))), (_tmp8 == NULL) ? NULL : (_tmp8 = (gee_collection_object_unref (_tmp8), NULL)), _tmp9);
			while (gee_iterator_next (_m_it)) {
				ValaMethod* m;
				m = (ValaMethod*) gee_iterator_get (_m_it);
				vala_code_node_accept ((ValaCodeNode*) m, (ValaCodeVisitor*) self);
				(m == NULL) ? NULL : (m = (vala_code_node_unref (m), NULL));
			}
			(_m_it == NULL) ? NULL : (_m_it = (gee_collection_object_unref (_m_it), NULL));
		}
	}
}


static void vsc_completion_visitor_real_visit_enum (ValaCodeVisitor* base, ValaEnum* en) {
	VscCompletionVisitor * self;
	self = (VscCompletionVisitor*) base;
	g_return_if_fail (en != NULL);
	if (self->priv->_parent_type_already_visited) {
		gboolean _tmp0;
		_tmp0 = FALSE;
		if (!vsc_symbol_completion_result_enums_contain (self->priv->_results, vala_symbol_get_name ((ValaSymbol*) en))) {
			_tmp0 = vsc_completion_visitor_test_symbol (self, self->priv->_options, (ValaSymbol*) en);
		} else {
			_tmp0 = FALSE;
		}
		if (_tmp0) {
			VscSymbolCompletionItem* _tmp1;
			_tmp1 = NULL;
			gee_collection_add ((GeeCollection*) self->priv->_results->enums, _tmp1 = vsc_symbol_completion_item_new (vala_symbol_get_name ((ValaSymbol*) en)));
			(_tmp1 == NULL) ? NULL : (_tmp1 = (g_object_unref (_tmp1), NULL));
		}
	} else {
		self->priv->_parent_type_already_visited = TRUE;
		vala_code_node_accept_children ((ValaCodeNode*) en, (ValaCodeVisitor*) self);
	}
}


static void vsc_completion_visitor_real_visit_data_type (ValaCodeVisitor* base, ValaDataType* data_type) {
	VscCompletionVisitor * self;
	ValaUnresolvedType* _tmp0;
	ValaUnresolvedType* unresolved_type;
	ValaSymbol* sym;
	self = (VscCompletionVisitor*) base;
	g_return_if_fail (data_type != NULL);
	if (!(VALA_IS_UNRESOLVED_TYPE (data_type))) {
		return;
	}
	g_return_if_fail (self->priv->_context != NULL);
	_tmp0 = NULL;
	unresolved_type = (_tmp0 = VALA_UNRESOLVED_TYPE (data_type), (_tmp0 == NULL) ? NULL : vala_code_node_ref (_tmp0));
	sym = vsc_completion_visitor_resolve_type (self, unresolved_type);
	if (sym == NULL) {
		g_warning ("vsccompletionvisitor.vala:244: (visit-data-type): can't resolve type");
		(unresolved_type == NULL) ? NULL : (unresolved_type = (vala_code_node_unref (unresolved_type), NULL));
		(sym == NULL) ? NULL : (sym = (vala_code_node_unref (sym), NULL));
		return;
	}
	if (self->priv->_parent_type_already_visited) {
		vala_code_node_accept ((ValaCodeNode*) sym, (ValaCodeVisitor*) self);
	} else {
		self->priv->_parent_type_already_visited = TRUE;
		vala_code_node_accept_children ((ValaCodeNode*) sym, (ValaCodeVisitor*) self);
	}
	(unresolved_type == NULL) ? NULL : (unresolved_type = (vala_code_node_unref (unresolved_type), NULL));
	(sym == NULL) ? NULL : (sym = (vala_code_node_unref (sym), NULL));
}


static void vsc_completion_visitor_real_visit_method (ValaCodeVisitor* base, ValaMethod* m) {
	VscCompletionVisitor * self;
	self = (VscCompletionVisitor*) base;
	g_return_if_fail (m != NULL);
	if (vsc_completion_visitor_test_symbol (self, self->priv->_options, (ValaSymbol*) m)) {
		VscSymbolCompletionItem* _tmp0;
		_tmp0 = NULL;
		gee_collection_add ((GeeCollection*) self->priv->_results->methods, _tmp0 = vsc_symbol_completion_item_new_with_method (m));
		(_tmp0 == NULL) ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL));
	}
}


static void vsc_completion_visitor_real_visit_enum_value (ValaCodeVisitor* base, ValaEnumValue* e) {
	VscCompletionVisitor * self;
	VscSymbolCompletionItem* _tmp0;
	self = (VscCompletionVisitor*) base;
	g_return_if_fail (e != NULL);
	_tmp0 = NULL;
	gee_collection_add ((GeeCollection*) self->priv->_results->constants, _tmp0 = vsc_symbol_completion_item_new (vala_symbol_get_name ((ValaSymbol*) e)));
	(_tmp0 == NULL) ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL));
}


static void vsc_completion_visitor_real_visit_creation_method (ValaCodeVisitor* base, ValaCreationMethod* m) {
	VscCompletionVisitor * self;
	self = (VscCompletionVisitor*) base;
	g_return_if_fail (m != NULL);
	if (vsc_completion_visitor_test_symbol (self, self->priv->_options, (ValaSymbol*) m)) {
		VscSymbolCompletionItem* _tmp0;
		_tmp0 = NULL;
		gee_collection_add ((GeeCollection*) self->priv->_results->methods, _tmp0 = vsc_symbol_completion_item_new_with_method ((ValaMethod*) m));
		(_tmp0 == NULL) ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL));
	}
}


static void vsc_completion_visitor_real_visit_delegate (ValaCodeVisitor* base, ValaDelegate* d) {
	VscCompletionVisitor * self;
	self = (VscCompletionVisitor*) base;
	g_return_if_fail (d != NULL);
	if (vsc_completion_visitor_test_symbol (self, self->priv->_options, (ValaSymbol*) d)) {
		VscSymbolCompletionItem* _tmp0;
		_tmp0 = NULL;
		gee_collection_add ((GeeCollection*) self->priv->_results->others, _tmp0 = vsc_symbol_completion_item_new (vala_symbol_get_name ((ValaSymbol*) d)));
		(_tmp0 == NULL) ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL));
	}
}


static void vsc_completion_visitor_real_visit_signal (ValaCodeVisitor* base, ValaSignal* s) {
	VscCompletionVisitor * self;
	self = (VscCompletionVisitor*) base;
	g_return_if_fail (s != NULL);
	if (vsc_completion_visitor_test_symbol (self, self->priv->_options, (ValaSymbol*) s)) {
		VscSymbolCompletionItem* _tmp0;
		_tmp0 = NULL;
		gee_collection_add ((GeeCollection*) self->priv->_results->signals, _tmp0 = vsc_symbol_completion_item_new_with_signal (s));
		(_tmp0 == NULL) ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL));
	}
}


static void vsc_completion_visitor_real_visit_field (ValaCodeVisitor* base, ValaField* f) {
	VscCompletionVisitor * self;
	self = (VscCompletionVisitor*) base;
	g_return_if_fail (f != NULL);
	if (vsc_completion_visitor_test_symbol (self, self->priv->_options, (ValaSymbol*) f)) {
		VscSymbolCompletionItem* _tmp0;
		_tmp0 = NULL;
		gee_collection_add ((GeeCollection*) self->priv->_results->fields, _tmp0 = vsc_symbol_completion_item_new_with_field (f));
		(_tmp0 == NULL) ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL));
	}
}


static void vsc_completion_visitor_real_visit_constant (ValaCodeVisitor* base, ValaConstant* c) {
	VscCompletionVisitor * self;
	self = (VscCompletionVisitor*) base;
	g_return_if_fail (c != NULL);
	if (vsc_completion_visitor_test_symbol (self, self->priv->_options, (ValaSymbol*) c)) {
		VscSymbolCompletionItem* _tmp0;
		_tmp0 = NULL;
		gee_collection_add ((GeeCollection*) self->priv->_results->constants, _tmp0 = vsc_symbol_completion_item_new (vala_symbol_get_name ((ValaSymbol*) c)));
		(_tmp0 == NULL) ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL));
	}
}


static void vsc_completion_visitor_real_visit_property (ValaCodeVisitor* base, ValaProperty* p) {
	VscCompletionVisitor * self;
	self = (VscCompletionVisitor*) base;
	g_return_if_fail (p != NULL);
	if (vsc_completion_visitor_test_symbol (self, self->priv->_options, (ValaSymbol*) p)) {
		VscSymbolCompletionItem* _tmp0;
		_tmp0 = NULL;
		gee_collection_add ((GeeCollection*) self->priv->_results->properties, _tmp0 = vsc_symbol_completion_item_new_with_property (p));
		(_tmp0 == NULL) ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL));
	}
}


static void vsc_completion_visitor_real_visit_error_domain (ValaCodeVisitor* base, ValaErrorDomain* ed) {
	VscCompletionVisitor * self;
	self = (VscCompletionVisitor*) base;
	g_return_if_fail (ed != NULL);
	if (!self->priv->_options->error_domains) {
		return;
	}
	if (self->priv->_parent_type_already_visited) {
		VscSymbolCompletionItem* _tmp0;
		_tmp0 = NULL;
		gee_collection_add ((GeeCollection*) self->priv->_results->error_domains, _tmp0 = vsc_symbol_completion_item_new (vala_symbol_get_name ((ValaSymbol*) ed)));
		(_tmp0 == NULL) ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL));
	} else {
		self->priv->_parent_type_already_visited = TRUE;
		vala_code_node_accept_children ((ValaCodeNode*) ed, (ValaCodeVisitor*) self);
	}
}


static void vsc_completion_visitor_real_visit_error_code (ValaCodeVisitor* base, ValaErrorCode* ec) {
	VscCompletionVisitor * self;
	VscSymbolCompletionItem* _tmp0;
	self = (VscCompletionVisitor*) base;
	g_return_if_fail (ec != NULL);
	_tmp0 = NULL;
	gee_collection_add ((GeeCollection*) self->priv->_results->constants, _tmp0 = vsc_symbol_completion_item_new (vala_symbol_get_name ((ValaSymbol*) ec)));
	(_tmp0 == NULL) ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL));
}


static gboolean vsc_completion_visitor_test_symbol (VscCompletionVisitor* self, VscSymbolCompletionFilterOptions* options, ValaSymbol* symbol) {
	gboolean res;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (options != NULL, FALSE);
	g_return_val_if_fail (symbol != NULL, FALSE);
	res = FALSE;
	if (options->error_domains) {
		gboolean _tmp0;
		_tmp0 = FALSE;
		if (options->error_base) {
			_tmp0 = VALA_IS_CLASS (symbol);
		} else {
			_tmp0 = FALSE;
		}
		if (_tmp0) {
			ValaClass* _tmp1;
			ValaClass* cl;
			_tmp1 = NULL;
			cl = (_tmp1 = VALA_CLASS (symbol), (_tmp1 == NULL) ? NULL : vala_code_node_ref (_tmp1));
			res = vala_class_get_is_error_base (cl);
			(cl == NULL) ? NULL : (cl = (vala_code_node_unref (cl), NULL));
		}
	} else {
		gboolean is_static;
		gboolean _tmp2;
		is_static = vsc_completion_visitor_symbol_is_static (self, symbol, FALSE);
		_tmp2 = FALSE;
		if (options->static_symbols) {
			_tmp2 = is_static;
		} else {
			_tmp2 = FALSE;
		}
		/*test for static or instance symbols*/
		if (_tmp2) {
			res = TRUE;
		} else {
			gboolean _tmp3;
			_tmp3 = FALSE;
			if (!options->static_symbols) {
				_tmp3 = !is_static;
			} else {
				_tmp3 = FALSE;
			}
			if (_tmp3) {
				res = TRUE;
			} else {
				gboolean _tmp4;
				_tmp4 = FALSE;
				if (!is_static) {
					_tmp4 = options->instance_symbols;
				} else {
					_tmp4 = FALSE;
				}
				if (_tmp4) {
					res = TRUE;
				}
			}
		}
		if (options->constructors) {
			if (VALA_IS_ENUM (symbol)) {
				res = FALSE | options->local_variables;
			} else {
				if (VALA_IS_CONSTANT (symbol)) {
					res = FALSE | options->local_variables;
				} else {
					gboolean _tmp5;
					_tmp5 = FALSE;
					if (VALA_IS_METHOD (symbol)) {
						_tmp5 = !(VALA_IS_CREATION_METHOD (symbol));
					} else {
						_tmp5 = FALSE;
					}
					if (_tmp5) {
						res = FALSE | options->local_variables;
					} else {
						gboolean has_constructors;
						has_constructors = vsc_completion_visitor_symbol_has_constructors (self, symbol);
						res = res | has_constructors;
					}
				}
			}
		}
		/*test symbol accessibility*/
		if (res) {
			gboolean _tmp6;
			gboolean _tmp7;
			gboolean _tmp8;
			gboolean _tmp9;
			_tmp6 = FALSE;
			_tmp7 = FALSE;
			_tmp8 = FALSE;
			_tmp9 = FALSE;
			if (options->public_symbols) {
				_tmp9 = vala_symbol_get_access (symbol) == VALA_SYMBOL_ACCESSIBILITY_PUBLIC;
			} else {
				_tmp9 = FALSE;
			}
			if (_tmp9) {
				_tmp8 = TRUE;
			} else {
				gboolean _tmp10;
				_tmp10 = FALSE;
				if (options->private_symbols) {
					_tmp10 = vala_symbol_get_access (symbol) == VALA_SYMBOL_ACCESSIBILITY_PRIVATE;
				} else {
					_tmp10 = FALSE;
				}
				_tmp8 = _tmp10;
			}
			if (_tmp8) {
				_tmp7 = TRUE;
			} else {
				gboolean _tmp11;
				_tmp11 = FALSE;
				if (options->protected_symbols) {
					_tmp11 = vala_symbol_get_access (symbol) == VALA_SYMBOL_ACCESSIBILITY_PROTECTED;
				} else {
					_tmp11 = FALSE;
				}
				_tmp7 = _tmp11;
			}
			if (_tmp7) {
				_tmp6 = TRUE;
			} else {
				gboolean _tmp12;
				_tmp12 = FALSE;
				if (options->internal_symbols) {
					_tmp12 = vala_symbol_get_access (symbol) == VALA_SYMBOL_ACCESSIBILITY_INTERNAL;
				} else {
					_tmp12 = FALSE;
				}
				_tmp6 = _tmp12;
			}
			if (_tmp6) {
				res = TRUE;
			} else {
				res = FALSE;
			}
		}
	}
	return res;
}


/*
 * Return if the Symbol can be constructed
 */
static gboolean vsc_completion_visitor_symbol_has_constructors (VscCompletionVisitor* self, ValaSymbol* symbol) {
	gboolean _tmp0;
	gboolean _tmp1;
	gboolean _tmp2;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (symbol != NULL, FALSE);
	_tmp0 = FALSE;
	_tmp1 = FALSE;
	_tmp2 = FALSE;
	if ((VALA_IS_CONSTRUCTOR (symbol))) {
		_tmp2 = TRUE;
	} else {
		_tmp2 = (VALA_IS_CLASS (symbol));
	}
	if (_tmp2) {
		_tmp1 = TRUE;
	} else {
		_tmp1 = (VALA_IS_NAMESPACE (symbol));
	}
	if (_tmp1) {
		_tmp0 = TRUE;
	} else {
		_tmp0 = (VALA_IS_CREATION_METHOD (symbol));
	}
	return _tmp0;
}


static gboolean vsc_completion_visitor_symbol_is_static (VscCompletionVisitor* self, ValaSymbol* symbol, gboolean instanziable_types) {
	gboolean res;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (symbol != NULL, FALSE);
	res = FALSE;
	if (VALA_IS_METHOD (symbol)) {
		res = vala_method_get_binding ((VALA_METHOD (symbol))) == MEMBER_BINDING_STATIC;
	} else {
		if (VALA_IS_FIELD (symbol)) {
			res = vala_field_get_binding ((VALA_FIELD (symbol))) == MEMBER_BINDING_STATIC;
		} else {
			if (VALA_IS_PROPERTY (symbol)) {
				res = vala_property_get_binding ((VALA_PROPERTY (symbol))) == MEMBER_BINDING_STATIC;
			} else {
				if (VALA_IS_CONSTRUCTOR (symbol)) {
					res = vala_constructor_get_binding ((VALA_CONSTRUCTOR (symbol))) == MEMBER_BINDING_STATIC;
				} else {
					if (VALA_IS_DESTRUCTOR (symbol)) {
						res = vala_destructor_get_binding ((VALA_DESTRUCTOR (symbol))) == MEMBER_BINDING_STATIC;
					} else {
						if (VALA_IS_ENUM (symbol)) {
							res = TRUE;
						} else {
							if (VALA_IS_CONSTANT (symbol)) {
								res = TRUE;
							} else {
								if (VALA_IS_NAMESPACE (symbol)) {
									res = TRUE;
								} else {
									if (VALA_IS_CLASS (symbol)) {
										/*class is fully visited only if looking for
										static symbols*/
										if (self->priv->_options->static_symbols) {
											ValaClass* _tmp0;
											ValaClass* cl;
											_tmp0 = NULL;
											cl = (_tmp0 = VALA_CLASS (symbol), (_tmp0 == NULL) ? NULL : vala_code_node_ref (_tmp0));
											res = vsc_completion_visitor_class_is_static (self, cl);
											(cl == NULL) ? NULL : (cl = (vala_code_node_unref (cl), NULL));
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}
	return res;
}


static gboolean vsc_completion_visitor_class_is_static (VscCompletionVisitor* self, ValaClass* cl) {
	gboolean _tmp0;
	GeeList* _tmp1;
	gboolean _tmp2;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (cl != NULL, FALSE);
	_tmp0 = FALSE;
	_tmp1 = NULL;
	if ((_tmp2 = gee_collection_get_size ((GeeCollection*) (_tmp1 = vala_class_get_enums (cl))) > 0, (_tmp1 == NULL) ? NULL : (_tmp1 = (gee_collection_object_unref (_tmp1), NULL)), _tmp2)) {
		_tmp0 = TRUE;
	} else {
		GeeList* _tmp3;
		_tmp3 = NULL;
		_tmp0 = gee_collection_get_size ((GeeCollection*) (_tmp3 = vala_class_get_constants (cl))) > 0;
		(_tmp3 == NULL) ? NULL : (_tmp3 = (gee_collection_object_unref (_tmp3), NULL));
	}
	if (_tmp0) {
		return TRUE;
	}
	{
		GeeList* _tmp5;
		GeeIterator* _tmp6;
		GeeIterator* _f_it;
		_tmp5 = NULL;
		_tmp6 = NULL;
		_f_it = (_tmp6 = gee_iterable_iterator ((GeeIterable*) (_tmp5 = vala_class_get_fields (cl))), (_tmp5 == NULL) ? NULL : (_tmp5 = (gee_collection_object_unref (_tmp5), NULL)), _tmp6);
		while (gee_iterator_next (_f_it)) {
			ValaField* f;
			f = (ValaField*) gee_iterator_get (_f_it);
			if (vsc_completion_visitor_symbol_is_static (self, (ValaSymbol*) f, FALSE)) {
				gboolean _tmp7;
				return (_tmp7 = TRUE, (f == NULL) ? NULL : (f = (vala_code_node_unref (f), NULL)), (_f_it == NULL) ? NULL : (_f_it = (gee_collection_object_unref (_f_it), NULL)), _tmp7);
			}
			(f == NULL) ? NULL : (f = (vala_code_node_unref (f), NULL));
		}
		(_f_it == NULL) ? NULL : (_f_it = (gee_collection_object_unref (_f_it), NULL));
	}
	{
		GeeList* _tmp8;
		GeeIterator* _tmp9;
		GeeIterator* _m_it;
		_tmp8 = NULL;
		_tmp9 = NULL;
		_m_it = (_tmp9 = gee_iterable_iterator ((GeeIterable*) (_tmp8 = vala_object_type_symbol_get_methods ((ValaObjectTypeSymbol*) cl))), (_tmp8 == NULL) ? NULL : (_tmp8 = (gee_collection_object_unref (_tmp8), NULL)), _tmp9);
		while (gee_iterator_next (_m_it)) {
			ValaMethod* m;
			m = (ValaMethod*) gee_iterator_get (_m_it);
			if (vsc_completion_visitor_symbol_is_static (self, (ValaSymbol*) m, FALSE)) {
				gboolean _tmp10;
				return (_tmp10 = TRUE, (m == NULL) ? NULL : (m = (vala_code_node_unref (m), NULL)), (_m_it == NULL) ? NULL : (_m_it = (gee_collection_object_unref (_m_it), NULL)), _tmp10);
			}
			(m == NULL) ? NULL : (m = (vala_code_node_unref (m), NULL));
		}
		(_m_it == NULL) ? NULL : (_m_it = (gee_collection_object_unref (_m_it), NULL));
	}
	{
		GeeList* _tmp11;
		GeeIterator* _tmp12;
		GeeIterator* _prop_it;
		_tmp11 = NULL;
		_tmp12 = NULL;
		_prop_it = (_tmp12 = gee_iterable_iterator ((GeeIterable*) (_tmp11 = vala_object_type_symbol_get_properties ((ValaObjectTypeSymbol*) cl))), (_tmp11 == NULL) ? NULL : (_tmp11 = (gee_collection_object_unref (_tmp11), NULL)), _tmp12);
		while (gee_iterator_next (_prop_it)) {
			ValaProperty* prop;
			prop = (ValaProperty*) gee_iterator_get (_prop_it);
			if (vsc_completion_visitor_symbol_is_static (self, (ValaSymbol*) prop, FALSE)) {
				gboolean _tmp13;
				return (_tmp13 = TRUE, (prop == NULL) ? NULL : (prop = (vala_code_node_unref (prop), NULL)), (_prop_it == NULL) ? NULL : (_prop_it = (gee_collection_object_unref (_prop_it), NULL)), _tmp13);
			}
			(prop == NULL) ? NULL : (prop = (vala_code_node_unref (prop), NULL));
		}
		(_prop_it == NULL) ? NULL : (_prop_it = (gee_collection_object_unref (_prop_it), NULL));
	}
	{
		GeeList* _tmp14;
		GeeIterator* _tmp15;
		GeeIterator* _st_it;
		_tmp14 = NULL;
		_tmp15 = NULL;
		_st_it = (_tmp15 = gee_iterable_iterator ((GeeIterable*) (_tmp14 = vala_class_get_structs (cl))), (_tmp14 == NULL) ? NULL : (_tmp14 = (gee_collection_object_unref (_tmp14), NULL)), _tmp15);
		while (gee_iterator_next (_st_it)) {
			ValaStruct* st;
			st = (ValaStruct*) gee_iterator_get (_st_it);
			if (vsc_completion_visitor_symbol_is_static (self, (ValaSymbol*) st, FALSE)) {
				gboolean _tmp16;
				return (_tmp16 = TRUE, (st == NULL) ? NULL : (st = (vala_code_node_unref (st), NULL)), (_st_it == NULL) ? NULL : (_st_it = (gee_collection_object_unref (_st_it), NULL)), _tmp16);
			}
			(st == NULL) ? NULL : (st = (vala_code_node_unref (st), NULL));
		}
		(_st_it == NULL) ? NULL : (_st_it = (gee_collection_object_unref (_st_it), NULL));
	}
	{
		GeeList* _tmp17;
		GeeIterator* _tmp18;
		GeeIterator* _d_it;
		_tmp17 = NULL;
		_tmp18 = NULL;
		_d_it = (_tmp18 = gee_iterable_iterator ((GeeIterable*) (_tmp17 = vala_class_get_delegates (cl))), (_tmp17 == NULL) ? NULL : (_tmp17 = (gee_collection_object_unref (_tmp17), NULL)), _tmp18);
		while (gee_iterator_next (_d_it)) {
			ValaDelegate* d;
			d = (ValaDelegate*) gee_iterator_get (_d_it);
			if (vsc_completion_visitor_symbol_is_static (self, (ValaSymbol*) d, FALSE)) {
				gboolean _tmp19;
				return (_tmp19 = TRUE, (d == NULL) ? NULL : (d = (vala_code_node_unref (d), NULL)), (_d_it == NULL) ? NULL : (_d_it = (gee_collection_object_unref (_d_it), NULL)), _tmp19);
			}
			(d == NULL) ? NULL : (d = (vala_code_node_unref (d), NULL));
		}
		(_d_it == NULL) ? NULL : (_d_it = (gee_collection_object_unref (_d_it), NULL));
	}
	{
		GeeList* _tmp20;
		GeeIterator* _tmp21;
		GeeIterator* _type_it;
		_tmp20 = NULL;
		_tmp21 = NULL;
		_type_it = (_tmp21 = gee_iterable_iterator ((GeeIterable*) (_tmp20 = vala_class_get_base_types (cl))), (_tmp20 == NULL) ? NULL : (_tmp20 = (gee_collection_object_unref (_tmp20), NULL)), _tmp21);
		while (gee_iterator_next (_type_it)) {
			ValaDataType* type;
			type = (ValaDataType*) gee_iterator_get (_type_it);
			if (VALA_IS_UNRESOLVED_TYPE (type)) {
				ValaUnresolvedType* _tmp22;
				ValaUnresolvedType* unresolved_type;
				ValaSymbol* sym;
				_tmp22 = NULL;
				unresolved_type = (_tmp22 = VALA_UNRESOLVED_TYPE (type), (_tmp22 == NULL) ? NULL : vala_code_node_ref (_tmp22));
				sym = vsc_completion_visitor_resolve_type (self, unresolved_type);
				if (sym != NULL) {
					if (vsc_completion_visitor_symbol_is_static (self, sym, FALSE)) {
						gboolean _tmp23;
						return (_tmp23 = TRUE, (unresolved_type == NULL) ? NULL : (unresolved_type = (vala_code_node_unref (unresolved_type), NULL)), (sym == NULL) ? NULL : (sym = (vala_code_node_unref (sym), NULL)), (type == NULL) ? NULL : (type = (vala_code_node_unref (type), NULL)), (_type_it == NULL) ? NULL : (_type_it = (gee_collection_object_unref (_type_it), NULL)), _tmp23);
					}
				}
				(unresolved_type == NULL) ? NULL : (unresolved_type = (vala_code_node_unref (unresolved_type), NULL));
				(sym == NULL) ? NULL : (sym = (vala_code_node_unref (sym), NULL));
			} else {
				if (vsc_completion_visitor_symbol_is_static (self, (ValaSymbol*) vala_data_type_get_data_type (type), FALSE)) {
					gboolean _tmp24;
					return (_tmp24 = TRUE, (type == NULL) ? NULL : (type = (vala_code_node_unref (type), NULL)), (_type_it == NULL) ? NULL : (_type_it = (gee_collection_object_unref (_type_it), NULL)), _tmp24);
				}
			}
			(type == NULL) ? NULL : (type = (vala_code_node_unref (type), NULL));
		}
		(_type_it == NULL) ? NULL : (_type_it = (gee_collection_object_unref (_type_it), NULL));
	}
	{
		GeeList* _tmp25;
		GeeIterator* _tmp26;
		GeeIterator* _subcl_it;
		_tmp25 = NULL;
		_tmp26 = NULL;
		_subcl_it = (_tmp26 = gee_iterable_iterator ((GeeIterable*) (_tmp25 = vala_class_get_classes (cl))), (_tmp25 == NULL) ? NULL : (_tmp25 = (gee_collection_object_unref (_tmp25), NULL)), _tmp26);
		while (gee_iterator_next (_subcl_it)) {
			ValaClass* subcl;
			subcl = (ValaClass*) gee_iterator_get (_subcl_it);
			if (vsc_completion_visitor_symbol_is_static (self, (ValaSymbol*) subcl, FALSE)) {
				gboolean _tmp27;
				return (_tmp27 = TRUE, (subcl == NULL) ? NULL : (subcl = (vala_code_node_unref (subcl), NULL)), (_subcl_it == NULL) ? NULL : (_subcl_it = (gee_collection_object_unref (_subcl_it), NULL)), _tmp27);
			}
			(subcl == NULL) ? NULL : (subcl = (vala_code_node_unref (subcl), NULL));
		}
		(_subcl_it == NULL) ? NULL : (_subcl_it = (gee_collection_object_unref (_subcl_it), NULL));
	}
	return FALSE;
}


static ValaSymbol* vsc_completion_visitor_resolve_type (VscCompletionVisitor* self, ValaUnresolvedType* unresolved_type) {
	ValaSymbol* sym;
	char* name;
	gboolean _tmp2;
	ValaSymbol* _tmp12;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (unresolved_type != NULL, NULL);
	sym = NULL;
	name = vala_data_type_to_qualified_string ((ValaDataType*) unresolved_type, NULL);
	if (name == NULL) {
		char* _tmp1;
		const char* _tmp0;
		_tmp1 = NULL;
		_tmp0 = NULL;
		name = (_tmp1 = (_tmp0 = vala_symbol_get_name ((ValaSymbol*) vala_unresolved_type_get_unresolved_symbol (unresolved_type)), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), name = (g_free (name), NULL), _tmp1);
	}
	_tmp2 = FALSE;
	if (sym == NULL) {
		_tmp2 = self->priv->_current_file != NULL;
	} else {
		_tmp2 = FALSE;
	}
	if (_tmp2) {
		VscTypeFinderVisitor* typefinder;
		ValaSymbol* _tmp4;
		ValaSymbol* _tmp3;
		gboolean _tmp5;
		typefinder = vsc_typefinder_visitor_new (NULL, NULL);
		/*trying on the root context*/
		vsc_typefinder_visitor_set_searched_typename (typefinder, name);
		vala_code_visitor_visit_namespace ((ValaCodeVisitor*) typefinder, vala_code_context_get_root (self->priv->_context));
		_tmp4 = NULL;
		_tmp3 = NULL;
		sym = (_tmp4 = (_tmp3 = vsc_typefinder_visitor_get_result (typefinder), (_tmp3 == NULL) ? NULL : vala_code_node_ref (_tmp3)), (sym == NULL) ? NULL : (sym = (vala_code_node_unref (sym), NULL)), _tmp4);
		_tmp5 = FALSE;
		if (sym == NULL) {
			_tmp5 = !vsc_symbol_completion_symbol_has_known_namespace (name);
		} else {
			_tmp5 = FALSE;
		}
		if (_tmp5) {
			{
				GeeList* _tmp6;
				GeeIterator* _tmp7;
				GeeIterator* _item_it;
				_tmp6 = NULL;
				_tmp7 = NULL;
				_item_it = (_tmp7 = gee_iterable_iterator ((GeeIterable*) (_tmp6 = vala_source_file_get_using_directives (self->priv->_current_file))), (_tmp6 == NULL) ? NULL : (_tmp6 = (gee_collection_object_unref (_tmp6), NULL)), _tmp7);
				while (gee_iterator_next (_item_it)) {
					ValaUsingDirective* item;
					char* ns_name;
					char* _tmp8;
					gboolean _tmp9;
					char* using_name;
					item = (ValaUsingDirective*) gee_iterator_get (_item_it);
					ns_name = vsc_completion_visitor_get_qualified_namespace_name (self, vala_using_directive_get_namespace_symbol (item));
					_tmp8 = NULL;
					if ((_tmp9 = g_str_has_prefix (name, _tmp8 = g_strdup_printf ("%s.", ns_name)), _tmp8 = (g_free (_tmp8), NULL), _tmp9)) {
						(item == NULL) ? NULL : (item = (vala_code_node_unref (item), NULL));
						ns_name = (g_free (ns_name), NULL);
						break;
					}
					/*exit since symbol is fully qualified*/
					using_name = g_strdup_printf ("%s.%s", ns_name, name);
					vsc_typefinder_visitor_set_searched_typename (typefinder, using_name);
					vala_code_visitor_visit_namespace ((ValaCodeVisitor*) typefinder, vala_code_context_get_root (self->priv->_context));
					if (vsc_typefinder_visitor_get_result (typefinder) != NULL) {
						ValaSymbol* _tmp11;
						ValaSymbol* _tmp10;
						_tmp11 = NULL;
						_tmp10 = NULL;
						sym = (_tmp11 = (_tmp10 = vsc_typefinder_visitor_get_result (typefinder), (_tmp10 == NULL) ? NULL : vala_code_node_ref (_tmp10)), (sym == NULL) ? NULL : (sym = (vala_code_node_unref (sym), NULL)), _tmp11);
						(item == NULL) ? NULL : (item = (vala_code_node_unref (item), NULL));
						ns_name = (g_free (ns_name), NULL);
						using_name = (g_free (using_name), NULL);
						break;
					}
					(item == NULL) ? NULL : (item = (vala_code_node_unref (item), NULL));
					ns_name = (g_free (ns_name), NULL);
					using_name = (g_free (using_name), NULL);
				}
				(_item_it == NULL) ? NULL : (_item_it = (gee_collection_object_unref (_item_it), NULL));
			}
		}
		(typefinder == NULL) ? NULL : (typefinder = (vala_code_visitor_unref (typefinder), NULL));
	}
	_tmp12 = NULL;
	return (_tmp12 = sym, name = (g_free (name), NULL), _tmp12);
}


static char* vsc_completion_visitor_get_qualified_namespace_name (VscCompletionVisitor* self, ValaSymbol* namespace_symbol) {
	char* ns_name;
	ValaSymbol* _tmp0;
	ValaSymbol* ns;
	char* _tmp8;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (namespace_symbol != NULL, NULL);
	ns_name = NULL;
	_tmp0 = NULL;
	ns = (_tmp0 = namespace_symbol, (_tmp0 == NULL) ? NULL : vala_code_node_ref (_tmp0));
	while (ns != NULL) {
		if (ns_name == NULL) {
			char* _tmp2;
			const char* _tmp1;
			_tmp2 = NULL;
			_tmp1 = NULL;
			ns_name = (_tmp2 = (_tmp1 = vala_symbol_get_name (ns), (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), ns_name = (g_free (ns_name), NULL), _tmp2);
		} else {
			char* _tmp3;
			_tmp3 = NULL;
			ns_name = (_tmp3 = g_strdup_printf ("%s.%s", vala_symbol_get_name (ns), ns_name), ns_name = (g_free (ns_name), NULL), _tmp3);
		}
		if (VALA_IS_UNRESOLVED_SYMBOL (ns)) {
			ValaSymbol* _tmp5;
			ValaSymbol* _tmp4;
			_tmp5 = NULL;
			_tmp4 = NULL;
			ns = (_tmp5 = (_tmp4 = (ValaSymbol*) vala_unresolved_symbol_get_inner ((VALA_UNRESOLVED_SYMBOL (ns))), (_tmp4 == NULL) ? NULL : vala_code_node_ref (_tmp4)), (ns == NULL) ? NULL : (ns = (vala_code_node_unref (ns), NULL)), _tmp5);
		} else {
			ValaSymbol* _tmp7;
			ValaSymbol* _tmp6;
			_tmp7 = NULL;
			_tmp6 = NULL;
			ns = (_tmp7 = (_tmp6 = vala_symbol_get_parent_symbol (ns), (_tmp6 == NULL) ? NULL : vala_code_node_ref (_tmp6)), (ns == NULL) ? NULL : (ns = (vala_code_node_unref (ns), NULL)), _tmp7);
		}
	}
	_tmp8 = NULL;
	return (_tmp8 = ns_name, (ns == NULL) ? NULL : (ns = (vala_code_node_unref (ns), NULL)), _tmp8);
}


const char* vsc_completion_visitor_get_searched_typename (VscCompletionVisitor* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_searched_typename;
}


void vsc_completion_visitor_set_searched_typename (VscCompletionVisitor* self, const char* value) {
	char* _tmp2;
	const char* _tmp1;
	g_return_if_fail (self != NULL);
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_searched_typename = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), self->priv->_searched_typename = (g_free (self->priv->_searched_typename), NULL), _tmp2);
}


static void vsc_completion_visitor_class_init (VscCompletionVisitorClass * klass) {
	vsc_completion_visitor_parent_class = g_type_class_peek_parent (klass);
	VALA_CODE_VISITOR_CLASS (klass)->finalize = vsc_completion_visitor_finalize;
	g_type_class_add_private (klass, sizeof (VscCompletionVisitorPrivate));
	VALA_CODE_VISITOR_CLASS (klass)->visit_namespace = vsc_completion_visitor_real_visit_namespace;
	VALA_CODE_VISITOR_CLASS (klass)->visit_class = vsc_completion_visitor_real_visit_class;
	VALA_CODE_VISITOR_CLASS (klass)->visit_interface = vsc_completion_visitor_real_visit_interface;
	VALA_CODE_VISITOR_CLASS (klass)->visit_struct = vsc_completion_visitor_real_visit_struct;
	VALA_CODE_VISITOR_CLASS (klass)->visit_enum = vsc_completion_visitor_real_visit_enum;
	VALA_CODE_VISITOR_CLASS (klass)->visit_data_type = vsc_completion_visitor_real_visit_data_type;
	VALA_CODE_VISITOR_CLASS (klass)->visit_method = vsc_completion_visitor_real_visit_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_enum_value = vsc_completion_visitor_real_visit_enum_value;
	VALA_CODE_VISITOR_CLASS (klass)->visit_creation_method = vsc_completion_visitor_real_visit_creation_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_delegate = vsc_completion_visitor_real_visit_delegate;
	VALA_CODE_VISITOR_CLASS (klass)->visit_signal = vsc_completion_visitor_real_visit_signal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_field = vsc_completion_visitor_real_visit_field;
	VALA_CODE_VISITOR_CLASS (klass)->visit_constant = vsc_completion_visitor_real_visit_constant;
	VALA_CODE_VISITOR_CLASS (klass)->visit_property = vsc_completion_visitor_real_visit_property;
	VALA_CODE_VISITOR_CLASS (klass)->visit_error_domain = vsc_completion_visitor_real_visit_error_domain;
	VALA_CODE_VISITOR_CLASS (klass)->visit_error_code = vsc_completion_visitor_real_visit_error_code;
}


static void vsc_completion_visitor_instance_init (VscCompletionVisitor * self) {
	self->priv = VSC_COMPLETION_VISITOR_GET_PRIVATE (self);
	self->priv->_current_file = NULL;
	self->priv->_parent_type_already_visited = FALSE;
	self->priv->_searched_typename = NULL;
}


static void vsc_completion_visitor_finalize (ValaCodeVisitor* obj) {
	VscCompletionVisitor * self;
	self = VSC_COMPLETION_VISITOR (obj);
	(self->priv->_results == NULL) ? NULL : (self->priv->_results = (g_object_unref (self->priv->_results), NULL));
	(self->priv->_options == NULL) ? NULL : (self->priv->_options = (vsc_symbol_completion_filter_options_unref (self->priv->_options), NULL));
	(self->priv->_context == NULL) ? NULL : (self->priv->_context = (vala_code_context_unref (self->priv->_context), NULL));
	(self->priv->_current_file == NULL) ? NULL : (self->priv->_current_file = (vala_source_file_unref (self->priv->_current_file), NULL));
	self->priv->_searched_typename = (g_free (self->priv->_searched_typename), NULL);
	VALA_CODE_VISITOR_CLASS (vsc_completion_visitor_parent_class)->finalize (obj);
}


GType vsc_completion_visitor_get_type (void) {
	static GType vsc_completion_visitor_type_id = 0;
	if (vsc_completion_visitor_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (VscCompletionVisitorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vsc_completion_visitor_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VscCompletionVisitor), 0, (GInstanceInitFunc) vsc_completion_visitor_instance_init, NULL };
		vsc_completion_visitor_type_id = g_type_register_static (VALA_TYPE_CODE_VISITOR, "VscCompletionVisitor", &g_define_type_info, 0);
	}
	return vsc_completion_visitor_type_id;
}




