/*
 *  vscparsermanager.vala - Vala symbol completion library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#include <vscparsermanager.h>
#include <gee/arraylist.h>
#include <gee/list.h>
#include <gee/collection.h>
#include <vala/valasourcefile.h>
#include <gee/iterable.h>
#include <gee/iterator.h>
#include <vala/valausingdirective.h>
#include <vala/valaunresolvedsymbol.h>
#include <vala/valasourcereference.h>
#include <vala/valasymbol.h>
#include <vala/valaparser.h>
#include <vala/valasymbolresolver.h>
#include <glib/gstdio.h>
#include <vscsymbolcompletion.h>




static glong string_get_length (const char* self);
static char* string_substring (const char* self, glong offset, glong len);
struct _VscParserManagerPrivate {
	char* glib_file;
	char* gobject_file;
	GList* _vapidirs;
	GeeList* _packages;
	GeeList* _built_packages;
	GeeList* _sources;
	GeeList* _source_buffers;
	ValaCodeContext* _pri_context;
	ValaCodeContext* _sec_context;
	gint need_parse_sec_context;
	gint need_parse_pri_context;
	GThread* parser_pri_thread;
	GThread* parser_sec_thread;
	GMutex* mutex_pri_context;
	GMutex* mutex_sec_context;
	gboolean parsing_suspended;
};

#define VSC_PARSER_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VSC_TYPE_PARSER_MANAGER, VscParserManagerPrivate))
enum  {
	VSC_PARSER_MANAGER_DUMMY_PROPERTY,
	VSC_PARSER_MANAGER_PRI_CONTEXT,
	VSC_PARSER_MANAGER_SEC_CONTEXT
};
static void _g_list_free_g_free (GList* self);
static void* _vsc_parser_manager_parse_pri_contexts_gthread_func (gpointer self);
static void vsc_parser_manager_create_pri_thread (VscParserManager* self);
static void* _vsc_parser_manager_parse_sec_contexts_gthread_func (gpointer self);
static void vsc_parser_manager_create_sec_thread (VscParserManager* self);
static void vsc_parser_manager_schedule_parse_source_buffers (VscParserManager* self);
static void vsc_parser_manager_schedule_parse (VscParserManager* self);
static void* vsc_parser_manager_parse_pri_contexts (VscParserManager* self);
static void* vsc_parser_manager_parse_sec_contexts (VscParserManager* self);
static void vsc_parser_manager_parse_source_buffers (VscParserManager* self);
static void vsc_parser_manager_parse (VscParserManager* self);
static void vsc_parser_manager_analyze_context (VscParserManager* self, ValaCodeContext* context);
static char* vsc_parser_manager_find_vala_package_name (VscParserManager* self, const char* namespace, GError** error);
static GeeList* vsc_parser_manager_find_vala_package_filename (VscParserManager* self, const char* package_name, GError** error);
static gboolean vsc_parser_manager_list_contains_string (VscParserManager* self, GeeList* list, const char* value);
static GObject * vsc_parser_manager_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer vsc_parser_manager_parent_class = NULL;
static void vsc_parser_manager_finalize (GObject* obj);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);



static glong string_get_length (const char* self) {
	g_return_val_if_fail (self != NULL, 0L);
	return g_utf8_strlen (self, -1);
}


static char* string_substring (const char* self, glong offset, glong len) {
	glong string_length;
	const char* start;
	g_return_val_if_fail (self != NULL, NULL);
	string_length = g_utf8_strlen (self, -1);
	if (offset < 0) {
		offset = string_length + offset;
		g_return_val_if_fail (offset >= 0, NULL);
	} else {
		g_return_val_if_fail (offset <= string_length, NULL);
	}
	if (len < 0) {
		len = string_length - offset;
	}
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
	start = g_utf8_offset_to_pointer (self, offset);
	return g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
}


static void _g_list_free_g_free (GList* self) {
	g_list_foreach (self, (GFunc) g_free, NULL);
	g_list_free (self);
}


void vsc_parser_manager_lock_all_contexts (VscParserManager* self) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (self->priv->mutex_pri_context != NULL);
	g_mutex_lock (self->priv->mutex_pri_context);
	g_return_if_fail (self->priv->mutex_sec_context != NULL);
	g_mutex_lock (self->priv->mutex_sec_context);
}


void vsc_parser_manager_unlock_all_contexts (VscParserManager* self) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (self->priv->mutex_sec_context != NULL);
	g_mutex_unlock (self->priv->mutex_sec_context);
	g_return_if_fail (self->priv->mutex_pri_context != NULL);
	g_mutex_unlock (self->priv->mutex_pri_context);
}


void vsc_parser_manager_lock_pri_context (VscParserManager* self) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (self->priv->mutex_pri_context != NULL);
	g_mutex_lock (self->priv->mutex_pri_context);
}


void vsc_parser_manager_unlock_pri_context (VscParserManager* self) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (self->priv->mutex_pri_context != NULL);
	g_mutex_unlock (self->priv->mutex_pri_context);
}


void vsc_parser_manager_lock_sec_context (VscParserManager* self) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (self->priv->mutex_sec_context != NULL);
	g_mutex_lock (self->priv->mutex_sec_context);
}


void vsc_parser_manager_unlock_sec_context (VscParserManager* self) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (self->priv->mutex_sec_context != NULL);
	g_mutex_unlock (self->priv->mutex_sec_context);
}


static void* _vsc_parser_manager_parse_pri_contexts_gthread_func (gpointer self) {
	return vsc_parser_manager_parse_pri_contexts (self);
}


static void vsc_parser_manager_create_pri_thread (VscParserManager* self) {
	GError * inner_error;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	{
		GThread* _tmp0;
		if (self->priv->parser_pri_thread != NULL) {
			g_thread_join (self->priv->parser_pri_thread);
		}
		_tmp0 = g_thread_create (_vsc_parser_manager_parse_pri_contexts_gthread_func, self, TRUE, &inner_error);
		if (inner_error != NULL) {
			if (inner_error->domain == G_THREAD_ERROR) {
				goto __catch1_g_thread_error;
			}
			goto __finally1;
		}
		self->priv->parser_pri_thread = _tmp0;
	}
	goto __finally1;
	__catch1_g_thread_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			g_error ("vscparsermanager.vala:154: Can't create parser thread: %s", err->message);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally1:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
}


static void* _vsc_parser_manager_parse_sec_contexts_gthread_func (gpointer self) {
	return vsc_parser_manager_parse_sec_contexts (self);
}


static void vsc_parser_manager_create_sec_thread (VscParserManager* self) {
	GError * inner_error;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	if (self->priv->parsing_suspended) {
		return;
	}
	{
		GThread* _tmp0;
		if (self->priv->parser_sec_thread != NULL) {
			g_thread_join (self->priv->parser_sec_thread);
		}
		_tmp0 = g_thread_create (_vsc_parser_manager_parse_sec_contexts_gthread_func, self, TRUE, &inner_error);
		if (inner_error != NULL) {
			if (inner_error->domain == G_THREAD_ERROR) {
				goto __catch2_g_thread_error;
			}
			goto __finally2;
		}
		self->priv->parser_sec_thread = _tmp0;
	}
	goto __finally2;
	__catch2_g_thread_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			g_error ("vscparsermanager.vala:169: Can't create parser thread: %s", err->message);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally2:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
}


void vsc_parser_manager_suspend_parsing (VscParserManager* self) {
	g_return_if_fail (self != NULL);
	vsc_parser_manager_lock_all_contexts (self);
	self->priv->parsing_suspended = TRUE;
	vsc_parser_manager_unlock_all_contexts (self);
}


void vsc_parser_manager_resume_parsing (VscParserManager* self) {
	g_return_if_fail (self != NULL);
	vsc_parser_manager_lock_all_contexts (self);
	self->priv->parsing_suspended = FALSE;
	vsc_parser_manager_unlock_all_contexts (self);
	vsc_parser_manager_schedule_parse (self);
}


gboolean vsc_parser_manager_add_path_to_vapi_search_dir (VscParserManager* self, const char* path) {
	gboolean _tmp0;
	const char* _tmp2;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (path != NULL, FALSE);
	_tmp0 = FALSE;
	if (!g_file_test (path, G_FILE_TEST_IS_DIR)) {
		_tmp0 = TRUE;
	} else {
		_tmp0 = g_list_find_custom (self->priv->_vapidirs, path, strcmp) != NULL;
	}
	if (_tmp0) {
		return FALSE;
	}
	_tmp2 = NULL;
	self->priv->_vapidirs = g_list_append (self->priv->_vapidirs, (_tmp2 = path, (_tmp2 == NULL) ? NULL : g_strdup (_tmp2)));
	vsc_parser_manager_schedule_parse (self);
	return TRUE;
}


gboolean vsc_parser_manager_add_built_package (VscParserManager* self, const char* vapi_file) {
	gboolean result;
	char* built_vapi_file;
	gboolean _tmp3;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (vapi_file != NULL, FALSE);
	result = FALSE;
	built_vapi_file = NULL;
	if (g_str_has_suffix (vapi_file, ".vapi")) {
		char* _tmp1;
		const char* _tmp0;
		_tmp1 = NULL;
		_tmp0 = NULL;
		built_vapi_file = (_tmp1 = (_tmp0 = vapi_file, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), built_vapi_file = (g_free (built_vapi_file), NULL), _tmp1);
	} else {
		char* _tmp2;
		_tmp2 = NULL;
		built_vapi_file = (_tmp2 = g_strdup_printf ("%s.vapi", vapi_file), built_vapi_file = (g_free (built_vapi_file), NULL), _tmp2);
	}
	if (!vsc_parser_manager_list_contains_string (self, self->priv->_built_packages, built_vapi_file)) {
		result = TRUE;
		gee_collection_add ((GeeCollection*) self->priv->_built_packages, built_vapi_file);
		vsc_parser_manager_schedule_parse (self);
	}
	return (_tmp3 = result, built_vapi_file = (g_free (built_vapi_file), NULL), _tmp3);
}


gboolean vsc_parser_manager_remove_build_package (VscParserManager* self, const char* vapi_file) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (vapi_file != NULL, FALSE);
	result = FALSE;
	if (vsc_parser_manager_list_contains_string (self, self->priv->_built_packages, vapi_file)) {
		result = TRUE;
		gee_collection_remove ((GeeCollection*) self->priv->_built_packages, vapi_file);
		vsc_parser_manager_schedule_parse (self);
	}
	return result;
}


gboolean vsc_parser_manager_add_package_from_namespace (VscParserManager* self, const char* namespace, gboolean auto_schedule_parse, GError** error) {
	GError * inner_error;
	char* package_name;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (namespace != NULL, FALSE);
	inner_error = NULL;
	if (namespace == NULL) {
		return FALSE;
	}
	package_name = vsc_parser_manager_find_vala_package_name (self, namespace, &inner_error);
	if (inner_error != NULL) {
		g_propagate_error (error, inner_error);
		return FALSE;
	}
	if (package_name != NULL) {
		gboolean _tmp1;
		gboolean _tmp2;
		_tmp1 = vsc_parser_manager_add_package (self, package_name, auto_schedule_parse, &inner_error);
		if (inner_error != NULL) {
			g_propagate_error (error, inner_error);
			package_name = (g_free (package_name), NULL);
			return FALSE;
		}
		return (_tmp2 = _tmp1, package_name = (g_free (package_name), NULL), _tmp2);
	} else {
		gboolean _tmp3;
		return (_tmp3 = FALSE, package_name = (g_free (package_name), NULL), _tmp3);
	}
	package_name = (g_free (package_name), NULL);
}


void vsc_parser_manager_remove_package_from_namespace (VscParserManager* self, const char* namespace, GError** error) {
	GError * inner_error;
	char* package_name;
	g_return_if_fail (self != NULL);
	g_return_if_fail (namespace != NULL);
	inner_error = NULL;
	package_name = vsc_parser_manager_find_vala_package_name (self, namespace, &inner_error);
	if (inner_error != NULL) {
		g_propagate_error (error, inner_error);
		return;
	}
	vsc_parser_manager_remove_package (self, package_name, &inner_error);
	if (inner_error != NULL) {
		g_propagate_error (error, inner_error);
		package_name = (g_free (package_name), NULL);
		return;
	}
	package_name = (g_free (package_name), NULL);
}


void vsc_parser_manager_remove_package (VscParserManager* self, const char* package_name, GError** error) {
	GError * inner_error;
	GeeList* files;
	char* _tmp0;
	gboolean _tmp1;
	g_return_if_fail (self != NULL);
	g_return_if_fail (package_name != NULL);
	inner_error = NULL;
	files = vsc_parser_manager_find_vala_package_filename (self, package_name, &inner_error);
	if (inner_error != NULL) {
		g_propagate_error (error, inner_error);
		return;
	}
	_tmp0 = NULL;
	if ((_tmp1 = vsc_parser_manager_list_contains_string (self, self->priv->_packages, _tmp0 = (char*) gee_list_get (files, 0)), _tmp0 = (g_free (_tmp0), NULL), _tmp1)) {
		char* _tmp2;
		vsc_parser_manager_lock_pri_context (self);
		_tmp2 = NULL;
		gee_collection_remove ((GeeCollection*) files, _tmp2 = (char*) gee_list_get (files, 0));
		_tmp2 = (g_free (_tmp2), NULL);
		vsc_parser_manager_unlock_pri_context (self);
		vsc_parser_manager_schedule_parse (self);
	}
	(files == NULL) ? NULL : (files = (gee_collection_object_unref (files), NULL));
}


gboolean vsc_parser_manager_try_add_package (VscParserManager* self, const char* package_name, gboolean auto_schedule_parse) {
	GError * inner_error;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (package_name != NULL, FALSE);
	inner_error = NULL;
	{
		vsc_parser_manager_add_package (self, package_name, auto_schedule_parse, &inner_error);
		if (inner_error != NULL) {
			goto __catch3_g_error;
			goto __finally3;
		}
		return TRUE;
	}
	goto __finally3;
	__catch3_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			gboolean _tmp1;
			return (_tmp1 = FALSE, (err == NULL) ? NULL : (err = (g_error_free (err), NULL)), _tmp1);
		}
	}
	__finally3:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return FALSE;
	}
}


gboolean vsc_parser_manager_add_package (VscParserManager* self, const char* package_name, gboolean auto_schedule_parse, GError** error) {
	GError * inner_error;
	char* vapi_file;
	GeeList* files;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (package_name != NULL, FALSE);
	inner_error = NULL;
	vapi_file = NULL;
	if (g_str_has_suffix (package_name, ".vapi")) {
		char* _tmp1;
		const char* _tmp0;
		_tmp1 = NULL;
		_tmp0 = NULL;
		vapi_file = (_tmp1 = (_tmp0 = package_name, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), vapi_file = (g_free (vapi_file), NULL), _tmp1);
	} else {
		char* _tmp2;
		_tmp2 = NULL;
		vapi_file = (_tmp2 = g_strdup_printf ("%s.vapi", package_name), vapi_file = (g_free (vapi_file), NULL), _tmp2);
	}
	if (vsc_parser_manager_list_contains_string (self, self->priv->_built_packages, vapi_file)) {
		gboolean _tmp3;
		return (_tmp3 = FALSE, vapi_file = (g_free (vapi_file), NULL), _tmp3);
	}
	files = vsc_parser_manager_find_vala_package_filename (self, vapi_file, &inner_error);
	if (inner_error != NULL) {
		g_propagate_error (error, inner_error);
		vapi_file = (g_free (vapi_file), NULL);
		return FALSE;
	}
	if (gee_collection_get_size ((GeeCollection*) files) > 0) {
		gboolean need_parse;
		gboolean _tmp4;
		gboolean _tmp5;
		need_parse = FALSE;
		vsc_parser_manager_lock_pri_context (self);
		{
			GeeIterator* _filename_it;
			_filename_it = gee_iterable_iterator ((GeeIterable*) files);
			while (gee_iterator_next (_filename_it)) {
				char* filename;
				filename = (char*) gee_iterator_get (_filename_it);
				if (!vsc_parser_manager_list_contains_string (self, self->priv->_packages, filename)) {
					gee_collection_add ((GeeCollection*) self->priv->_packages, filename);
					need_parse = TRUE;
				}
				filename = (g_free (filename), NULL);
			}
			(_filename_it == NULL) ? NULL : (_filename_it = (gee_collection_object_unref (_filename_it), NULL));
		}
		vsc_parser_manager_unlock_pri_context (self);
		_tmp4 = FALSE;
		if (need_parse) {
			_tmp4 = auto_schedule_parse;
		} else {
			_tmp4 = FALSE;
		}
		if (_tmp4) {
			vsc_parser_manager_schedule_parse (self);
		}
		return (_tmp5 = need_parse, vapi_file = (g_free (vapi_file), NULL), (files == NULL) ? NULL : (files = (gee_collection_object_unref (files), NULL)), _tmp5);
	} else {
		inner_error = g_error_new_literal (VSC_SYMBOL_COMPLETION_ERROR, VSC_SYMBOL_COMPLETION_ERROR_PACKAGE_FILE_NOT_FOUND, "package file not found");
		if (inner_error != NULL) {
			g_propagate_error (error, inner_error);
			vapi_file = (g_free (vapi_file), NULL);
			(files == NULL) ? NULL : (files = (gee_collection_object_unref (files), NULL));
			return FALSE;
		}
	}
	vapi_file = (g_free (vapi_file), NULL);
	(files == NULL) ? NULL : (files = (gee_collection_object_unref (files), NULL));
}


void vsc_parser_manager_add_source (VscParserManager* self, const char* filename, GError** error) {
	GError * inner_error;
	g_return_if_fail (self != NULL);
	g_return_if_fail (filename != NULL);
	inner_error = NULL;
	if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
		if (!vsc_parser_manager_list_contains_string (self, self->priv->_sources, filename)) {
			vsc_parser_manager_lock_pri_context (self);
			gee_collection_add ((GeeCollection*) self->priv->_sources, filename);
			vsc_parser_manager_unlock_pri_context (self);
			vsc_parser_manager_schedule_parse (self);
		}
	} else {
		inner_error = g_error_new_literal (VSC_SYMBOL_COMPLETION_ERROR, VSC_SYMBOL_COMPLETION_ERROR_PACKAGE_FILE_NOT_FOUND, "source file not found");
		if (inner_error != NULL) {
			g_propagate_error (error, inner_error);
			return;
		}
	}
}


void vsc_parser_manager_remove_source (VscParserManager* self, const char* filename, GError** error) {
	GError * inner_error;
	g_return_if_fail (self != NULL);
	g_return_if_fail (filename != NULL);
	inner_error = NULL;
	if (!vsc_parser_manager_list_contains_string (self, self->priv->_sources, filename)) {
		vsc_parser_manager_lock_pri_context (self);
		gee_collection_remove ((GeeCollection*) self->priv->_sources, filename);
		vsc_parser_manager_unlock_pri_context (self);
		vsc_parser_manager_schedule_parse (self);
	} else {
		inner_error = g_error_new_literal (VSC_SYMBOL_COMPLETION_ERROR, VSC_SYMBOL_COMPLETION_ERROR_PACKAGE_FILE_NOT_FOUND, "source file not found");
		if (inner_error != NULL) {
			g_propagate_error (error, inner_error);
			return;
		}
	}
}


gboolean vsc_parser_manager_contains_source (VscParserManager* self, const char* filename) {
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (filename != NULL, FALSE);
	return vsc_parser_manager_list_contains_string (self, self->priv->_sources, filename);
}


void vsc_parser_manager_add_source_buffer (VscParserManager* self, VscSourceBuffer* source, GError** error) {
	GError * inner_error;
	g_return_if_fail (self != NULL);
	g_return_if_fail (source != NULL);
	inner_error = NULL;
	if (vsc_parser_manager_contains_source_buffer (self, source)) {
		inner_error = g_error_new_literal (VSC_SYMBOL_COMPLETION_ERROR, VSC_SYMBOL_COMPLETION_ERROR_SOURCE_BUFFER, "source already added");
		if (inner_error != NULL) {
			g_propagate_error (error, inner_error);
			return;
		}
	}
	vsc_parser_manager_lock_sec_context (self);
	gee_collection_add ((GeeCollection*) self->priv->_source_buffers, source);
	vsc_parser_manager_unlock_sec_context (self);
	vsc_parser_manager_schedule_parse_source_buffers (self);
}


void vsc_parser_manager_remove_source_buffer_by_name (VscParserManager* self, const char* name, GError** error) {
	GError * inner_error;
	g_return_if_fail (self != NULL);
	g_return_if_fail (name != NULL);
	inner_error = NULL;
	{
		GeeIterator* _item_it;
		_item_it = gee_iterable_iterator ((GeeIterable*) self->priv->_source_buffers);
		while (gee_iterator_next (_item_it)) {
			VscSourceBuffer* item;
			item = (VscSourceBuffer*) gee_iterator_get (_item_it);
			if (_vala_strcmp0 (vsc_source_buffer_get_name (item), name) == 0) {
				vsc_parser_manager_remove_source_buffer (self, item);
				(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
				(_item_it == NULL) ? NULL : (_item_it = (gee_collection_object_unref (_item_it), NULL));
				return;
			}
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (gee_collection_object_unref (_item_it), NULL));
	}
	inner_error = g_error_new_literal (VSC_SYMBOL_COMPLETION_ERROR, VSC_SYMBOL_COMPLETION_ERROR_SOURCE_BUFFER, "source not found");
	if (inner_error != NULL) {
		g_propagate_error (error, inner_error);
		return;
	}
}


gboolean vsc_parser_manager_contains_source_buffer (VscParserManager* self, VscSourceBuffer* source) {
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (source != NULL, FALSE);
	return vsc_parser_manager_contains_source_buffer_by_name (self, vsc_source_buffer_get_name (source));
}


gboolean vsc_parser_manager_contains_source_buffer_by_name (VscParserManager* self, const char* name) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (name != NULL, FALSE);
	result = FALSE;
	vsc_parser_manager_lock_sec_context (self);
	{
		GeeIterator* _item_it;
		_item_it = gee_iterable_iterator ((GeeIterable*) self->priv->_source_buffers);
		while (gee_iterator_next (_item_it)) {
			VscSourceBuffer* item;
			item = (VscSourceBuffer*) gee_iterator_get (_item_it);
			if (_vala_strcmp0 (vsc_source_buffer_get_name (item), name) == 0) {
				result = TRUE;
				(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
				break;
			}
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (gee_collection_object_unref (_item_it), NULL));
	}
	vsc_parser_manager_unlock_sec_context (self);
	return result;
}


void vsc_parser_manager_remove_source_buffer (VscParserManager* self, VscSourceBuffer* source) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (source != NULL);
	vsc_parser_manager_lock_sec_context (self);
	gee_collection_remove ((GeeCollection*) self->priv->_source_buffers, source);
	vsc_parser_manager_unlock_sec_context (self);
	vsc_parser_manager_schedule_parse_source_buffers (self);
}


void vsc_parser_manager_reparse_source_buffers (VscParserManager* self) {
	g_return_if_fail (self != NULL);
	vsc_parser_manager_schedule_parse_source_buffers (self);
}


gboolean vsc_parser_manager_is_cache_building (VscParserManager* self) {
	gboolean result;
	gboolean _tmp0;
	g_return_val_if_fail (self != NULL, FALSE);
	result = FALSE;
	_tmp0 = FALSE;
	if (self->priv->need_parse_pri_context > 0) {
		_tmp0 = TRUE;
	} else {
		_tmp0 = self->priv->need_parse_sec_context > 0;
	}
	result = _tmp0;
	return result;
}


void vsc_parser_manager_reparse (VscParserManager* self, gboolean all_context) {
	g_return_if_fail (self != NULL);
	vsc_parser_manager_schedule_parse_source_buffers (self);
	if (all_context) {
		vsc_parser_manager_schedule_parse (self);
	}
}


static void vsc_parser_manager_schedule_parse_source_buffers (VscParserManager* self) {
	g_return_if_fail (self != NULL);
	if (self->priv->parsing_suspended) {
		return;
	}
	/*scheduling parse for secondary context*/
	if (g_atomic_int_compare_and_exchange (&self->priv->need_parse_sec_context, 0, 1)) {
		vsc_parser_manager_create_sec_thread (self);
	} else {
		g_atomic_int_inc (&self->priv->need_parse_sec_context);
	}
}


static void vsc_parser_manager_schedule_parse (VscParserManager* self) {
	g_return_if_fail (self != NULL);
	if (self->priv->parsing_suspended) {
		return;
	}
	/*scheduling parse for primary context*/
	if (g_atomic_int_compare_and_exchange (&self->priv->need_parse_pri_context, 0, 1)) {
		vsc_parser_manager_create_pri_thread (self);
	} else {
		g_atomic_int_inc (&self->priv->need_parse_pri_context);
	}
}


static void* vsc_parser_manager_parse_pri_contexts (VscParserManager* self) {
	g_return_val_if_fail (self != NULL, NULL);
	g_signal_emit_by_name (self, "cache-building");
	while (TRUE) {
		gint stamp;
		stamp = g_atomic_int_get (&self->priv->need_parse_pri_context);
		vsc_parser_manager_parse (self);
		/*check for changes*/
		if (g_atomic_int_compare_and_exchange (&self->priv->need_parse_pri_context, stamp, 0)) {
			break;
		}
	}
	g_signal_emit_by_name (self, "cache-builded");
	return ((void*) 0);
}


static void* vsc_parser_manager_parse_sec_contexts (VscParserManager* self) {
	g_return_val_if_fail (self != NULL, NULL);
	g_signal_emit_by_name (self, "cache-building");
	while (TRUE) {
		gint stamp;
		stamp = g_atomic_int_get (&self->priv->need_parse_sec_context);
		vsc_parser_manager_parse_source_buffers (self);
		/*check for changes*/
		if (g_atomic_int_compare_and_exchange (&self->priv->need_parse_sec_context, stamp, 0)) {
			break;
		}
	}
	g_signal_emit_by_name (self, "cache-builded");
	return ((void*) 0);
}


static void vsc_parser_manager_parse_source_buffers (VscParserManager* self) {
	GError * inner_error;
	ValaCodeContext* current_context;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	current_context = vala_code_context_new ();
	vsc_parser_manager_lock_sec_context (self);
	if (gee_collection_get_size ((GeeCollection*) self->priv->_source_buffers) != 0) {
		ValaSourceFile* source;
		ValaSourceFile* _tmp0;
		ValaSourceFile* _tmp1;
		gboolean need_reparse;
		ValaCodeContext* _tmp15;
		ValaCodeContext* _tmp14;
		source = NULL;
		_tmp0 = NULL;
		source = (_tmp0 = vala_source_file_new (current_context, self->priv->glib_file, TRUE, NULL), (source == NULL) ? NULL : (source = (vala_source_file_unref (source), NULL)), _tmp0);
		vala_code_context_add_source_file (current_context, source);
		_tmp1 = NULL;
		source = (_tmp1 = vala_source_file_new (current_context, self->priv->gobject_file, TRUE, NULL), (source == NULL) ? NULL : (source = (vala_source_file_unref (source), NULL)), _tmp1);
		vala_code_context_add_source_file (current_context, source);
		{
			GeeIterator* _src_it;
			_src_it = gee_iterable_iterator ((GeeIterable*) self->priv->_source_buffers);
			while (gee_iterator_next (_src_it)) {
				VscSourceBuffer* src;
				gboolean _tmp2;
				src = (VscSourceBuffer*) gee_iterator_get (_src_it);
				_tmp2 = FALSE;
				if (vsc_source_buffer_get_name (src) != NULL) {
					_tmp2 = vsc_source_buffer_get_source (src) != NULL;
				} else {
					_tmp2 = FALSE;
				}
				if (_tmp2) {
					const char* _tmp3;
					char* name;
					ValaSourceFile* _tmp5;
					ValaUsingDirective* _tmp7;
					ValaUnresolvedSymbol* _tmp6;
					_tmp3 = NULL;
					name = (_tmp3 = vsc_source_buffer_get_name (src), (_tmp3 == NULL) ? NULL : g_strdup (_tmp3));
					if (!g_str_has_suffix (name, ".vala")) {
						char* _tmp4;
						_tmp4 = NULL;
						name = (_tmp4 = g_strdup_printf ("%s.vala", name), name = (g_free (name), NULL), _tmp4);
					}
					_tmp5 = NULL;
					source = (_tmp5 = vala_source_file_new (current_context, name, FALSE, vsc_source_buffer_get_source (src)), (source == NULL) ? NULL : (source = (vala_source_file_unref (source), NULL)), _tmp5);
					_tmp7 = NULL;
					_tmp6 = NULL;
					vala_source_file_add_using_directive (source, _tmp7 = vala_using_directive_new ((ValaSymbol*) (_tmp6 = vala_unresolved_symbol_new (NULL, "GLib", NULL)), NULL));
					(_tmp7 == NULL) ? NULL : (_tmp7 = (vala_code_node_unref (_tmp7), NULL));
					(_tmp6 == NULL) ? NULL : (_tmp6 = (vala_code_node_unref (_tmp6), NULL));
					vala_code_context_add_source_file (current_context, source);
					name = (g_free (name), NULL);
				}
				(src == NULL) ? NULL : (src = (g_object_unref (src), NULL));
			}
			(_src_it == NULL) ? NULL : (_src_it = (gee_collection_object_unref (_src_it), NULL));
		}
		vsc_parser_manager_unlock_sec_context (self);
		vsc_parser_manager_parse_context (self, current_context);
		need_reparse = FALSE;
		/*add new namespaces to standard context)*/
		{
			GeeList* _tmp8;
			GeeIterator* _tmp9;
			GeeIterator* _src_it;
			/*add new namespaces to standard context)*/
			_tmp8 = NULL;
			_tmp9 = NULL;
			_src_it = (_tmp9 = gee_iterable_iterator ((GeeIterable*) (_tmp8 = vala_code_context_get_source_files (current_context))), (_tmp8 == NULL) ? NULL : (_tmp8 = (gee_collection_object_unref (_tmp8), NULL)), _tmp9);
			/*add new namespaces to standard context)*/
			while (gee_iterator_next (_src_it)) {
				ValaSourceFile* src;
				/*add new namespaces to standard context)*/
				src = (ValaSourceFile*) gee_iterator_get (_src_it);
				{
					GeeList* _tmp10;
					GeeIterator* _tmp11;
					GeeIterator* _nr_it;
					_tmp10 = NULL;
					_tmp11 = NULL;
					_nr_it = (_tmp11 = gee_iterable_iterator ((GeeIterable*) (_tmp10 = vala_source_file_get_using_directives (src))), (_tmp10 == NULL) ? NULL : (_tmp10 = (gee_collection_object_unref (_tmp10), NULL)), _tmp11);
					while (gee_iterator_next (_nr_it)) {
						ValaUsingDirective* nr;
						nr = (ValaUsingDirective*) gee_iterator_get (_nr_it);
						{
							gboolean _tmp12;
							_tmp12 = FALSE;
							if (vala_symbol_get_name (vala_using_directive_get_namespace_symbol (nr)) != NULL) {
								_tmp12 = _vala_strcmp0 (vala_symbol_get_name (vala_using_directive_get_namespace_symbol (nr)), "") != 0;
							} else {
								_tmp12 = FALSE;
							}
							if (_tmp12) {
								gboolean _tmp13;
								_tmp13 = vsc_parser_manager_add_package_from_namespace (self, vala_symbol_get_name (vala_using_directive_get_namespace_symbol (nr)), FALSE, &inner_error);
								if (inner_error != NULL) {
									goto __catch4_g_error;
									goto __finally4;
								}
								need_reparse = _tmp13;
							}
						}
						goto __finally4;
						__catch4_g_error:
						{
							GError * err;
							err = inner_error;
							inner_error = NULL;
							{
								g_warning ("vscparsermanager.vala:508: Error adding namespace %s from file %s", vala_symbol_get_name (vala_using_directive_get_namespace_symbol (nr)), vala_source_file_get_filename (src));
								(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
							}
						}
						__finally4:
						if (inner_error != NULL) {
							(nr == NULL) ? NULL : (nr = (vala_code_node_unref (nr), NULL));
							(_nr_it == NULL) ? NULL : (_nr_it = (gee_collection_object_unref (_nr_it), NULL));
							(src == NULL) ? NULL : (src = (vala_source_file_unref (src), NULL));
							(_src_it == NULL) ? NULL : (_src_it = (gee_collection_object_unref (_src_it), NULL));
							(source == NULL) ? NULL : (source = (vala_source_file_unref (source), NULL));
							(current_context == NULL) ? NULL : (current_context = (vala_code_context_unref (current_context), NULL));
							g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
							g_clear_error (&inner_error);
							return;
						}
						(nr == NULL) ? NULL : (nr = (vala_code_node_unref (nr), NULL));
					}
					(_nr_it == NULL) ? NULL : (_nr_it = (gee_collection_object_unref (_nr_it), NULL));
				}
				(src == NULL) ? NULL : (src = (vala_source_file_unref (src), NULL));
			}
			(_src_it == NULL) ? NULL : (_src_it = (gee_collection_object_unref (_src_it), NULL));
		}
		vsc_parser_manager_lock_sec_context (self);
		_tmp15 = NULL;
		_tmp14 = NULL;
		self->priv->_sec_context = (_tmp15 = (_tmp14 = current_context, (_tmp14 == NULL) ? NULL : vala_code_context_ref (_tmp14)), (self->priv->_sec_context == NULL) ? NULL : (self->priv->_sec_context = (vala_code_context_unref (self->priv->_sec_context), NULL)), _tmp15);
		/*primary context reparse?*/
		if (need_reparse) {
			vsc_parser_manager_schedule_parse (self);
		}
		(source == NULL) ? NULL : (source = (vala_source_file_unref (source), NULL));
	}
	vsc_parser_manager_unlock_sec_context (self);
	(current_context == NULL) ? NULL : (current_context = (vala_code_context_unref (current_context), NULL));
}


static void vsc_parser_manager_parse (VscParserManager* self) {
	ValaCodeContext* current_context;
	gboolean _tmp0;
	g_return_if_fail (self != NULL);
	current_context = vala_code_context_new ();
	vsc_parser_manager_lock_pri_context (self);
	_tmp0 = FALSE;
	if (gee_collection_get_size ((GeeCollection*) self->priv->_packages) != 0) {
		_tmp0 = TRUE;
	} else {
		_tmp0 = gee_collection_get_size ((GeeCollection*) self->priv->_sources) != 0;
	}
	if (_tmp0) {
		ValaSourceFile* source;
		ValaSourceFile* _tmp1;
		ValaSourceFile* _tmp2;
		ValaCodeContext* _tmp10;
		ValaCodeContext* _tmp9;
		source = NULL;
		_tmp1 = NULL;
		source = (_tmp1 = vala_source_file_new (current_context, self->priv->glib_file, TRUE, NULL), (source == NULL) ? NULL : (source = (vala_source_file_unref (source), NULL)), _tmp1);
		vala_code_context_add_source_file (current_context, source);
		_tmp2 = NULL;
		source = (_tmp2 = vala_source_file_new (current_context, self->priv->gobject_file, TRUE, NULL), (source == NULL) ? NULL : (source = (vala_source_file_unref (source), NULL)), _tmp2);
		vala_code_context_add_source_file (current_context, source);
		{
			GeeIterator* _item_it;
			_item_it = gee_iterable_iterator ((GeeIterable*) self->priv->_packages);
			while (gee_iterator_next (_item_it)) {
				char* item;
				gboolean _tmp3;
				gboolean _tmp4;
				item = (char*) gee_iterator_get (_item_it);
				_tmp3 = FALSE;
				_tmp4 = FALSE;
				if (item != NULL) {
					_tmp4 = _vala_strcmp0 (item, self->priv->glib_file) != 0;
				} else {
					_tmp4 = FALSE;
				}
				if (_tmp4) {
					_tmp3 = _vala_strcmp0 (item, self->priv->gobject_file) != 0;
				} else {
					_tmp3 = FALSE;
				}
				if (_tmp3) {
					ValaSourceFile* _tmp5;
					_tmp5 = NULL;
					source = (_tmp5 = vala_source_file_new (current_context, item, TRUE, NULL), (source == NULL) ? NULL : (source = (vala_source_file_unref (source), NULL)), _tmp5);
					vala_code_context_add_source_file (current_context, source);
				}
				item = (g_free (item), NULL);
			}
			(_item_it == NULL) ? NULL : (_item_it = (gee_collection_object_unref (_item_it), NULL));
		}
		{
			GeeIterator* _item_it;
			_item_it = gee_iterable_iterator ((GeeIterable*) self->priv->_sources);
			while (gee_iterator_next (_item_it)) {
				char* item;
				ValaSourceFile* _tmp6;
				ValaUsingDirective* _tmp8;
				ValaUnresolvedSymbol* _tmp7;
				item = (char*) gee_iterator_get (_item_it);
				_tmp6 = NULL;
				source = (_tmp6 = vala_source_file_new (current_context, item, FALSE, NULL), (source == NULL) ? NULL : (source = (vala_source_file_unref (source), NULL)), _tmp6);
				_tmp8 = NULL;
				_tmp7 = NULL;
				vala_source_file_add_using_directive (source, _tmp8 = vala_using_directive_new ((ValaSymbol*) (_tmp7 = vala_unresolved_symbol_new (NULL, "GLib", NULL)), NULL));
				(_tmp8 == NULL) ? NULL : (_tmp8 = (vala_code_node_unref (_tmp8), NULL));
				(_tmp7 == NULL) ? NULL : (_tmp7 = (vala_code_node_unref (_tmp7), NULL));
				vala_code_context_add_source_file (current_context, source);
				item = (g_free (item), NULL);
			}
			(_item_it == NULL) ? NULL : (_item_it = (gee_collection_object_unref (_item_it), NULL));
		}
		vsc_parser_manager_unlock_pri_context (self);
		vsc_parser_manager_parse_context (self, current_context);
		vsc_parser_manager_analyze_context (self, current_context);
		vsc_parser_manager_lock_pri_context (self);
		_tmp10 = NULL;
		_tmp9 = NULL;
		self->priv->_pri_context = (_tmp10 = (_tmp9 = current_context, (_tmp9 == NULL) ? NULL : vala_code_context_ref (_tmp9)), (self->priv->_pri_context == NULL) ? NULL : (self->priv->_pri_context = (vala_code_context_unref (self->priv->_pri_context), NULL)), _tmp10);
		(source == NULL) ? NULL : (source = (vala_source_file_unref (source), NULL));
	}
	vsc_parser_manager_unlock_pri_context (self);
	(current_context == NULL) ? NULL : (current_context = (vala_code_context_unref (current_context), NULL));
}


void vsc_parser_manager_parse_context (VscParserManager* self, ValaCodeContext* context) {
	gint glib_major;
	gint glib_minor;
	ValaParser* parser;
	g_return_if_fail (self != NULL);
	g_return_if_fail (context != NULL);
	vala_code_context_set_assert (context, FALSE);
	vala_code_context_set_checking (context, FALSE);
	vala_code_context_set_non_null_experimental (context, FALSE);
	vala_code_context_set_compile_only (context, TRUE);
	glib_major = 2;
	glib_minor = 12;
	vala_code_context_set_target_glib_major (context, glib_major);
	vala_code_context_set_target_glib_minor (context, glib_minor);
	parser = vala_parser_new ();
	vala_parser_parse (parser, context);
	(parser == NULL) ? NULL : (parser = (vala_code_visitor_unref (parser), NULL));
}


static void vsc_parser_manager_analyze_context (VscParserManager* self, ValaCodeContext* context) {
	ValaSymbolResolver* symbol_resolver;
	g_return_if_fail (self != NULL);
	g_return_if_fail (context != NULL);
	symbol_resolver = vala_symbol_resolver_new ();
	vala_symbol_resolver_resolve (symbol_resolver, context);
	(symbol_resolver == NULL) ? NULL : (symbol_resolver = (vala_code_visitor_unref (symbol_resolver), NULL));
}


/*
var semantic = new SemanticAnalyzer ();
semantic.analyze (context);
*/
static char* vsc_parser_manager_find_vala_package_name (VscParserManager* self, const char* namespace, GError** error) {
	GError * inner_error;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (namespace != NULL, NULL);
	inner_error = NULL;
	{
		char** _tmp5;
		gint to_finds_size;
		gint to_finds_length1;
		char** _tmp3;
		char* _tmp2;
		char* _tmp1;
		char* _tmp0;
		char** _tmp4;
		char** to_finds;
		char* _tmp15;
		/*find for: foo.vapi
		or for: foo-1.0.vapi
		or for: foo+1.0.vapi*/
		_tmp5 = NULL;
		_tmp3 = NULL;
		_tmp2 = NULL;
		_tmp1 = NULL;
		_tmp0 = NULL;
		_tmp4 = NULL;
		to_finds = (_tmp5 = (_tmp4 = (_tmp3 = g_new0 (char*, 3 + 1), _tmp3[0] = g_strdup_printf ("%s.", _tmp0 = g_utf8_strdown (namespace, -1)), _tmp3[1] = g_strdup_printf ("%s-", _tmp1 = g_utf8_strdown (namespace, -1)), _tmp3[2] = g_strdup_printf ("%s+", _tmp2 = g_utf8_strdown (namespace, -1)), _tmp3), _tmp2 = (g_free (_tmp2), NULL), _tmp1 = (g_free (_tmp1), NULL), _tmp0 = (g_free (_tmp0), NULL), _tmp4), to_finds_length1 = 3, to_finds_size = to_finds_length1, _tmp5);
		{
			GList* vapidir_collection;
			GList* vapidir_it;
			vapidir_collection = self->priv->_vapidirs;
			for (vapidir_it = vapidir_collection; vapidir_it != NULL; vapidir_it = vapidir_it->next) {
				const char* _tmp14;
				char* vapidir;
				_tmp14 = NULL;
				vapidir = (_tmp14 = (const char*) vapidir_it->data, (_tmp14 == NULL) ? NULL : g_strdup (_tmp14));
				{
					GDir* dir;
					const char* _tmp8;
					char* filename;
					dir = NULL;
					{
						GDir* _tmp6;
						GDir* _tmp7;
						_tmp6 = g_dir_open (vapidir, 0, &inner_error);
						if (inner_error != NULL) {
							if (inner_error->domain == G_FILE_ERROR) {
								goto __catch6_g_file_error;
							}
							goto __finally6;
						}
						_tmp7 = NULL;
						dir = (_tmp7 = _tmp6, (dir == NULL) ? NULL : (dir = (g_dir_close (dir), NULL)), _tmp7);
					}
					goto __finally6;
					__catch6_g_file_error:
					{
						GError * err;
						err = inner_error;
						inner_error = NULL;
						{
							/*do nothing*/
							(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
							vapidir = (g_free (vapidir), NULL);
							(dir == NULL) ? NULL : (dir = (g_dir_close (dir), NULL));
							continue;
						}
					}
					__finally6:
					if (inner_error != NULL) {
						vapidir = (g_free (vapidir), NULL);
						(dir == NULL) ? NULL : (dir = (g_dir_close (dir), NULL));
						to_finds = (_vala_array_free (to_finds, to_finds_length1, (GDestroyNotify) g_free), NULL);
						goto __catch5_g_error;
						goto __finally5;
					}
					_tmp8 = NULL;
					filename = (_tmp8 = g_dir_read_name (dir), (_tmp8 == NULL) ? NULL : g_strdup (_tmp8));
					while (filename != NULL) {
						char* _tmp13;
						const char* _tmp12;
						if (g_str_has_suffix (filename, "vapi")) {
							char* _tmp9;
							_tmp9 = NULL;
							filename = (_tmp9 = g_utf8_strdown (filename, -1), filename = (g_free (filename), NULL), _tmp9);
							{
								char** to_find_collection;
								int to_find_collection_length1;
								int to_find_it;
								to_find_collection = to_finds;
								to_find_collection_length1 = to_finds_length1;
								for (to_find_it = 0; to_find_it < to_finds_length1; to_find_it = to_find_it + 1) {
									const char* _tmp11;
									char* to_find;
									_tmp11 = NULL;
									to_find = (_tmp11 = to_find_collection[to_find_it], (_tmp11 == NULL) ? NULL : g_strdup (_tmp11));
									{
										if (g_str_has_prefix (filename, to_find)) {
											char* _tmp10;
											_tmp10 = NULL;
											return (_tmp10 = filename, to_find = (g_free (to_find), NULL), vapidir = (g_free (vapidir), NULL), (dir == NULL) ? NULL : (dir = (g_dir_close (dir), NULL)), to_finds = (_vala_array_free (to_finds, to_finds_length1, (GDestroyNotify) g_free), NULL), _tmp10);
										}
										to_find = (g_free (to_find), NULL);
									}
								}
							}
						}
						_tmp13 = NULL;
						_tmp12 = NULL;
						filename = (_tmp13 = (_tmp12 = g_dir_read_name (dir), (_tmp12 == NULL) ? NULL : g_strdup (_tmp12)), filename = (g_free (filename), NULL), _tmp13);
					}
					vapidir = (g_free (vapidir), NULL);
					(dir == NULL) ? NULL : (dir = (g_dir_close (dir), NULL));
					filename = (g_free (filename), NULL);
				}
			}
		}
		_tmp15 = NULL;
		return (_tmp15 = NULL, to_finds = (_vala_array_free (to_finds, to_finds_length1, (GDestroyNotify) g_free), NULL), _tmp15);
	}
	goto __finally5;
	__catch5_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			GError* _tmp16;
			_tmp16 = NULL;
			inner_error = (_tmp16 = err, (_tmp16 == NULL) ? ((gpointer) _tmp16) : g_error_copy (_tmp16));
			if (inner_error != NULL) {
				g_propagate_error (error, inner_error);
				(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
				return NULL;
			}
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally5:
	if (inner_error != NULL) {
		g_propagate_error (error, inner_error);
		return NULL;
	}
}


static GeeList* vsc_parser_manager_find_vala_package_filename (VscParserManager* self, const char* package_name, GError** error) {
	GError * inner_error;
	GeeList* results;
	char* found_vapidir;
	char* filename;
	char* path;
	GeeList* _tmp14;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (package_name != NULL, NULL);
	inner_error = NULL;
	results = (GeeList*) gee_array_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_direct_equal);
	found_vapidir = NULL;
	filename = NULL;
	path = NULL;
	if (!g_str_has_suffix (package_name, ".vapi")) {
		char* _tmp0;
		_tmp0 = NULL;
		filename = (_tmp0 = g_strdup_printf ("%s.vapi", package_name), filename = (g_free (filename), NULL), _tmp0);
	} else {
		char* _tmp2;
		const char* _tmp1;
		_tmp2 = NULL;
		_tmp1 = NULL;
		filename = (_tmp2 = (_tmp1 = package_name, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), filename = (g_free (filename), NULL), _tmp2);
	}
	{
		GList* vapidir_collection;
		GList* vapidir_it;
		vapidir_collection = self->priv->_vapidirs;
		for (vapidir_it = vapidir_collection; vapidir_it != NULL; vapidir_it = vapidir_it->next) {
			const char* _tmp6;
			char* vapidir;
			_tmp6 = NULL;
			vapidir = (_tmp6 = (const char*) vapidir_it->data, (_tmp6 == NULL) ? NULL : g_strdup (_tmp6));
			{
				char* _tmp3;
				_tmp3 = NULL;
				path = (_tmp3 = g_strdup_printf ("%s/%s", vapidir, filename), path = (g_free (path), NULL), _tmp3);
				if (g_file_test (path, G_FILE_TEST_EXISTS)) {
					char* _tmp5;
					const char* _tmp4;
					gee_collection_add ((GeeCollection*) results, path);
					_tmp5 = NULL;
					_tmp4 = NULL;
					found_vapidir = (_tmp5 = (_tmp4 = vapidir, (_tmp4 == NULL) ? NULL : g_strdup (_tmp4)), found_vapidir = (g_free (found_vapidir), NULL), _tmp5);
					vapidir = (g_free (vapidir), NULL);
					break;
				}
				vapidir = (g_free (vapidir), NULL);
			}
		}
	}
	if (gee_collection_get_size ((GeeCollection*) results) > 0) {
		char* _tmp7;
		char* _tmp8;
		char* dep_file;
		/*dependency check*/
		_tmp7 = NULL;
		_tmp8 = NULL;
		dep_file = (_tmp8 = g_strdup_printf ("%s/%s.deps", found_vapidir, _tmp7 = string_substring (filename, (glong) 0, string_get_length (filename) - string_get_length (".vapi"))), _tmp7 = (g_free (_tmp7), NULL), _tmp8);
		if (g_file_test (dep_file, G_FILE_TEST_EXISTS)) {
			gsize len;
			char* buffer;
			char* _tmp11;
			gboolean _tmp10;
			char* _tmp9;
			len = 0UL;
			buffer = NULL;
			_tmp11 = NULL;
			_tmp9 = NULL;
			_tmp10 = g_file_get_contents (dep_file, &_tmp9, &len, &inner_error);
			buffer = (_tmp11 = _tmp9, buffer = (g_free (buffer), NULL), _tmp11);
			_tmp10;
			if (inner_error != NULL) {
				g_propagate_error (error, inner_error);
				buffer = (g_free (buffer), NULL);
				dep_file = (g_free (dep_file), NULL);
				(results == NULL) ? NULL : (results = (gee_collection_object_unref (results), NULL));
				found_vapidir = (g_free (found_vapidir), NULL);
				filename = (g_free (filename), NULL);
				path = (g_free (path), NULL);
				return NULL;
			}
			{
				char** _tmp12;
				char** dep_name_collection;
				int dep_name_collection_length1;
				int dep_name_it;
				_tmp12 = NULL;
				dep_name_collection = _tmp12 = g_strsplit (buffer, "\n", 0);
				dep_name_collection_length1 = _vala_array_length (_tmp12);
				for (dep_name_it = 0; dep_name_it < _vala_array_length (_tmp12); dep_name_it = dep_name_it + 1) {
					const char* _tmp13;
					char* dep_name;
					_tmp13 = NULL;
					dep_name = (_tmp13 = dep_name_collection[dep_name_it], (_tmp13 == NULL) ? NULL : g_strdup (_tmp13));
					{
						if (string_get_length (dep_name) > 1) {
							GeeList* deps;
							deps = vsc_parser_manager_find_vala_package_filename (self, dep_name, &inner_error);
							if (inner_error != NULL) {
								g_propagate_error (error, inner_error);
								dep_name = (g_free (dep_name), NULL);
								dep_name_collection = (_vala_array_free (dep_name_collection, dep_name_collection_length1, (GDestroyNotify) g_free), NULL);
								buffer = (g_free (buffer), NULL);
								dep_file = (g_free (dep_file), NULL);
								(results == NULL) ? NULL : (results = (gee_collection_object_unref (results), NULL));
								found_vapidir = (g_free (found_vapidir), NULL);
								filename = (g_free (filename), NULL);
								path = (g_free (path), NULL);
								return NULL;
							}
							{
								GeeIterator* _dep_it;
								_dep_it = gee_iterable_iterator ((GeeIterable*) deps);
								while (gee_iterator_next (_dep_it)) {
									char* dep;
									dep = (char*) gee_iterator_get (_dep_it);
									gee_list_insert (results, 0, dep);
									dep = (g_free (dep), NULL);
								}
								(_dep_it == NULL) ? NULL : (_dep_it = (gee_collection_object_unref (_dep_it), NULL));
							}
							(deps == NULL) ? NULL : (deps = (gee_collection_object_unref (deps), NULL));
						}
						dep_name = (g_free (dep_name), NULL);
					}
				}
				dep_name_collection = (_vala_array_free (dep_name_collection, dep_name_collection_length1, (GDestroyNotify) g_free), NULL);
			}
			buffer = (g_free (buffer), NULL);
		}
		dep_file = (g_free (dep_file), NULL);
	}
	_tmp14 = NULL;
	return (_tmp14 = results, found_vapidir = (g_free (found_vapidir), NULL), filename = (g_free (filename), NULL), path = (g_free (path), NULL), _tmp14);
}


static gboolean vsc_parser_manager_list_contains_string (VscParserManager* self, GeeList* list, const char* value) {
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (list != NULL, FALSE);
	g_return_val_if_fail (value != NULL, FALSE);
	{
		GeeIterator* _current_it;
		_current_it = gee_iterable_iterator ((GeeIterable*) list);
		while (gee_iterator_next (_current_it)) {
			char* current;
			current = (char*) gee_iterator_get (_current_it);
			if (_vala_strcmp0 (current, value) == 0) {
				gboolean _tmp0;
				return (_tmp0 = TRUE, current = (g_free (current), NULL), (_current_it == NULL) ? NULL : (_current_it = (gee_collection_object_unref (_current_it), NULL)), _tmp0);
			}
			current = (g_free (current), NULL);
		}
		(_current_it == NULL) ? NULL : (_current_it = (gee_collection_object_unref (_current_it), NULL));
	}
	return FALSE;
}


VscParserManager* vsc_parser_manager_construct (GType object_type) {
	VscParserManager * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


VscParserManager* vsc_parser_manager_new (void) {
	return vsc_parser_manager_construct (VSC_TYPE_PARSER_MANAGER);
}


ValaCodeContext* vsc_parser_manager_get_pri_context (VscParserManager* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_pri_context;
}


ValaCodeContext* vsc_parser_manager_get_sec_context (VscParserManager* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_sec_context;
}


static GObject * vsc_parser_manager_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	VscParserManagerClass * klass;
	GObjectClass * parent_class;
	VscParserManager * self;
	GError * inner_error;
	klass = VSC_PARSER_MANAGER_CLASS (g_type_class_peek (VSC_TYPE_PARSER_MANAGER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VSC_PARSER_MANAGER (obj);
	inner_error = NULL;
	{
		GMutex* _tmp0;
		GMutex* _tmp1;
		_tmp0 = NULL;
		self->priv->mutex_pri_context = (_tmp0 = g_mutex_new (), (self->priv->mutex_pri_context == NULL) ? NULL : (self->priv->mutex_pri_context = (g_mutex_free (self->priv->mutex_pri_context), NULL)), _tmp0);
		_tmp1 = NULL;
		self->priv->mutex_sec_context = (_tmp1 = g_mutex_new (), (self->priv->mutex_sec_context == NULL) ? NULL : (self->priv->mutex_sec_context = (g_mutex_free (self->priv->mutex_sec_context), NULL)), _tmp1);
		vsc_parser_manager_add_path_to_vapi_search_dir (self, "/usr/share/vala/vapi");
		vsc_parser_manager_add_path_to_vapi_search_dir (self, "/usr/local/share/vala/vapi");
		{
			GeeList* _tmp2;
			char* _tmp4;
			GeeList* _tmp3;
			_tmp2 = vsc_parser_manager_find_vala_package_filename (self, "glib-2.0", &inner_error);
			if (inner_error != NULL) {
				goto __catch7_g_error;
				goto __finally7;
			}
			_tmp4 = NULL;
			_tmp3 = NULL;
			self->priv->glib_file = (_tmp4 = (char*) gee_list_get (_tmp3 = _tmp2, 0), self->priv->glib_file = (g_free (self->priv->glib_file), NULL), _tmp4);
			(_tmp3 == NULL) ? NULL : (_tmp3 = (gee_collection_object_unref (_tmp3), NULL));
		}
		goto __finally7;
		__catch7_g_error:
		{
			GError * err;
			err = inner_error;
			inner_error = NULL;
			{
				g_error ("vscparsermanager.vala:64: Can't find glib vapi file: %s", err->message);
				(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
			}
		}
		__finally7:
		if (inner_error != NULL) {
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
			g_clear_error (&inner_error);
		}
		{
			GeeList* _tmp5;
			char* _tmp7;
			GeeList* _tmp6;
			_tmp5 = vsc_parser_manager_find_vala_package_filename (self, "gobject-2.0", &inner_error);
			if (inner_error != NULL) {
				goto __catch8_g_error;
				goto __finally8;
			}
			_tmp7 = NULL;
			_tmp6 = NULL;
			self->priv->gobject_file = (_tmp7 = (char*) gee_list_get (_tmp6 = _tmp5, 0), self->priv->gobject_file = (g_free (self->priv->gobject_file), NULL), _tmp7);
			(_tmp6 == NULL) ? NULL : (_tmp6 = (gee_collection_object_unref (_tmp6), NULL));
		}
		goto __finally8;
		__catch8_g_error:
		{
			GError * err;
			err = inner_error;
			inner_error = NULL;
			{
				g_error ("vscparsermanager.vala:70: Can't find gobject vapi file: %s", err->message);
				(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
			}
		}
		__finally8:
		if (inner_error != NULL) {
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
			g_clear_error (&inner_error);
		}
	}
	return obj;
}


static void vsc_parser_manager_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	VscParserManager * self;
	gpointer boxed;
	self = VSC_PARSER_MANAGER (object);
	switch (property_id) {
		case VSC_PARSER_MANAGER_PRI_CONTEXT:
		vala_value_set_code_context (value, vsc_parser_manager_get_pri_context (self));
		break;
		case VSC_PARSER_MANAGER_SEC_CONTEXT:
		vala_value_set_code_context (value, vsc_parser_manager_get_sec_context (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vsc_parser_manager_class_init (VscParserManagerClass * klass) {
	vsc_parser_manager_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VscParserManagerPrivate));
	G_OBJECT_CLASS (klass)->get_property = vsc_parser_manager_get_property;
	G_OBJECT_CLASS (klass)->constructor = vsc_parser_manager_constructor;
	G_OBJECT_CLASS (klass)->finalize = vsc_parser_manager_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VSC_PARSER_MANAGER_PRI_CONTEXT, vala_param_spec_code_context ("pri-context", "pri-context", "pri-context", VALA_TYPE_CODE_CONTEXT, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VSC_PARSER_MANAGER_SEC_CONTEXT, vala_param_spec_code_context ("sec-context", "sec-context", "sec-context", VALA_TYPE_CODE_CONTEXT, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_signal_new ("cache_building", VSC_TYPE_PARSER_MANAGER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	g_signal_new ("cache_builded", VSC_TYPE_PARSER_MANAGER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
}


static void vsc_parser_manager_instance_init (VscParserManager * self) {
	self->priv = VSC_PARSER_MANAGER_GET_PRIVATE (self);
	self->priv->_vapidirs = NULL;
	self->priv->_packages = (GeeList*) gee_array_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_direct_equal);
	self->priv->_built_packages = (GeeList*) gee_array_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_direct_equal);
	self->priv->_sources = (GeeList*) gee_array_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_direct_equal);
	self->priv->_source_buffers = (GeeList*) gee_array_list_new (VSC_TYPE_SOURCE_BUFFER, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->priv->need_parse_sec_context = 0;
	self->priv->need_parse_pri_context = 0;
	self->priv->parser_pri_thread = NULL;
	self->priv->parser_sec_thread = NULL;
	self->priv->mutex_pri_context = NULL;
	self->priv->mutex_sec_context = NULL;
	self->priv->parsing_suspended = TRUE;
}


static void vsc_parser_manager_finalize (GObject* obj) {
	VscParserManager * self;
	self = VSC_PARSER_MANAGER (obj);
	{
		GMutex* _tmp8;
		GMutex* _tmp9;
		if (self->priv->parser_pri_thread != NULL) {
			g_thread_join (self->priv->parser_pri_thread);
		}
		if (self->priv->parser_sec_thread != NULL) {
			g_thread_join (self->priv->parser_sec_thread);
		}
		vsc_parser_manager_lock_all_contexts (self);
		gee_collection_clear ((GeeCollection*) self->priv->_packages);
		gee_collection_clear ((GeeCollection*) self->priv->_built_packages);
		gee_collection_clear ((GeeCollection*) self->priv->_sources);
		gee_collection_clear ((GeeCollection*) self->priv->_source_buffers);
		vsc_parser_manager_unlock_all_contexts (self);
		_tmp8 = NULL;
		self->priv->mutex_pri_context = (_tmp8 = NULL, (self->priv->mutex_pri_context == NULL) ? NULL : (self->priv->mutex_pri_context = (g_mutex_free (self->priv->mutex_pri_context), NULL)), _tmp8);
		_tmp9 = NULL;
		self->priv->mutex_sec_context = (_tmp9 = NULL, (self->priv->mutex_sec_context == NULL) ? NULL : (self->priv->mutex_sec_context = (g_mutex_free (self->priv->mutex_sec_context), NULL)), _tmp9);
	}
	self->priv->glib_file = (g_free (self->priv->glib_file), NULL);
	self->priv->gobject_file = (g_free (self->priv->gobject_file), NULL);
	(self->priv->_vapidirs == NULL) ? NULL : (self->priv->_vapidirs = (_g_list_free_g_free (self->priv->_vapidirs), NULL));
	(self->priv->_packages == NULL) ? NULL : (self->priv->_packages = (gee_collection_object_unref (self->priv->_packages), NULL));
	(self->priv->_built_packages == NULL) ? NULL : (self->priv->_built_packages = (gee_collection_object_unref (self->priv->_built_packages), NULL));
	(self->priv->_sources == NULL) ? NULL : (self->priv->_sources = (gee_collection_object_unref (self->priv->_sources), NULL));
	(self->priv->_source_buffers == NULL) ? NULL : (self->priv->_source_buffers = (gee_collection_object_unref (self->priv->_source_buffers), NULL));
	(self->priv->_pri_context == NULL) ? NULL : (self->priv->_pri_context = (vala_code_context_unref (self->priv->_pri_context), NULL));
	(self->priv->_sec_context == NULL) ? NULL : (self->priv->_sec_context = (vala_code_context_unref (self->priv->_sec_context), NULL));
	(self->priv->mutex_pri_context == NULL) ? NULL : (self->priv->mutex_pri_context = (g_mutex_free (self->priv->mutex_pri_context), NULL));
	(self->priv->mutex_sec_context == NULL) ? NULL : (self->priv->mutex_sec_context = (g_mutex_free (self->priv->mutex_sec_context), NULL));
	G_OBJECT_CLASS (vsc_parser_manager_parent_class)->finalize (obj);
}


GType vsc_parser_manager_get_type (void) {
	static GType vsc_parser_manager_type_id = 0;
	if (vsc_parser_manager_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (VscParserManagerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vsc_parser_manager_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VscParserManager), 0, (GInstanceInitFunc) vsc_parser_manager_instance_init, NULL };
		vsc_parser_manager_type_id = g_type_register_static (G_TYPE_OBJECT, "VscParserManager", &g_define_type_info, 0);
	}
	return vsc_parser_manager_type_id;
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




