/*
 *  vscsymbolcompletion.vala - Vala symbol completion library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#include <vscsymbolcompletion.h>
#include <vala/valasourcefile.h>
#include <vala/valacodecontext.h>
#include <gee/iterable.h>
#include <gee/iterator.h>
#include <vala/valaclass.h>
#include <vala/valacodenode.h>
#include <vala/valaobjecttypesymbol.h>
#include <vala/valamethod.h>
#include <vala/valaproperty.h>
#include <vala/valasourcereference.h>
#include <vala/valapropertyaccessor.h>
#include <vala/valaconstructor.h>
#include <vala/valadestructor.h>
#include <vala/valatrystatement.h>
#include <vala/valacatchclause.h>
#include <vala/valaexpressionstatement.h>
#include <vala/valalambdaexpression.h>
#include <vala/valaassignment.h>
#include <vala/valaifstatement.h>
#include <vala/valaforeachstatement.h>
#include <vala/valawhilestatement.h>
#include <vala/valaforstatement.h>
#include <vala/valablock.h>
#include <vala/valareturnstatement.h>
#include <vala/valadeclarationstatement.h>
#include <vala/valamethodcall.h>
#include <vala/valacastexpression.h>
#include <vala/valaclasstype.h>
#include <vala/valasymbol.h>
#include <vala/valascope.h>
#include <gee/arraylist.h>
#include <vala/valacodevisitor.h>
#include <vala/valacreationmethod.h>
#include <vala/valalocalvariable.h>
#include <vala/valaexpression.h>
#include <vala/valastatement.h>
#include <vala/valaobjectcreationexpression.h>
#include <vala/valamemberaccess.h>
#include <vala/valanamespace.h>
#include <vala/valafield.h>
#include <vala/valastruct.h>
#include <vala/valastructvaluetype.h>
#include <vala/valausingdirective.h>
#include <vala/valaformalparameter.h>
#include <vala/valainterface.h>
#include <vala/valaobjecttype.h>
#include <vala/valavaluetype.h>
#include <vala/valaunresolvedsymbol.h>
#include <gee/collection.h>
#include <vscsymbolcompletionitem.h>
#include <vscmethodlistvisitor.h>
#include <vsctypefindvisitor.h>
#include <vsccompletionvisitor.h>




static char* string_replace (const char* self, const char* old, const char* replacement);
static glong string_get_length (const char* self);
static char* string_substring (const char* self, glong offset, glong len);
struct _VscSymbolCompletionPrivate {
	VscParserManager* _parser;
};

#define VSC_SYMBOL_COMPLETION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VSC_TYPE_SYMBOL_COMPLETION, VscSymbolCompletionPrivate))
enum  {
	VSC_SYMBOL_COMPLETION_DUMMY_PROPERTY,
	VSC_SYMBOL_COMPLETION_PARSER
};
static ValaSourceFile* vsc_symbol_completion_find_sourcefile (VscSymbolCompletion* self, ValaCodeContext* context, const char* sourcefile);
static ValaClass* vsc_symbol_completion_get_class (VscSymbolCompletion* self, ValaSourceFile* source, gint line, gint column, GError** error);
static ValaCodeNode* vsc_symbol_completion_find_sub_codenode (VscSymbolCompletion* self, ValaCodeNode* node, gint line, gint column, ValaCodeNode* parent_node);
static ValaCodeNode* vsc_symbol_completion_find_codenode (VscSymbolCompletion* self, ValaSourceFile* source, gint line, gint column, ValaClass** cl, ValaMethod** method, GError** error);
static ValaDataType* vsc_symbol_completion_get_inner_datatype (VscSymbolCompletion* self, ValaDataType* datatype, const char* fields_path, ValaSourceFile* source, GError** error);
static ValaDataType* vsc_symbol_completion_symbol_to_datatype (VscSymbolCompletion* self, ValaSymbol* symbol);
static ValaBlock* vsc_symbol_completion_get_codenode_body (VscSymbolCompletion* self, ValaCodeNode* codenode);
static ValaDataType* vsc_symbol_completion_get_datatype_for_name_in_code_node_with_context (VscSymbolCompletion* self, ValaCodeNode* codenode, ValaCodeContext* context, const char* symbolname, ValaSourceFile* source, gint line, gint column, GError** error);
static ValaDataType* vsc_symbol_completion_get_datatype_for_name_with_context (VscSymbolCompletion* self, ValaCodeContext* context, const char* symbolname, ValaSourceFile* source, gint line, gint column, GError** error);
static ValaDataType* vsc_symbol_completion_datatype_for_localvariable (VscSymbolCompletion* self, ValaCodeContext* context, ValaSourceFile* source, gint line, gint column, ValaLocalVariable* lv);
static gboolean vsc_symbol_completion_node_contains_position (VscSymbolCompletion* self, ValaCodeNode* node, gint line, gint column);
static char* vsc_symbol_completion_normalize_typename (VscSymbolCompletion* self, const char* typename, const char* namespace_name);
static char* vsc_symbol_completion_get_qualified_namespace_name (VscSymbolCompletion* self, ValaSymbol* namespace_symbol);
static void vsc_symbol_completion_get_completion_for_name_in_namespace_with_context (VscSymbolCompletion* self, const char* namespace_name, const char* typename, VscTypeFinderVisitor* finder, VscCompletionVisitor* completion, ValaCodeContext* context);
static gpointer vsc_symbol_completion_parent_class = NULL;
static void vsc_symbol_completion_finalize (GObject* obj);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);



static char* string_replace (const char* self, const char* old, const char* replacement) {
	GError * inner_error;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (old != NULL, NULL);
	g_return_val_if_fail (replacement != NULL, NULL);
	inner_error = NULL;
	{
		char* _tmp0;
		GRegex* _tmp1;
		GRegex* regex;
		char* _tmp2;
		char* _tmp3;
		_tmp0 = NULL;
		_tmp1 = NULL;
		regex = (_tmp1 = g_regex_new (_tmp0 = g_regex_escape_string (old, -1), 0, 0, &inner_error), _tmp0 = (g_free (_tmp0), NULL), _tmp1);
		if (inner_error != NULL) {
			if (inner_error->domain == G_REGEX_ERROR) {
				goto __catch0_g_regex_error;
			}
			goto __finally0;
		}
		_tmp2 = g_regex_replace_literal (regex, self, (glong) (-1), 0, replacement, 0, &inner_error);
		if (inner_error != NULL) {
			(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
			if (inner_error->domain == G_REGEX_ERROR) {
				goto __catch0_g_regex_error;
			}
			goto __finally0;
		}
		_tmp3 = NULL;
		return (_tmp3 = _tmp2, (regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL)), _tmp3);
	}
	goto __finally0;
	__catch0_g_regex_error:
	{
		GError * e;
		e = inner_error;
		inner_error = NULL;
		{
			g_assert_not_reached ();
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally0:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return NULL;
	}
}


static glong string_get_length (const char* self) {
	g_return_val_if_fail (self != NULL, 0L);
	return g_utf8_strlen (self, -1);
}


static char* string_substring (const char* self, glong offset, glong len) {
	glong string_length;
	const char* start;
	g_return_val_if_fail (self != NULL, NULL);
	string_length = g_utf8_strlen (self, -1);
	if (offset < 0) {
		offset = string_length + offset;
		g_return_val_if_fail (offset >= 0, NULL);
	} else {
		g_return_val_if_fail (offset <= string_length, NULL);
	}
	if (len < 0) {
		len = string_length - offset;
	}
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
	start = g_utf8_offset_to_pointer (self, offset);
	return g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
}


GQuark vsc_symbol_completion_error_quark (void) {
	return g_quark_from_static_string ("vsc_symbol_completion_error-quark");
}


void vsc_symbol_completion_cleanup (VscSymbolCompletion* self) {
	VscParserManager* _tmp0;
	g_return_if_fail (self != NULL);
	_tmp0 = NULL;
	self->priv->_parser = (_tmp0 = NULL, (self->priv->_parser == NULL) ? NULL : (self->priv->_parser = (g_object_unref (self->priv->_parser), NULL)), _tmp0);
}


static ValaSourceFile* vsc_symbol_completion_find_sourcefile (VscSymbolCompletion* self, ValaCodeContext* context, const char* sourcefile) {
	char* name;
	GeeList* sources;
	ValaSourceFile* _tmp4;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (context != NULL, NULL);
	g_return_val_if_fail (sourcefile != NULL, NULL);
	name = NULL;
	if (!g_str_has_suffix (sourcefile, ".vala")) {
		char* _tmp0;
		_tmp0 = NULL;
		name = (_tmp0 = g_strdup_printf ("%s.vala", sourcefile), name = (g_free (name), NULL), _tmp0);
	} else {
		char* _tmp2;
		const char* _tmp1;
		_tmp2 = NULL;
		_tmp1 = NULL;
		name = (_tmp2 = (_tmp1 = sourcefile, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), name = (g_free (name), NULL), _tmp2);
	}
	sources = vala_code_context_get_source_files (context);
	if (sources != NULL) {
		{
			GeeIterator* _source_it;
			_source_it = gee_iterable_iterator ((GeeIterable*) sources);
			while (gee_iterator_next (_source_it)) {
				ValaSourceFile* source;
				source = (ValaSourceFile*) gee_iterator_get (_source_it);
				if (_vala_strcmp0 (vala_source_file_get_filename (source), name) == 0) {
					ValaSourceFile* _tmp3;
					_tmp3 = NULL;
					return (_tmp3 = source, (_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL)), name = (g_free (name), NULL), (sources == NULL) ? NULL : (sources = (gee_collection_object_unref (sources), NULL)), _tmp3);
				}
				(source == NULL) ? NULL : (source = (vala_source_file_unref (source), NULL));
			}
			(_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL));
		}
	}
	_tmp4 = NULL;
	return (_tmp4 = NULL, name = (g_free (name), NULL), (sources == NULL) ? NULL : (sources = (gee_collection_object_unref (sources), NULL)), _tmp4);
}


static ValaClass* vsc_symbol_completion_get_class (VscSymbolCompletion* self, ValaSourceFile* source, gint line, gint column, GError** error) {
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (source != NULL, NULL);
	{
		GeeList* _tmp0;
		GeeIterator* _tmp1;
		GeeIterator* _node_it;
		_tmp0 = NULL;
		_tmp1 = NULL;
		_node_it = (_tmp1 = gee_iterable_iterator ((GeeIterable*) (_tmp0 = vala_source_file_get_nodes (source))), (_tmp0 == NULL) ? NULL : (_tmp0 = (gee_collection_object_unref (_tmp0), NULL)), _tmp1);
		while (gee_iterator_next (_node_it)) {
			ValaCodeNode* node;
			node = (ValaCodeNode*) gee_iterator_get (_node_it);
			if (VALA_IS_CLASS (node)) {
				ValaClass* _tmp2;
				ValaClass* cl;
				_tmp2 = NULL;
				cl = (_tmp2 = VALA_CLASS (node), (_tmp2 == NULL) ? NULL : vala_code_node_ref (_tmp2));
				/*TODO: fields, signal subclasses*/
				{
					GeeList* _tmp3;
					GeeIterator* _tmp4;
					GeeIterator* _md_it;
					/*TODO: fields, signal subclasses*/
					_tmp3 = NULL;
					_tmp4 = NULL;
					_md_it = (_tmp4 = gee_iterable_iterator ((GeeIterable*) (_tmp3 = vala_object_type_symbol_get_methods ((ValaObjectTypeSymbol*) cl))), (_tmp3 == NULL) ? NULL : (_tmp3 = (gee_collection_object_unref (_tmp3), NULL)), _tmp4);
					/*TODO: fields, signal subclasses*/
					while (gee_iterator_next (_md_it)) {
						ValaMethod* md;
						ValaCodeNode* _tmp5;
						gboolean _tmp6;
						/*TODO: fields, signal subclasses*/
						md = (ValaMethod*) gee_iterator_get (_md_it);
						_tmp5 = NULL;
						if ((_tmp6 = (_tmp5 = vsc_symbol_completion_find_sub_codenode (self, (ValaCodeNode*) md, line, column, NULL)) != NULL, (_tmp5 == NULL) ? NULL : (_tmp5 = (vala_code_node_unref (_tmp5), NULL)), _tmp6)) {
							ValaClass* _tmp7;
							_tmp7 = NULL;
							return (_tmp7 = cl, (md == NULL) ? NULL : (md = (vala_code_node_unref (md), NULL)), (_md_it == NULL) ? NULL : (_md_it = (gee_collection_object_unref (_md_it), NULL)), (node == NULL) ? NULL : (node = (vala_code_node_unref (node), NULL)), (_node_it == NULL) ? NULL : (_node_it = (gee_collection_object_unref (_node_it), NULL)), _tmp7);
						}
						(md == NULL) ? NULL : (md = (vala_code_node_unref (md), NULL));
					}
					(_md_it == NULL) ? NULL : (_md_it = (gee_collection_object_unref (_md_it), NULL));
				}
				(cl == NULL) ? NULL : (cl = (vala_code_node_unref (cl), NULL));
			}
			(node == NULL) ? NULL : (node = (vala_code_node_unref (node), NULL));
		}
		(_node_it == NULL) ? NULL : (_node_it = (gee_collection_object_unref (_node_it), NULL));
	}
	return NULL;
}


static ValaCodeNode* vsc_symbol_completion_find_sub_codenode (VscSymbolCompletion* self, ValaCodeNode* node, gint line, gint column, ValaCodeNode* parent_node) {
	ValaCodeNode* result;
	gboolean _tmp0;
	gboolean _tmp1;
	ValaCodeNode* _tmp91;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (node != NULL, NULL);
	result = NULL;
	_tmp0 = FALSE;
	_tmp1 = FALSE;
	if (parent_node != NULL) {
		_tmp1 = vala_code_node_get_parent_node (node) == NULL;
	} else {
		_tmp1 = FALSE;
	}
	if (_tmp1) {
		_tmp0 = node != parent_node;
	} else {
		_tmp0 = FALSE;
	}
	/*this is a HACK since vala set the parent_node property only for expression*/
	if (_tmp0) {
		vala_code_node_set_parent_node (node, parent_node);
	}
	if (VALA_IS_PROPERTY (node)) {
		ValaProperty* _tmp2;
		ValaProperty* prop;
		_tmp2 = NULL;
		prop = (_tmp2 = VALA_PROPERTY (node), (_tmp2 == NULL) ? NULL : vala_code_node_ref (_tmp2));
		if (vala_property_get_get_accessor (prop) != NULL) {
			ValaCodeNode* _tmp3;
			_tmp3 = NULL;
			result = (_tmp3 = vsc_symbol_completion_find_sub_codenode (self, (ValaCodeNode*) vala_property_get_get_accessor (prop), line, column, node), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp3);
			if (result != NULL) {
				ValaCodeNode* _tmp4;
				_tmp4 = NULL;
				return (_tmp4 = result, (prop == NULL) ? NULL : (prop = (vala_code_node_unref (prop), NULL)), _tmp4);
			}
		}
		if (vala_property_get_set_accessor (prop) != NULL) {
			ValaCodeNode* _tmp5;
			_tmp5 = NULL;
			result = (_tmp5 = vsc_symbol_completion_find_sub_codenode (self, (ValaCodeNode*) vala_property_get_set_accessor (prop), line, column, node), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp5);
			if (result != NULL) {
				ValaCodeNode* _tmp6;
				_tmp6 = NULL;
				return (_tmp6 = result, (prop == NULL) ? NULL : (prop = (vala_code_node_unref (prop), NULL)), _tmp6);
			}
		}
		if (vsc_symbol_completion_node_contains_position (self, (ValaCodeNode*) prop, line, column)) {
			ValaCodeNode* _tmp7;
			_tmp7 = NULL;
			return (_tmp7 = (ValaCodeNode*) prop, (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp7);
		}
		(prop == NULL) ? NULL : (prop = (vala_code_node_unref (prop), NULL));
	} else {
		if (VALA_IS_PROPERTY_ACCESSOR (node)) {
			ValaPropertyAccessor* _tmp8;
			ValaPropertyAccessor* pa;
			ValaCodeNode* _tmp9;
			_tmp8 = NULL;
			pa = (_tmp8 = VALA_PROPERTY_ACCESSOR (node), (_tmp8 == NULL) ? NULL : vala_code_node_ref (_tmp8));
			_tmp9 = NULL;
			result = (_tmp9 = vsc_symbol_completion_find_sub_codenode (self, (ValaCodeNode*) vala_property_accessor_get_body (pa), line, column, node), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp9);
			if (result != NULL) {
				ValaCodeNode* _tmp10;
				_tmp10 = NULL;
				return (_tmp10 = result, (pa == NULL) ? NULL : (pa = (vala_code_node_unref (pa), NULL)), _tmp10);
			}
			if (vsc_symbol_completion_node_contains_position (self, (ValaCodeNode*) pa, line, column)) {
				ValaCodeNode* _tmp11;
				_tmp11 = NULL;
				return (_tmp11 = (ValaCodeNode*) pa, (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp11);
			}
			(pa == NULL) ? NULL : (pa = (vala_code_node_unref (pa), NULL));
		} else {
			if (VALA_IS_CONSTRUCTOR (node)) {
				ValaConstructor* _tmp12;
				ValaConstructor* cs;
				ValaCodeNode* _tmp13;
				gboolean _tmp15;
				_tmp12 = NULL;
				cs = (_tmp12 = VALA_CONSTRUCTOR (node), (_tmp12 == NULL) ? NULL : vala_code_node_ref (_tmp12));
				_tmp13 = NULL;
				result = (_tmp13 = vsc_symbol_completion_find_sub_codenode (self, (ValaCodeNode*) vala_constructor_get_body (cs), line, column, node), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp13);
				if (result != NULL) {
					ValaCodeNode* _tmp14;
					_tmp14 = NULL;
					return (_tmp14 = result, (cs == NULL) ? NULL : (cs = (vala_code_node_unref (cs), NULL)), _tmp14);
				}
				_tmp15 = FALSE;
				if (vsc_symbol_completion_node_contains_position (self, (ValaCodeNode*) vala_constructor_get_body (cs), line, column)) {
					_tmp15 = TRUE;
				} else {
					_tmp15 = vsc_symbol_completion_node_contains_position (self, (ValaCodeNode*) cs, line, column);
				}
				if (_tmp15) {
					ValaCodeNode* _tmp16;
					_tmp16 = NULL;
					return (_tmp16 = (ValaCodeNode*) cs, (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp16);
				}
				(cs == NULL) ? NULL : (cs = (vala_code_node_unref (cs), NULL));
			} else {
				if (VALA_IS_DESTRUCTOR (node)) {
					ValaDestructor* _tmp17;
					ValaDestructor* ds;
					ValaCodeNode* _tmp18;
					gboolean _tmp20;
					_tmp17 = NULL;
					ds = (_tmp17 = VALA_DESTRUCTOR (node), (_tmp17 == NULL) ? NULL : vala_code_node_ref (_tmp17));
					_tmp18 = NULL;
					result = (_tmp18 = vsc_symbol_completion_find_sub_codenode (self, (ValaCodeNode*) vala_destructor_get_body (ds), line, column, node), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp18);
					if (result != NULL) {
						ValaCodeNode* _tmp19;
						_tmp19 = NULL;
						return (_tmp19 = result, (ds == NULL) ? NULL : (ds = (vala_code_node_unref (ds), NULL)), _tmp19);
					}
					_tmp20 = FALSE;
					if (vsc_symbol_completion_node_contains_position (self, (ValaCodeNode*) vala_destructor_get_body (ds), line, column)) {
						_tmp20 = TRUE;
					} else {
						_tmp20 = vsc_symbol_completion_node_contains_position (self, (ValaCodeNode*) ds, line, column);
					}
					if (_tmp20) {
						ValaCodeNode* _tmp21;
						_tmp21 = NULL;
						return (_tmp21 = (ValaCodeNode*) ds, (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp21);
					}
					(ds == NULL) ? NULL : (ds = (vala_code_node_unref (ds), NULL));
				} else {
					if (VALA_IS_METHOD (node)) {
						ValaMethod* _tmp22;
						ValaMethod* md;
						ValaCodeNode* _tmp23;
						gboolean _tmp25;
						_tmp22 = NULL;
						md = (_tmp22 = VALA_METHOD (node), (_tmp22 == NULL) ? NULL : vala_code_node_ref (_tmp22));
						_tmp23 = NULL;
						result = (_tmp23 = vsc_symbol_completion_find_sub_codenode (self, (ValaCodeNode*) vala_method_get_body (md), line, column, node), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp23);
						if (result != NULL) {
							ValaCodeNode* _tmp24;
							_tmp24 = NULL;
							return (_tmp24 = result, (md == NULL) ? NULL : (md = (vala_code_node_unref (md), NULL)), _tmp24);
						}
						_tmp25 = FALSE;
						if (vsc_symbol_completion_node_contains_position (self, (ValaCodeNode*) vala_method_get_body (md), line, column)) {
							_tmp25 = TRUE;
						} else {
							_tmp25 = vsc_symbol_completion_node_contains_position (self, (ValaCodeNode*) md, line, column);
						}
						if (_tmp25) {
							ValaCodeNode* _tmp26;
							_tmp26 = NULL;
							return (_tmp26 = (ValaCodeNode*) md, (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp26);
						}
						(md == NULL) ? NULL : (md = (vala_code_node_unref (md), NULL));
					} else {
						if (VALA_IS_TRY_STATEMENT (node)) {
							ValaTryStatement* _tmp27;
							ValaTryStatement* tr;
							ValaCodeNode* _tmp28;
							ValaCodeNode* _tmp32;
							_tmp27 = NULL;
							tr = (_tmp27 = VALA_TRY_STATEMENT (node), (_tmp27 == NULL) ? NULL : vala_code_node_ref (_tmp27));
							_tmp28 = NULL;
							result = (_tmp28 = vsc_symbol_completion_find_sub_codenode (self, (ValaCodeNode*) vala_try_statement_get_body (tr), line, column, (ValaCodeNode*) tr), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp28);
							if (result != NULL) {
								ValaCodeNode* _tmp29;
								_tmp29 = NULL;
								return (_tmp29 = result, (tr == NULL) ? NULL : (tr = (vala_code_node_unref (tr), NULL)), _tmp29);
							}
							if (vsc_symbol_completion_node_contains_position (self, (ValaCodeNode*) vala_try_statement_get_body (tr), line, column)) {
								ValaCodeNode* _tmp30;
								ValaCodeNode* _tmp31;
								_tmp30 = NULL;
								_tmp31 = NULL;
								return (_tmp31 = (_tmp30 = (ValaCodeNode*) vala_try_statement_get_body (tr), (_tmp30 == NULL) ? NULL : vala_code_node_ref (_tmp30)), (tr == NULL) ? NULL : (tr = (vala_code_node_unref (tr), NULL)), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp31);
							}
							_tmp32 = NULL;
							result = (_tmp32 = vsc_symbol_completion_find_sub_codenode (self, (ValaCodeNode*) vala_try_statement_get_finally_body (tr), line, column, (ValaCodeNode*) tr), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp32);
							if (result != NULL) {
								ValaCodeNode* _tmp33;
								_tmp33 = NULL;
								return (_tmp33 = result, (tr == NULL) ? NULL : (tr = (vala_code_node_unref (tr), NULL)), _tmp33);
							}
							if (vsc_symbol_completion_node_contains_position (self, (ValaCodeNode*) vala_try_statement_get_finally_body (tr), line, column)) {
								ValaCodeNode* _tmp34;
								ValaCodeNode* _tmp35;
								_tmp34 = NULL;
								_tmp35 = NULL;
								return (_tmp35 = (_tmp34 = (ValaCodeNode*) vala_try_statement_get_finally_body (tr), (_tmp34 == NULL) ? NULL : vala_code_node_ref (_tmp34)), (tr == NULL) ? NULL : (tr = (vala_code_node_unref (tr), NULL)), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp35);
							}
							{
								GeeList* _tmp36;
								GeeIterator* _tmp37;
								GeeIterator* _clause_it;
								_tmp36 = NULL;
								_tmp37 = NULL;
								_clause_it = (_tmp37 = gee_iterable_iterator ((GeeIterable*) (_tmp36 = vala_try_statement_get_catch_clauses (tr))), (_tmp36 == NULL) ? NULL : (_tmp36 = (gee_collection_object_unref (_tmp36), NULL)), _tmp37);
								while (gee_iterator_next (_clause_it)) {
									ValaCatchClause* clause;
									ValaCodeNode* _tmp38;
									clause = (ValaCatchClause*) gee_iterator_get (_clause_it);
									_tmp38 = NULL;
									result = (_tmp38 = vsc_symbol_completion_find_sub_codenode (self, (ValaCodeNode*) clause, line, column, (ValaCodeNode*) tr), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp38);
									if (result != NULL) {
										ValaCodeNode* _tmp39;
										_tmp39 = NULL;
										return (_tmp39 = result, (clause == NULL) ? NULL : (clause = (vala_code_node_unref (clause), NULL)), (_clause_it == NULL) ? NULL : (_clause_it = (gee_collection_object_unref (_clause_it), NULL)), (tr == NULL) ? NULL : (tr = (vala_code_node_unref (tr), NULL)), _tmp39);
									}
									(clause == NULL) ? NULL : (clause = (vala_code_node_unref (clause), NULL));
								}
								(_clause_it == NULL) ? NULL : (_clause_it = (gee_collection_object_unref (_clause_it), NULL));
							}
							if (vsc_symbol_completion_node_contains_position (self, (ValaCodeNode*) tr, line, column)) {
								ValaCodeNode* _tmp40;
								_tmp40 = NULL;
								return (_tmp40 = (ValaCodeNode*) tr, (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp40);
							}
							(tr == NULL) ? NULL : (tr = (vala_code_node_unref (tr), NULL));
						} else {
							if (VALA_IS_CATCH_CLAUSE (node)) {
								ValaCatchClause* _tmp41;
								ValaCatchClause* cc;
								ValaCodeNode* _tmp42;
								gboolean _tmp44;
								_tmp41 = NULL;
								cc = (_tmp41 = VALA_CATCH_CLAUSE (node), (_tmp41 == NULL) ? NULL : vala_code_node_ref (_tmp41));
								_tmp42 = NULL;
								result = (_tmp42 = vsc_symbol_completion_find_sub_codenode (self, (ValaCodeNode*) vala_catch_clause_get_body (cc), line, column, (ValaCodeNode*) cc), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp42);
								if (result != NULL) {
									ValaCodeNode* _tmp43;
									_tmp43 = NULL;
									return (_tmp43 = result, (cc == NULL) ? NULL : (cc = (vala_code_node_unref (cc), NULL)), _tmp43);
								}
								_tmp44 = FALSE;
								if (vsc_symbol_completion_node_contains_position (self, (ValaCodeNode*) vala_catch_clause_get_body (cc), line, column)) {
									_tmp44 = TRUE;
								} else {
									_tmp44 = vsc_symbol_completion_node_contains_position (self, (ValaCodeNode*) cc, line, column);
								}
								if (_tmp44) {
									ValaCodeNode* _tmp45;
									_tmp45 = NULL;
									return (_tmp45 = (ValaCodeNode*) cc, (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp45);
								}
								(cc == NULL) ? NULL : (cc = (vala_code_node_unref (cc), NULL));
							} else {
								if (VALA_IS_EXPRESSION_STATEMENT (node)) {
									ValaCodeNode* _tmp46;
									_tmp46 = NULL;
									result = (_tmp46 = vsc_symbol_completion_find_sub_codenode (self, (ValaCodeNode*) vala_expression_statement_get_expression ((VALA_EXPRESSION_STATEMENT (node))), line, column, node), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp46);
									if (result != NULL) {
										return result;
									}
								} else {
									if (VALA_IS_LAMBDA_EXPRESSION (node)) {
										ValaLambdaExpression* _tmp48;
										ValaLambdaExpression* lambda;
										_tmp48 = NULL;
										lambda = (_tmp48 = VALA_LAMBDA_EXPRESSION (node), (_tmp48 == NULL) ? NULL : vala_code_node_ref (_tmp48));
										if (vsc_symbol_completion_node_contains_position (self, (ValaCodeNode*) vala_lambda_expression_get_statement_body (lambda), line, column)) {
											ValaCodeNode* _tmp49;
											ValaCodeNode* _tmp51;
											_tmp49 = NULL;
											result = (_tmp49 = vsc_symbol_completion_find_sub_codenode (self, (ValaCodeNode*) vala_lambda_expression_get_statement_body (lambda), line, column, node), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp49);
											if (result != NULL) {
												ValaCodeNode* _tmp50;
												_tmp50 = NULL;
												return (_tmp50 = result, (lambda == NULL) ? NULL : (lambda = (vala_code_node_unref (lambda), NULL)), _tmp50);
											}
											_tmp51 = NULL;
											return (_tmp51 = (ValaCodeNode*) lambda, (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp51);
										}
										(lambda == NULL) ? NULL : (lambda = (vala_code_node_unref (lambda), NULL));
									} else {
										if (VALA_IS_ASSIGNMENT (node)) {
											ValaCodeNode* _tmp52;
											_tmp52 = NULL;
											result = (_tmp52 = vsc_symbol_completion_find_sub_codenode (self, (ValaCodeNode*) vala_assignment_get_right ((VALA_ASSIGNMENT (node))), line, column, node), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp52);
											if (result != NULL) {
												return result;
											}
											if (vsc_symbol_completion_node_contains_position (self, (ValaCodeNode*) vala_assignment_get_right ((VALA_ASSIGNMENT (node))), line, column)) {
												ValaCodeNode* _tmp54;
												ValaCodeNode* _tmp55;
												_tmp54 = NULL;
												_tmp55 = NULL;
												return (_tmp55 = (_tmp54 = node, (_tmp54 == NULL) ? NULL : vala_code_node_ref (_tmp54)), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp55);
											}
										} else {
											if (VALA_IS_IF_STATEMENT (node)) {
												ValaIfStatement* _tmp56;
												ValaIfStatement* ifs;
												_tmp56 = NULL;
												ifs = (_tmp56 = VALA_IF_STATEMENT (node), (_tmp56 == NULL) ? NULL : vala_code_node_ref (_tmp56));
												if (vala_if_statement_get_true_statement (ifs) != NULL) {
													ValaCodeNode* _tmp57;
													_tmp57 = NULL;
													result = (_tmp57 = vsc_symbol_completion_find_sub_codenode (self, (ValaCodeNode*) vala_if_statement_get_true_statement (ifs), line, column, node), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp57);
													if (result != NULL) {
														ValaCodeNode* _tmp58;
														_tmp58 = NULL;
														return (_tmp58 = result, (ifs == NULL) ? NULL : (ifs = (vala_code_node_unref (ifs), NULL)), _tmp58);
													}
													if (vsc_symbol_completion_node_contains_position (self, (ValaCodeNode*) vala_if_statement_get_true_statement (ifs), line, column)) {
														ValaCodeNode* _tmp59;
														ValaCodeNode* _tmp60;
														_tmp59 = NULL;
														_tmp60 = NULL;
														return (_tmp60 = (_tmp59 = (ValaCodeNode*) vala_if_statement_get_true_statement (ifs), (_tmp59 == NULL) ? NULL : vala_code_node_ref (_tmp59)), (ifs == NULL) ? NULL : (ifs = (vala_code_node_unref (ifs), NULL)), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp60);
													}
												}
												if (vala_if_statement_get_false_statement (ifs) != NULL) {
													ValaCodeNode* _tmp61;
													_tmp61 = NULL;
													result = (_tmp61 = vsc_symbol_completion_find_sub_codenode (self, (ValaCodeNode*) vala_if_statement_get_false_statement (ifs), line, column, node), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp61);
													if (result != NULL) {
														ValaCodeNode* _tmp62;
														_tmp62 = NULL;
														return (_tmp62 = result, (ifs == NULL) ? NULL : (ifs = (vala_code_node_unref (ifs), NULL)), _tmp62);
													}
													if (vsc_symbol_completion_node_contains_position (self, (ValaCodeNode*) vala_if_statement_get_false_statement (ifs), line, column)) {
														ValaCodeNode* _tmp63;
														ValaCodeNode* _tmp64;
														_tmp63 = NULL;
														_tmp64 = NULL;
														return (_tmp64 = (_tmp63 = (ValaCodeNode*) vala_if_statement_get_false_statement (ifs), (_tmp63 == NULL) ? NULL : vala_code_node_ref (_tmp63)), (ifs == NULL) ? NULL : (ifs = (vala_code_node_unref (ifs), NULL)), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp64);
													}
												}
												if (vsc_symbol_completion_node_contains_position (self, (ValaCodeNode*) ifs, line, column)) {
													ValaCodeNode* _tmp65;
													_tmp65 = NULL;
													return (_tmp65 = (ValaCodeNode*) ifs, (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp65);
												}
												(ifs == NULL) ? NULL : (ifs = (vala_code_node_unref (ifs), NULL));
											} else {
												if (VALA_IS_FOREACH_STATEMENT (node)) {
													ValaForeachStatement* _tmp66;
													ValaForeachStatement* fe;
													gboolean _tmp67;
													_tmp66 = NULL;
													fe = (_tmp66 = VALA_FOREACH_STATEMENT (node), (_tmp66 == NULL) ? NULL : vala_code_node_ref (_tmp66));
													_tmp67 = FALSE;
													if (vsc_symbol_completion_node_contains_position (self, (ValaCodeNode*) vala_foreach_statement_get_body (fe), line, column)) {
														_tmp67 = TRUE;
													} else {
														_tmp67 = vsc_symbol_completion_node_contains_position (self, (ValaCodeNode*) fe, line, column);
													}
													if (_tmp67) {
														ValaCodeNode* _tmp68;
														ValaCodeNode* _tmp70;
														_tmp68 = NULL;
														result = (_tmp68 = vsc_symbol_completion_find_sub_codenode (self, (ValaCodeNode*) vala_foreach_statement_get_body (fe), line, column, node), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp68);
														if (result != NULL) {
															ValaCodeNode* _tmp69;
															_tmp69 = NULL;
															return (_tmp69 = result, (fe == NULL) ? NULL : (fe = (vala_code_node_unref (fe), NULL)), _tmp69);
														}
														_tmp70 = NULL;
														return (_tmp70 = (ValaCodeNode*) fe, (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp70);
													}
													(fe == NULL) ? NULL : (fe = (vala_code_node_unref (fe), NULL));
												} else {
													if (VALA_IS_WHILE_STATEMENT (node)) {
														ValaWhileStatement* _tmp71;
														ValaWhileStatement* ws;
														gboolean _tmp72;
														_tmp71 = NULL;
														ws = (_tmp71 = VALA_WHILE_STATEMENT (node), (_tmp71 == NULL) ? NULL : vala_code_node_ref (_tmp71));
														_tmp72 = FALSE;
														if (vsc_symbol_completion_node_contains_position (self, (ValaCodeNode*) vala_while_statement_get_body (ws), line, column)) {
															_tmp72 = TRUE;
														} else {
															_tmp72 = vsc_symbol_completion_node_contains_position (self, (ValaCodeNode*) ws, line, column);
														}
														if (_tmp72) {
															ValaCodeNode* _tmp73;
															ValaCodeNode* _tmp75;
															_tmp73 = NULL;
															result = (_tmp73 = vsc_symbol_completion_find_sub_codenode (self, (ValaCodeNode*) vala_while_statement_get_body (ws), line, column, node), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp73);
															if (result != NULL) {
																ValaCodeNode* _tmp74;
																_tmp74 = NULL;
																return (_tmp74 = result, (ws == NULL) ? NULL : (ws = (vala_code_node_unref (ws), NULL)), _tmp74);
															}
															_tmp75 = NULL;
															return (_tmp75 = (ValaCodeNode*) ws, (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp75);
														}
														(ws == NULL) ? NULL : (ws = (vala_code_node_unref (ws), NULL));
													} else {
														if (VALA_IS_FOR_STATEMENT (node)) {
															ValaForStatement* _tmp76;
															ValaForStatement* fs;
															gboolean _tmp77;
															_tmp76 = NULL;
															fs = (_tmp76 = VALA_FOR_STATEMENT (node), (_tmp76 == NULL) ? NULL : vala_code_node_ref (_tmp76));
															_tmp77 = FALSE;
															if (vsc_symbol_completion_node_contains_position (self, (ValaCodeNode*) vala_for_statement_get_body (fs), line, column)) {
																_tmp77 = TRUE;
															} else {
																_tmp77 = vsc_symbol_completion_node_contains_position (self, (ValaCodeNode*) fs, line, column);
															}
															if (_tmp77) {
																ValaCodeNode* _tmp78;
																ValaCodeNode* _tmp80;
																_tmp78 = NULL;
																result = (_tmp78 = vsc_symbol_completion_find_sub_codenode (self, (ValaCodeNode*) vala_for_statement_get_body (fs), line, column, node), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp78);
																if (result != NULL) {
																	ValaCodeNode* _tmp79;
																	_tmp79 = NULL;
																	return (_tmp79 = result, (fs == NULL) ? NULL : (fs = (vala_code_node_unref (fs), NULL)), _tmp79);
																}
																_tmp80 = NULL;
																return (_tmp80 = (ValaCodeNode*) fs, (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp80);
															}
															(fs == NULL) ? NULL : (fs = (vala_code_node_unref (fs), NULL));
														} else {
															if (VALA_IS_BLOCK (node)) {
																ValaBlock* _tmp81;
																ValaBlock* block;
																_tmp81 = NULL;
																block = (_tmp81 = VALA_BLOCK (node), (_tmp81 == NULL) ? NULL : vala_code_node_ref (_tmp81));
																/*check first in inner sub-nodes*/
																{
																	GeeList* _tmp82;
																	GeeIterator* _tmp83;
																	GeeIterator* _subnode_it;
																	/*check first in inner sub-nodes*/
																	_tmp82 = NULL;
																	_tmp83 = NULL;
																	_subnode_it = (_tmp83 = gee_iterable_iterator ((GeeIterable*) (_tmp82 = vala_block_get_statements (block))), (_tmp82 == NULL) ? NULL : (_tmp82 = (gee_collection_object_unref (_tmp82), NULL)), _tmp83);
																	/*check first in inner sub-nodes*/
																	while (gee_iterator_next (_subnode_it)) {
																		ValaCodeNode* subnode;
																		ValaCodeNode* _tmp84;
																		/*check first in inner sub-nodes*/
																		subnode = (ValaCodeNode*) ((ValaStatement*) gee_iterator_get (_subnode_it));
																		_tmp84 = NULL;
																		result = (_tmp84 = vsc_symbol_completion_find_sub_codenode (self, subnode, line, column, node), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp84);
																		if (result != NULL) {
																			ValaCodeNode* _tmp85;
																			_tmp85 = NULL;
																			return (_tmp85 = result, (subnode == NULL) ? NULL : (subnode = (vala_code_node_unref (subnode), NULL)), (_subnode_it == NULL) ? NULL : (_subnode_it = (gee_collection_object_unref (_subnode_it), NULL)), (block == NULL) ? NULL : (block = (vala_code_node_unref (block), NULL)), _tmp85);
																		}
																		(subnode == NULL) ? NULL : (subnode = (vala_code_node_unref (subnode), NULL));
																	}
																	(_subnode_it == NULL) ? NULL : (_subnode_it = (gee_collection_object_unref (_subnode_it), NULL));
																}
																(block == NULL) ? NULL : (block = (vala_code_node_unref (block), NULL));
															} else {
																gboolean _tmp86;
																gboolean _tmp87;
																_tmp86 = FALSE;
																_tmp87 = FALSE;
																if (VALA_IS_RETURN_STATEMENT (node)) {
																	_tmp87 = TRUE;
																} else {
																	_tmp87 = VALA_IS_DECLARATION_STATEMENT (node);
																}
																if (_tmp87) {
																	_tmp86 = TRUE;
																} else {
																	_tmp86 = VALA_IS_METHOD_CALL (node);
																}
																if (_tmp86) {
																	ValaCodeNode* _tmp88;
																	_tmp88 = NULL;
																	return (_tmp88 = NULL, (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp88);
																} else {
																	if (VALA_IS_CAST_EXPRESSION (node)) {
																		if (vsc_symbol_completion_node_contains_position (self, node, line, column)) {
																			ValaCodeNode* _tmp89;
																			ValaCodeNode* _tmp90;
																			_tmp89 = NULL;
																			_tmp90 = NULL;
																			return (_tmp90 = (_tmp89 = node, (_tmp89 == NULL) ? NULL : vala_code_node_ref (_tmp89)), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp90);
																		}
																	} else {
																		g_warning ("vscsymbolcompletion.vala:284: incomplete support for %s", g_type_name (G_TYPE_FROM_INSTANCE (node)));
																	}
																}
															}
														}
													}
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}
	_tmp91 = NULL;
	return (_tmp91 = NULL, (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp91);
}


static ValaCodeNode* vsc_symbol_completion_find_codenode (VscSymbolCompletion* self, ValaSourceFile* source, gint line, gint column, ValaClass** cl, ValaMethod** method, GError** error) {
	ValaClass* _tmp0;
	ValaMethod* _tmp1;
	ValaClass* _tmp26;
	ValaMethod* _tmp27;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (source != NULL, NULL);
	*cl = NULL;
	*method = NULL;
	_tmp0 = NULL;
	(*cl) = (_tmp0 = NULL, ((*cl) == NULL) ? NULL : ((*cl) = (vala_code_node_unref ((*cl)), NULL)), _tmp0);
	_tmp1 = NULL;
	(*method) = (_tmp1 = NULL, ((*method) == NULL) ? NULL : ((*method) = (vala_code_node_unref ((*method)), NULL)), _tmp1);
	{
		GeeList* _tmp2;
		GeeIterator* _tmp3;
		GeeIterator* _node_it;
		_tmp2 = NULL;
		_tmp3 = NULL;
		_node_it = (_tmp3 = gee_iterable_iterator ((GeeIterable*) (_tmp2 = vala_source_file_get_nodes (source))), (_tmp2 == NULL) ? NULL : (_tmp2 = (gee_collection_object_unref (_tmp2), NULL)), _tmp3);
		while (gee_iterator_next (_node_it)) {
			ValaCodeNode* node;
			ValaCodeNode* result;
			ValaMethod* _tmp4;
			node = (ValaCodeNode*) gee_iterator_get (_node_it);
			result = NULL;
			_tmp4 = NULL;
			(*method) = (_tmp4 = NULL, ((*method) == NULL) ? NULL : ((*method) = (vala_code_node_unref ((*method)), NULL)), _tmp4);
			if (VALA_IS_METHOD (node)) {
				ValaMethod* _tmp6;
				ValaMethod* _tmp5;
				ValaCodeNode* _tmp7;
				_tmp6 = NULL;
				_tmp5 = NULL;
				(*method) = (_tmp6 = (_tmp5 = VALA_METHOD (node), (_tmp5 == NULL) ? NULL : vala_code_node_ref (_tmp5)), ((*method) == NULL) ? NULL : ((*method) = (vala_code_node_unref ((*method)), NULL)), _tmp6);
				_tmp7 = NULL;
				result = (_tmp7 = vsc_symbol_completion_find_sub_codenode (self, node, line, column, NULL), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp7);
				if (result != NULL) {
					ValaCodeNode* _tmp8;
					_tmp8 = NULL;
					return (_tmp8 = result, (node == NULL) ? NULL : (node = (vala_code_node_unref (node), NULL)), (_node_it == NULL) ? NULL : (_node_it = (gee_collection_object_unref (_node_it), NULL)), _tmp8);
				}
			} else {
				if (VALA_IS_CLASS (node)) {
					ValaClass* _tmp10;
					ValaClass* _tmp9;
					ValaMethod* _tmp21;
					_tmp10 = NULL;
					_tmp9 = NULL;
					(*cl) = (_tmp10 = (_tmp9 = VALA_CLASS (node), (_tmp9 == NULL) ? NULL : vala_code_node_ref (_tmp9)), ((*cl) == NULL) ? NULL : ((*cl) = (vala_code_node_unref ((*cl)), NULL)), _tmp10);
					if (vala_class_get_constructor ((*cl)) != NULL) {
						ValaCodeNode* _tmp11;
						_tmp11 = NULL;
						result = (_tmp11 = vsc_symbol_completion_find_sub_codenode (self, (ValaCodeNode*) vala_class_get_constructor ((*cl)), line, column, (ValaCodeNode*) (*cl)), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp11);
						if (result != NULL) {
							ValaCodeNode* _tmp12;
							_tmp12 = NULL;
							return (_tmp12 = result, (node == NULL) ? NULL : (node = (vala_code_node_unref (node), NULL)), (_node_it == NULL) ? NULL : (_node_it = (gee_collection_object_unref (_node_it), NULL)), _tmp12);
						}
					}
					if (vala_class_get_destructor ((*cl)) != NULL) {
						ValaCodeNode* _tmp13;
						_tmp13 = NULL;
						result = (_tmp13 = vsc_symbol_completion_find_sub_codenode (self, (ValaCodeNode*) vala_class_get_destructor ((*cl)), line, column, (ValaCodeNode*) (*cl)), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp13);
						if (result != NULL) {
							ValaCodeNode* _tmp14;
							_tmp14 = NULL;
							return (_tmp14 = result, (node == NULL) ? NULL : (node = (vala_code_node_unref (node), NULL)), (_node_it == NULL) ? NULL : (_node_it = (gee_collection_object_unref (_node_it), NULL)), _tmp14);
						}
					}
					{
						GeeList* _tmp15;
						GeeIterator* _tmp16;
						GeeIterator* _md_it;
						_tmp15 = NULL;
						_tmp16 = NULL;
						_md_it = (_tmp16 = gee_iterable_iterator ((GeeIterable*) (_tmp15 = vala_object_type_symbol_get_methods ((ValaObjectTypeSymbol*) (*cl)))), (_tmp15 == NULL) ? NULL : (_tmp15 = (gee_collection_object_unref (_tmp15), NULL)), _tmp16);
						while (gee_iterator_next (_md_it)) {
							ValaMethod* md;
							ValaMethod* _tmp18;
							ValaMethod* _tmp17;
							ValaCodeNode* _tmp19;
							md = (ValaMethod*) gee_iterator_get (_md_it);
							_tmp18 = NULL;
							_tmp17 = NULL;
							(*method) = (_tmp18 = (_tmp17 = VALA_METHOD (node), (_tmp17 == NULL) ? NULL : vala_code_node_ref (_tmp17)), ((*method) == NULL) ? NULL : ((*method) = (vala_code_node_unref ((*method)), NULL)), _tmp18);
							_tmp19 = NULL;
							result = (_tmp19 = vsc_symbol_completion_find_sub_codenode (self, (ValaCodeNode*) md, line, column, (ValaCodeNode*) (*cl)), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp19);
							if (result != NULL) {
								ValaCodeNode* _tmp20;
								_tmp20 = NULL;
								return (_tmp20 = result, (md == NULL) ? NULL : (md = (vala_code_node_unref (md), NULL)), (_md_it == NULL) ? NULL : (_md_it = (gee_collection_object_unref (_md_it), NULL)), (node == NULL) ? NULL : (node = (vala_code_node_unref (node), NULL)), (_node_it == NULL) ? NULL : (_node_it = (gee_collection_object_unref (_node_it), NULL)), _tmp20);
							}
							(md == NULL) ? NULL : (md = (vala_code_node_unref (md), NULL));
						}
						(_md_it == NULL) ? NULL : (_md_it = (gee_collection_object_unref (_md_it), NULL));
					}
					_tmp21 = NULL;
					(*method) = (_tmp21 = NULL, ((*method) == NULL) ? NULL : ((*method) = (vala_code_node_unref ((*method)), NULL)), _tmp21);
					{
						GeeList* _tmp22;
						GeeIterator* _tmp23;
						GeeIterator* _prop_it;
						_tmp22 = NULL;
						_tmp23 = NULL;
						_prop_it = (_tmp23 = gee_iterable_iterator ((GeeIterable*) (_tmp22 = vala_object_type_symbol_get_properties ((ValaObjectTypeSymbol*) (*cl)))), (_tmp22 == NULL) ? NULL : (_tmp22 = (gee_collection_object_unref (_tmp22), NULL)), _tmp23);
						while (gee_iterator_next (_prop_it)) {
							ValaProperty* prop;
							ValaCodeNode* _tmp24;
							prop = (ValaProperty*) gee_iterator_get (_prop_it);
							_tmp24 = NULL;
							result = (_tmp24 = vsc_symbol_completion_find_sub_codenode (self, (ValaCodeNode*) prop, line, column, (ValaCodeNode*) (*cl)), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp24);
							if (result != NULL) {
								ValaCodeNode* _tmp25;
								_tmp25 = NULL;
								return (_tmp25 = result, (prop == NULL) ? NULL : (prop = (vala_code_node_unref (prop), NULL)), (_prop_it == NULL) ? NULL : (_prop_it = (gee_collection_object_unref (_prop_it), NULL)), (node == NULL) ? NULL : (node = (vala_code_node_unref (node), NULL)), (_node_it == NULL) ? NULL : (_node_it = (gee_collection_object_unref (_node_it), NULL)), _tmp25);
							}
							(prop == NULL) ? NULL : (prop = (vala_code_node_unref (prop), NULL));
						}
						(_prop_it == NULL) ? NULL : (_prop_it = (gee_collection_object_unref (_prop_it), NULL));
					}
				}
			}
			(node == NULL) ? NULL : (node = (vala_code_node_unref (node), NULL));
			(result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL));
		}
		(_node_it == NULL) ? NULL : (_node_it = (gee_collection_object_unref (_node_it), NULL));
	}
	_tmp26 = NULL;
	(*cl) = (_tmp26 = NULL, ((*cl) == NULL) ? NULL : ((*cl) = (vala_code_node_unref ((*cl)), NULL)), _tmp26);
	_tmp27 = NULL;
	(*method) = (_tmp27 = NULL, ((*method) == NULL) ? NULL : ((*method) = (vala_code_node_unref ((*method)), NULL)), _tmp27);
	return NULL;
}


char* vsc_symbol_completion_get_name_for_datatype (VscSymbolCompletion* self, ValaDataType* dt) {
	char* typename;
	char** _tmp1;
	gint toks_size;
	gint toks_length1;
	char** _tmp0;
	char** toks;
	gint count;
	const char* _tmp2;
	char* _tmp3;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (dt != NULL, NULL);
	typename = vsc_symbol_completion_get_qualified_name_for_datatype (self, dt);
	_tmp1 = NULL;
	_tmp0 = NULL;
	toks = (_tmp1 = _tmp0 = g_strsplit (typename, ".", 0), toks_length1 = _vala_array_length (_tmp0), toks_size = toks_length1, _tmp1);
	count = 0;
	while (toks[count + 1] != NULL) {
		count++;
	}
	_tmp2 = NULL;
	_tmp3 = NULL;
	return (_tmp3 = (_tmp2 = toks[count], (_tmp2 == NULL) ? NULL : g_strdup (_tmp2)), typename = (g_free (typename), NULL), toks = (_vala_array_free (toks, toks_length1, (GDestroyNotify) g_free), NULL), _tmp3);
}


char* vsc_symbol_completion_get_qualified_name_for_datatype (VscSymbolCompletion* self, ValaDataType* dt) {
	char* typename;
	gboolean _tmp3;
	char* _tmp8;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (dt != NULL, NULL);
	typename = NULL;
	if (VALA_IS_CLASS_TYPE (dt)) {
		char* _tmp0;
		_tmp0 = NULL;
		typename = (_tmp0 = vala_symbol_get_full_name ((ValaSymbol*) vala_class_type_get_class_symbol ((VALA_CLASS_TYPE (dt)))), typename = (g_free (typename), NULL), _tmp0);
	} else {
		char* _tmp1;
		_tmp1 = NULL;
		typename = (_tmp1 = vala_data_type_to_qualified_string (dt, NULL), typename = (g_free (typename), NULL), _tmp1);
	}
	if (g_str_has_suffix (typename, "?")) {
		char* _tmp2;
		_tmp2 = NULL;
		typename = (_tmp2 = string_substring (typename, (glong) 0, string_get_length (typename) - 1), typename = (g_free (typename), NULL), _tmp2);
	}
	_tmp3 = FALSE;
	if (strstr (typename, "<") != NULL) {
		_tmp3 = g_str_has_suffix (typename, ">");
	} else {
		_tmp3 = FALSE;
	}
	if (_tmp3) {
		char* _tmp7;
		const char* _tmp6;
		gint _tmp5_length1;
		char** _tmp5;
		char** _tmp4;
		/*generic type definition. delete the generic part from the typename*/
		_tmp7 = NULL;
		_tmp6 = NULL;
		_tmp5 = NULL;
		_tmp4 = NULL;
		typename = (_tmp7 = (_tmp6 = (_tmp5 = _tmp4 = g_strsplit (typename, "<", 2), _tmp5_length1 = _vala_array_length (_tmp4), _tmp5)[0], (_tmp6 == NULL) ? NULL : g_strdup (_tmp6)), typename = (g_free (typename), NULL), _tmp7);
		_tmp5 = (_vala_array_free (_tmp5, _tmp5_length1, (GDestroyNotify) g_free), NULL);
	}
	_tmp8 = NULL;
	typename = (_tmp8 = string_replace (typename, "[]", ""), typename = (g_free (typename), NULL), _tmp8);
	return typename;
}


GeeList* vsc_symbol_completion_get_methods_for_source (VscSymbolCompletion* self, const char* sourcefile) {
	ValaSourceFile* source;
	GeeArrayList* results;
	GeeList* _tmp2;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (sourcefile != NULL, NULL);
	g_warn_if_fail (self->priv->_parser != NULL);
	source = NULL;
	results = gee_array_list_new (VSC_TYPE_SYMBOL_COMPLETION_ITEM, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	if (sourcefile != NULL) {
		ValaSourceFile* _tmp0;
		vsc_parser_manager_lock_all_contexts (self->priv->_parser);
		_tmp0 = NULL;
		source = (_tmp0 = vsc_symbol_completion_find_sourcefile (self, vsc_parser_manager_get_sec_context (self->priv->_parser), sourcefile), (source == NULL) ? NULL : (source = (vala_source_file_unref (source), NULL)), _tmp0);
		if (source == NULL) {
			ValaSourceFile* _tmp1;
			_tmp1 = NULL;
			source = (_tmp1 = vsc_symbol_completion_find_sourcefile (self, vsc_parser_manager_get_pri_context (self->priv->_parser), sourcefile), (source == NULL) ? NULL : (source = (vala_source_file_unref (source), NULL)), _tmp1);
		}
		if (source != NULL) {
			VscMethodList* ml;
			ml = vsc_method_list_new ((GeeList*) results);
			vala_source_file_accept (source, (ValaCodeVisitor*) ml);
			(ml == NULL) ? NULL : (ml = (vala_code_visitor_unref (ml), NULL));
		}
		vsc_parser_manager_unlock_all_contexts (self->priv->_parser);
	}
	_tmp2 = NULL;
	return (_tmp2 = (GeeList*) results, (source == NULL) ? NULL : (source = (vala_source_file_unref (source), NULL)), _tmp2);
}


char* vsc_symbol_completion_get_datatype_name_for_name (VscSymbolCompletion* self, const char* symbolname, const char* sourcefile, gint line, gint column, GError** error) {
	GError * inner_error;
	char* typename;
	GTimer* timer;
	ValaDataType* dt;
	char* _tmp1;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (symbolname != NULL, NULL);
	g_return_val_if_fail (sourcefile != NULL, NULL);
	inner_error = NULL;
	typename = NULL;
	timer = g_timer_new ();
	g_timer_start (timer);
	dt = vsc_symbol_completion_get_datatype_for_name (self, symbolname, sourcefile, line, column, &inner_error);
	if (inner_error != NULL) {
		g_propagate_error (error, inner_error);
		typename = (g_free (typename), NULL);
		(timer == NULL) ? NULL : (timer = (g_timer_destroy (timer), NULL));
		return NULL;
	}
	g_timer_stop (timer);
	if (dt != NULL) {
		char* _tmp0;
		_tmp0 = NULL;
		typename = (_tmp0 = vsc_symbol_completion_get_qualified_name_for_datatype (self, dt), typename = (g_free (typename), NULL), _tmp0);
	}
	_tmp1 = NULL;
	return (_tmp1 = typename, (timer == NULL) ? NULL : (timer = (g_timer_destroy (timer), NULL)), (dt == NULL) ? NULL : (dt = (vala_code_node_unref (dt), NULL)), _tmp1);
}


ValaDataType* vsc_symbol_completion_get_datatype_for_name (VscSymbolCompletion* self, const char* symbolname, const char* sourcefile, gint line, gint column, GError** error) {
	GError * inner_error;
	ValaDataType* result;
	char** _tmp1;
	gint toks_size;
	gint toks_length1;
	char** _tmp0;
	char** toks;
	ValaSourceFile* source;
	ValaSourceFile* _tmp2;
	gboolean _tmp5;
	gboolean _tmp6;
	ValaDataType* _tmp9;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (symbolname != NULL, NULL);
	g_return_val_if_fail (sourcefile != NULL, NULL);
	inner_error = NULL;
	g_warn_if_fail (self->priv->_parser != NULL);
	result = NULL;
	_tmp1 = NULL;
	_tmp0 = NULL;
	toks = (_tmp1 = _tmp0 = g_strsplit (symbolname, ".", 2), toks_length1 = _vala_array_length (_tmp0), toks_size = toks_length1, _tmp1);
	source = NULL;
	vsc_parser_manager_lock_all_contexts (self->priv->_parser);
	_tmp2 = NULL;
	source = (_tmp2 = vsc_symbol_completion_find_sourcefile (self, vsc_parser_manager_get_sec_context (self->priv->_parser), sourcefile), (source == NULL) ? NULL : (source = (vala_source_file_unref (source), NULL)), _tmp2);
	if (source != NULL) {
		ValaDataType* _tmp3;
		ValaDataType* _tmp4;
		_tmp3 = vsc_symbol_completion_get_datatype_for_name_with_context (self, vsc_parser_manager_get_sec_context (self->priv->_parser), toks[0], source, line, column, &inner_error);
		if (inner_error != NULL) {
			g_propagate_error (error, inner_error);
			(result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL));
			toks = (_vala_array_free (toks, toks_length1, (GDestroyNotify) g_free), NULL);
			(source == NULL) ? NULL : (source = (vala_source_file_unref (source), NULL));
			return NULL;
		}
		/*first local*/
		_tmp4 = NULL;
		result = (_tmp4 = _tmp3, (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp4);
	} else {
		g_warning ("vscsymbolcompletion.vala:415: (get_datatype_for_name) no sourcefile found %s", sourcefile);
	}
	_tmp5 = FALSE;
	_tmp6 = FALSE;
	if (result != NULL) {
		_tmp6 = source != NULL;
	} else {
		_tmp6 = FALSE;
	}
	if (_tmp6) {
		_tmp5 = toks[1] != NULL;
	} else {
		_tmp5 = FALSE;
	}
	if (_tmp5) {
		ValaDataType* _tmp7;
		ValaDataType* _tmp8;
		_tmp7 = vsc_symbol_completion_get_inner_datatype (self, result, toks[1], source, &inner_error);
		if (inner_error != NULL) {
			g_propagate_error (error, inner_error);
			(result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL));
			toks = (_vala_array_free (toks, toks_length1, (GDestroyNotify) g_free), NULL);
			(source == NULL) ? NULL : (source = (vala_source_file_unref (source), NULL));
			return NULL;
		}
		_tmp8 = NULL;
		result = (_tmp8 = _tmp7, (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp8);
	}
	vsc_parser_manager_unlock_all_contexts (self->priv->_parser);
	_tmp9 = NULL;
	return (_tmp9 = result, toks = (_vala_array_free (toks, toks_length1, (GDestroyNotify) g_free), NULL), (source == NULL) ? NULL : (source = (vala_source_file_unref (source), NULL)), _tmp9);
}


static ValaDataType* vsc_symbol_completion_get_inner_datatype (VscSymbolCompletion* self, ValaDataType* datatype, const char* fields_path, ValaSourceFile* source, GError** error) {
	GError * inner_error;
	ValaDataType* result;
	VscTypeFinderVisitor* finder;
	char** _tmp1;
	gint toks_size;
	gint toks_length1;
	char** _tmp0;
	char** toks;
	char* _tmp2;
	char* _tmp3;
	char* typename;
	ValaDataType* _tmp10;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (datatype != NULL, NULL);
	g_return_val_if_fail (fields_path != NULL, NULL);
	g_return_val_if_fail (source != NULL, NULL);
	inner_error = NULL;
	result = NULL;
	finder = vsc_typefinder_visitor_new (source, vsc_parser_manager_get_pri_context (self->priv->_parser));
	_tmp1 = NULL;
	_tmp0 = NULL;
	toks = (_tmp1 = _tmp0 = g_strsplit (fields_path, ".", 2), toks_length1 = _vala_array_length (_tmp0), toks_size = toks_length1, _tmp1);
	_tmp2 = NULL;
	_tmp3 = NULL;
	typename = (_tmp3 = g_strdup_printf ("%s.%s", _tmp2 = vsc_symbol_completion_get_name_for_datatype (self, datatype), toks[0]), _tmp2 = (g_free (_tmp2), NULL), _tmp3);
	/* */
	vsc_typefinder_visitor_set_searched_typename (finder, typename);
	if (VALA_IS_OBJECT_TYPE (datatype)) {
		ValaObjectType* _tmp4;
		ValaObjectType* obj;
		_tmp4 = NULL;
		obj = (_tmp4 = VALA_OBJECT_TYPE (datatype), (_tmp4 == NULL) ? NULL : vala_code_node_ref (_tmp4));
		vala_code_node_accept ((ValaCodeNode*) vala_object_type_get_type_symbol (obj), (ValaCodeVisitor*) finder);
		(obj == NULL) ? NULL : (obj = (vala_code_node_unref (obj), NULL));
	} else {
		if (VALA_IS_CLASS_TYPE (datatype)) {
			ValaClassType* _tmp5;
			ValaClassType* cl;
			_tmp5 = NULL;
			cl = (_tmp5 = VALA_CLASS_TYPE (datatype), (_tmp5 == NULL) ? NULL : vala_code_node_ref (_tmp5));
			vala_code_node_accept ((ValaCodeNode*) vala_class_type_get_class_symbol (cl), (ValaCodeVisitor*) finder);
			(cl == NULL) ? NULL : (cl = (vala_code_node_unref (cl), NULL));
		} else {
			if (VALA_IS_VALUE_TYPE (datatype)) {
				ValaValueType* _tmp6;
				ValaValueType* vl;
				_tmp6 = NULL;
				vl = (_tmp6 = VALA_VALUE_TYPE (datatype), (_tmp6 == NULL) ? NULL : vala_code_node_ref (_tmp6));
				vala_code_node_accept ((ValaCodeNode*) vala_value_type_get_type_symbol (vl), (ValaCodeVisitor*) finder);
				(vl == NULL) ? NULL : (vl = (vala_code_node_unref (vl), NULL));
			} else {
				vala_code_node_accept ((ValaCodeNode*) datatype, (ValaCodeVisitor*) finder);
			}
		}
	}
	if (vsc_typefinder_visitor_get_result (finder) != NULL) {
		ValaDataType* _tmp7;
		_tmp7 = NULL;
		result = (_tmp7 = vsc_symbol_completion_symbol_to_datatype (self, vsc_typefinder_visitor_get_result (finder)), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp7);
		if (toks[1] != NULL) {
			ValaDataType* _tmp8;
			ValaDataType* _tmp9;
			_tmp8 = vsc_symbol_completion_get_inner_datatype (self, result, toks[1], source, &inner_error);
			if (inner_error != NULL) {
				g_propagate_error (error, inner_error);
				(result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL));
				(finder == NULL) ? NULL : (finder = (vala_code_visitor_unref (finder), NULL));
				toks = (_vala_array_free (toks, toks_length1, (GDestroyNotify) g_free), NULL);
				typename = (g_free (typename), NULL);
				return NULL;
			}
			_tmp9 = NULL;
			result = (_tmp9 = _tmp8, (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp9);
		}
	}
	_tmp10 = NULL;
	return (_tmp10 = result, (finder == NULL) ? NULL : (finder = (vala_code_visitor_unref (finder), NULL)), toks = (_vala_array_free (toks, toks_length1, (GDestroyNotify) g_free), NULL), typename = (g_free (typename), NULL), _tmp10);
}


static ValaDataType* vsc_symbol_completion_symbol_to_datatype (VscSymbolCompletion* self, ValaSymbol* symbol) {
	ValaDataType* result;
	g_return_val_if_fail (self != NULL, NULL);
	if (symbol == NULL) {
		return NULL;
	}
	result = NULL;
	if (VALA_IS_CLASS (symbol)) {
		ValaDataType* _tmp1;
		_tmp1 = NULL;
		result = (_tmp1 = (ValaDataType*) vala_class_type_new (VALA_CLASS (symbol)), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp1);
	} else {
		if (VALA_IS_FIELD (symbol)) {
			ValaField* _tmp2;
			ValaField* field;
			ValaDataType* _tmp4;
			ValaDataType* _tmp3;
			_tmp2 = NULL;
			field = (_tmp2 = VALA_FIELD (symbol), (_tmp2 == NULL) ? NULL : vala_code_node_ref (_tmp2));
			_tmp4 = NULL;
			_tmp3 = NULL;
			result = (_tmp4 = (_tmp3 = vala_field_get_field_type (field), (_tmp3 == NULL) ? NULL : vala_code_node_ref (_tmp3)), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp4);
			(field == NULL) ? NULL : (field = (vala_code_node_unref (field), NULL));
		} else {
			if (VALA_IS_PROPERTY (symbol)) {
				ValaProperty* _tmp5;
				ValaProperty* prop;
				ValaDataType* _tmp7;
				ValaDataType* _tmp6;
				_tmp5 = NULL;
				prop = (_tmp5 = VALA_PROPERTY (symbol), (_tmp5 == NULL) ? NULL : vala_code_node_ref (_tmp5));
				_tmp7 = NULL;
				_tmp6 = NULL;
				result = (_tmp7 = (_tmp6 = vala_property_get_property_type (prop), (_tmp6 == NULL) ? NULL : vala_code_node_ref (_tmp6)), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp7);
				(prop == NULL) ? NULL : (prop = (vala_code_node_unref (prop), NULL));
			} else {
				if (VALA_IS_STRUCT (symbol)) {
					ValaDataType* _tmp8;
					_tmp8 = NULL;
					result = (_tmp8 = (ValaDataType*) vala_struct_value_type_new (VALA_STRUCT (symbol)), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp8);
				} else {
					if (VALA_IS_METHOD (symbol)) {
						ValaMethod* _tmp9;
						ValaMethod* method;
						ValaDataType* _tmp11;
						ValaDataType* _tmp10;
						_tmp9 = NULL;
						method = (_tmp9 = VALA_METHOD (symbol), (_tmp9 == NULL) ? NULL : vala_code_node_ref (_tmp9));
						_tmp11 = NULL;
						_tmp10 = NULL;
						result = (_tmp11 = (_tmp10 = vala_method_get_return_type (method), (_tmp10 == NULL) ? NULL : vala_code_node_ref (_tmp10)), (result == NULL) ? NULL : (result = (vala_code_node_unref (result), NULL)), _tmp11);
						(method == NULL) ? NULL : (method = (vala_code_node_unref (method), NULL));
					} else {
						g_warning ("vscsymbolcompletion.vala:478: (get_datatype_for_symbol_name): unsupported type %s", g_type_name (G_TYPE_FROM_INSTANCE (symbol)));
					}
				}
			}
		}
	}
	return result;
}


static ValaBlock* vsc_symbol_completion_get_codenode_body (VscSymbolCompletion* self, ValaCodeNode* codenode) {
	ValaBlock* body;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (codenode != NULL, NULL);
	body = NULL;
	if (VALA_IS_METHOD (codenode)) {
		ValaBlock* _tmp1;
		ValaBlock* _tmp0;
		_tmp1 = NULL;
		_tmp0 = NULL;
		body = (_tmp1 = (_tmp0 = vala_method_get_body ((VALA_METHOD (codenode))), (_tmp0 == NULL) ? NULL : vala_code_node_ref (_tmp0)), (body == NULL) ? NULL : (body = (vala_code_node_unref (body), NULL)), _tmp1);
	} else {
		if (VALA_IS_CREATION_METHOD (codenode)) {
			ValaBlock* _tmp3;
			ValaBlock* _tmp2;
			_tmp3 = NULL;
			_tmp2 = NULL;
			body = (_tmp3 = (_tmp2 = vala_constructor_get_body ((VALA_CONSTRUCTOR (codenode))), (_tmp2 == NULL) ? NULL : vala_code_node_ref (_tmp2)), (body == NULL) ? NULL : (body = (vala_code_node_unref (body), NULL)), _tmp3);
		} else {
			if (VALA_IS_CONSTRUCTOR (codenode)) {
				ValaBlock* _tmp5;
				ValaBlock* _tmp4;
				_tmp5 = NULL;
				_tmp4 = NULL;
				body = (_tmp5 = (_tmp4 = vala_constructor_get_body ((VALA_CONSTRUCTOR (codenode))), (_tmp4 == NULL) ? NULL : vala_code_node_ref (_tmp4)), (body == NULL) ? NULL : (body = (vala_code_node_unref (body), NULL)), _tmp5);
			} else {
				if (VALA_IS_DESTRUCTOR (codenode)) {
					ValaBlock* _tmp7;
					ValaBlock* _tmp6;
					_tmp7 = NULL;
					_tmp6 = NULL;
					body = (_tmp7 = (_tmp6 = vala_destructor_get_body ((VALA_DESTRUCTOR (codenode))), (_tmp6 == NULL) ? NULL : vala_code_node_ref (_tmp6)), (body == NULL) ? NULL : (body = (vala_code_node_unref (body), NULL)), _tmp7);
				} else {
					if (VALA_IS_LAMBDA_EXPRESSION (codenode)) {
						ValaBlock* _tmp9;
						ValaBlock* _tmp8;
						_tmp9 = NULL;
						_tmp8 = NULL;
						body = (_tmp9 = (_tmp8 = vala_lambda_expression_get_statement_body ((VALA_LAMBDA_EXPRESSION (codenode))), (_tmp8 == NULL) ? NULL : vala_code_node_ref (_tmp8)), (body == NULL) ? NULL : (body = (vala_code_node_unref (body), NULL)), _tmp9);
					} else {
						if (VALA_IS_FOREACH_STATEMENT (codenode)) {
							ValaBlock* _tmp11;
							ValaBlock* _tmp10;
							_tmp11 = NULL;
							_tmp10 = NULL;
							body = (_tmp11 = (_tmp10 = vala_foreach_statement_get_body ((VALA_FOREACH_STATEMENT (codenode))), (_tmp10 == NULL) ? NULL : vala_code_node_ref (_tmp10)), (body == NULL) ? NULL : (body = (vala_code_node_unref (body), NULL)), _tmp11);
						} else {
							if (VALA_IS_FOR_STATEMENT (codenode)) {
								ValaBlock* _tmp13;
								ValaBlock* _tmp12;
								_tmp13 = NULL;
								_tmp12 = NULL;
								body = (_tmp13 = (_tmp12 = vala_for_statement_get_body ((VALA_FOR_STATEMENT (codenode))), (_tmp12 == NULL) ? NULL : vala_code_node_ref (_tmp12)), (body == NULL) ? NULL : (body = (vala_code_node_unref (body), NULL)), _tmp13);
							} else {
								if (VALA_IS_WHILE_STATEMENT (codenode)) {
									ValaBlock* _tmp15;
									ValaBlock* _tmp14;
									_tmp15 = NULL;
									_tmp14 = NULL;
									body = (_tmp15 = (_tmp14 = vala_while_statement_get_body ((VALA_WHILE_STATEMENT (codenode))), (_tmp14 == NULL) ? NULL : vala_code_node_ref (_tmp14)), (body == NULL) ? NULL : (body = (vala_code_node_unref (body), NULL)), _tmp15);
								} else {
									if (VALA_IS_BLOCK (codenode)) {
										ValaBlock* _tmp17;
										ValaBlock* _tmp16;
										_tmp17 = NULL;
										_tmp16 = NULL;
										body = (_tmp17 = (_tmp16 = VALA_BLOCK (codenode), (_tmp16 == NULL) ? NULL : vala_code_node_ref (_tmp16)), (body == NULL) ? NULL : (body = (vala_code_node_unref (body), NULL)), _tmp17);
									} else {
										if (VALA_IS_CATCH_CLAUSE (codenode)) {
											ValaBlock* _tmp19;
											ValaBlock* _tmp18;
											_tmp19 = NULL;
											_tmp18 = NULL;
											body = (_tmp19 = (_tmp18 = vala_catch_clause_get_body ((VALA_CATCH_CLAUSE (codenode))), (_tmp18 == NULL) ? NULL : vala_code_node_ref (_tmp18)), (body == NULL) ? NULL : (body = (vala_code_node_unref (body), NULL)), _tmp19);
										} else {
											if (VALA_IS_PROPERTY_ACCESSOR (codenode)) {
												ValaBlock* _tmp21;
												ValaBlock* _tmp20;
												_tmp21 = NULL;
												_tmp20 = NULL;
												body = (_tmp21 = (_tmp20 = vala_property_accessor_get_body ((VALA_PROPERTY_ACCESSOR (codenode))), (_tmp20 == NULL) ? NULL : vala_code_node_ref (_tmp20)), (body == NULL) ? NULL : (body = (vala_code_node_unref (body), NULL)), _tmp21);
											} else {
												g_warning ("vscsymbolcompletion.vala:511: (get_codenode_body) unsupported type %s", g_type_name (G_TYPE_FROM_INSTANCE (codenode)));
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}
	return body;
}


static ValaDataType* vsc_symbol_completion_get_datatype_for_name_in_code_node_with_context (VscSymbolCompletion* self, ValaCodeNode* codenode, ValaCodeContext* context, const char* symbolname, ValaSourceFile* source, gint line, gint column, GError** error) {
	GError * inner_error;
	gboolean _tmp0;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (codenode != NULL, NULL);
	g_return_val_if_fail (context != NULL, NULL);
	g_return_val_if_fail (symbolname != NULL, NULL);
	inner_error = NULL;
	_tmp0 = FALSE;
	if (_vala_strcmp0 (symbolname, "this") != 0) {
		_tmp0 = _vala_strcmp0 (symbolname, "base") != 0;
	} else {
		_tmp0 = FALSE;
	}
	if (_tmp0) {
		ValaBlock* body;
		body = vsc_symbol_completion_get_codenode_body (self, codenode);
		if (body != NULL) {
			/*method local vars*/
			{
				GeeList* _tmp1;
				GeeIterator* _tmp2;
				GeeIterator* _lvar_it;
				/*method local vars*/
				_tmp1 = NULL;
				_tmp2 = NULL;
				_lvar_it = (_tmp2 = gee_iterable_iterator ((GeeIterable*) (_tmp1 = vala_block_get_local_variables (body))), (_tmp1 == NULL) ? NULL : (_tmp1 = (gee_collection_object_unref (_tmp1), NULL)), _tmp2);
				/*method local vars*/
				while (gee_iterator_next (_lvar_it)) {
					ValaLocalVariable* lvar;
					/*method local vars*/
					lvar = (ValaLocalVariable*) gee_iterator_get (_lvar_it);
					if (_vala_strcmp0 (vala_symbol_get_name ((ValaSymbol*) lvar), symbolname) == 0) {
						gboolean _tmp3;
						gboolean _tmp4;
						_tmp3 = FALSE;
						_tmp4 = FALSE;
						if (vala_local_variable_get_variable_type (lvar) == NULL) {
							_tmp4 = vala_local_variable_get_initializer (lvar) != NULL;
						} else {
							_tmp4 = FALSE;
						}
						if (_tmp4) {
							_tmp3 = vala_expression_get_value_type (vala_local_variable_get_initializer (lvar)) != NULL;
						} else {
							_tmp3 = FALSE;
						}
						if (_tmp3) {
							ValaDataType* _tmp5;
							ValaDataType* _tmp6;
							_tmp5 = NULL;
							_tmp6 = NULL;
							return (_tmp6 = (_tmp5 = vala_expression_get_value_type (vala_local_variable_get_initializer (lvar)), (_tmp5 == NULL) ? NULL : vala_code_node_ref (_tmp5)), (lvar == NULL) ? NULL : (lvar = (vala_code_node_unref (lvar), NULL)), (_lvar_it == NULL) ? NULL : (_lvar_it = (gee_collection_object_unref (_lvar_it), NULL)), (body == NULL) ? NULL : (body = (vala_code_node_unref (body), NULL)), _tmp6);
						} else {
							ValaDataType* _tmp7;
							ValaDataType* _tmp8;
							_tmp7 = NULL;
							_tmp8 = NULL;
							return (_tmp8 = (_tmp7 = vala_local_variable_get_variable_type (lvar), (_tmp7 == NULL) ? NULL : vala_code_node_ref (_tmp7)), (lvar == NULL) ? NULL : (lvar = (vala_code_node_unref (lvar), NULL)), (_lvar_it == NULL) ? NULL : (_lvar_it = (gee_collection_object_unref (_lvar_it), NULL)), (body == NULL) ? NULL : (body = (vala_code_node_unref (body), NULL)), _tmp8);
						}
					}
					(lvar == NULL) ? NULL : (lvar = (vala_code_node_unref (lvar), NULL));
				}
				(_lvar_it == NULL) ? NULL : (_lvar_it = (gee_collection_object_unref (_lvar_it), NULL));
			}
			{
				GeeList* _tmp9;
				GeeIterator* _tmp10;
				GeeIterator* _st_it;
				_tmp9 = NULL;
				_tmp10 = NULL;
				_st_it = (_tmp10 = gee_iterable_iterator ((GeeIterable*) (_tmp9 = vala_block_get_statements (body))), (_tmp9 == NULL) ? NULL : (_tmp9 = (gee_collection_object_unref (_tmp9), NULL)), _tmp10);
				while (gee_iterator_next (_st_it)) {
					ValaStatement* st;
					st = (ValaStatement*) gee_iterator_get (_st_it);
					if (VALA_IS_DECLARATION_STATEMENT (st)) {
						ValaDeclarationStatement* _tmp11;
						ValaDeclarationStatement* decl;
						_tmp11 = NULL;
						decl = (_tmp11 = VALA_DECLARATION_STATEMENT (st), (_tmp11 == NULL) ? NULL : vala_code_node_ref (_tmp11));
						if (_vala_strcmp0 (vala_symbol_get_name (vala_declaration_statement_get_declaration (decl)), symbolname) == 0) {
							if (VALA_IS_LOCAL_VARIABLE (vala_declaration_statement_get_declaration (decl))) {
								ValaDataType* type;
								type = vsc_symbol_completion_datatype_for_localvariable (self, context, source, line, column, VALA_LOCAL_VARIABLE ((vala_declaration_statement_get_declaration (decl))));
								if (type != NULL) {
									ValaDataType* _tmp12;
									_tmp12 = NULL;
									return (_tmp12 = type, (decl == NULL) ? NULL : (decl = (vala_code_node_unref (decl), NULL)), (st == NULL) ? NULL : (st = (vala_code_node_unref (st), NULL)), (_st_it == NULL) ? NULL : (_st_it = (gee_collection_object_unref (_st_it), NULL)), (body == NULL) ? NULL : (body = (vala_code_node_unref (body), NULL)), _tmp12);
								}
								(type == NULL) ? NULL : (type = (vala_code_node_unref (type), NULL));
							} else {
								g_warning ("vscsymbolcompletion.vala:540: (get_datatype_for_name_with_context) unsupported type %s for %s", g_type_name (G_TYPE_FROM_INSTANCE (codenode)), symbolname);
							}
						}
						(decl == NULL) ? NULL : (decl = (vala_code_node_unref (decl), NULL));
					}
					(st == NULL) ? NULL : (st = (vala_code_node_unref (st), NULL));
				}
				(_st_it == NULL) ? NULL : (_st_it = (gee_collection_object_unref (_st_it), NULL));
			}
			if (VALA_IS_METHOD (codenode)) {
				/*method arguments*/
				{
					GeeList* _tmp13;
					GeeIterator* _tmp14;
					GeeIterator* _par_it;
					/*method arguments*/
					_tmp13 = NULL;
					_tmp14 = NULL;
					_par_it = (_tmp14 = gee_iterable_iterator ((GeeIterable*) (_tmp13 = vala_method_get_parameters ((VALA_METHOD (codenode))))), (_tmp13 == NULL) ? NULL : (_tmp13 = (gee_collection_object_unref (_tmp13), NULL)), _tmp14);
					/*method arguments*/
					while (gee_iterator_next (_par_it)) {
						ValaFormalParameter* par;
						/*method arguments*/
						par = (ValaFormalParameter*) gee_iterator_get (_par_it);
						if (_vala_strcmp0 (vala_symbol_get_name ((ValaSymbol*) par), symbolname) == 0) {
							ValaDataType* _tmp15;
							ValaDataType* _tmp16;
							_tmp15 = NULL;
							_tmp16 = NULL;
							return (_tmp16 = (_tmp15 = vala_formal_parameter_get_parameter_type (par), (_tmp15 == NULL) ? NULL : vala_code_node_ref (_tmp15)), (par == NULL) ? NULL : (par = (vala_code_node_unref (par), NULL)), (_par_it == NULL) ? NULL : (_par_it = (gee_collection_object_unref (_par_it), NULL)), (body == NULL) ? NULL : (body = (vala_code_node_unref (body), NULL)), _tmp16);
						}
						(par == NULL) ? NULL : (par = (vala_code_node_unref (par), NULL));
					}
					(_par_it == NULL) ? NULL : (_par_it = (gee_collection_object_unref (_par_it), NULL));
				}
			} else {
				if (VALA_IS_FOREACH_STATEMENT (codenode)) {
					ValaForeachStatement* _tmp17;
					ValaForeachStatement* fe;
					_tmp17 = NULL;
					fe = (_tmp17 = VALA_FOREACH_STATEMENT (codenode), (_tmp17 == NULL) ? NULL : vala_code_node_ref (_tmp17));
					/*foreach statement iterator*/
					if (_vala_strcmp0 (vala_foreach_statement_get_variable_name (fe), symbolname) == 0) {
						ValaDataType* _tmp18;
						ValaDataType* _tmp19;
						_tmp18 = NULL;
						_tmp19 = NULL;
						return (_tmp19 = (_tmp18 = vala_foreach_statement_get_type_reference (fe), (_tmp18 == NULL) ? NULL : vala_code_node_ref (_tmp18)), (fe == NULL) ? NULL : (fe = (vala_code_node_unref (fe), NULL)), (body == NULL) ? NULL : (body = (vala_code_node_unref (body), NULL)), _tmp19);
					}
					(fe == NULL) ? NULL : (fe = (vala_code_node_unref (fe), NULL));
				}
			}
		}
		(body == NULL) ? NULL : (body = (vala_code_node_unref (body), NULL));
	}
	if (vala_code_node_get_parent_node (codenode) != NULL) {
		ValaDataType* _tmp20;
		_tmp20 = vsc_symbol_completion_get_datatype_for_name_in_code_node_with_context (self, vala_code_node_get_parent_node (codenode), context, symbolname, source, line, column, &inner_error);
		if (inner_error != NULL) {
			g_propagate_error (error, inner_error);
			return NULL;
		}
		return _tmp20;
	} else {
		return NULL;
	}
}


static ValaDataType* vsc_symbol_completion_get_datatype_for_name_with_context (VscSymbolCompletion* self, ValaCodeContext* context, const char* symbolname, ValaSourceFile* source, gint line, gint column, GError** error) {
	GError * inner_error;
	ValaClass* cl;
	ValaMethod* md;
	ValaMethod* _tmp5;
	ValaCodeNode* _tmp4;
	ValaMethod* _tmp3;
	ValaClass* _tmp2;
	ValaCodeNode* _tmp1;
	ValaClass* _tmp0;
	ValaCodeNode* codenode;
	ValaDataType* _tmp27;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (context != NULL, NULL);
	g_return_val_if_fail (symbolname != NULL, NULL);
	inner_error = NULL;
	cl = NULL;
	md = NULL;
	_tmp5 = NULL;
	_tmp4 = NULL;
	_tmp3 = NULL;
	_tmp2 = NULL;
	_tmp1 = NULL;
	_tmp0 = NULL;
	codenode = (_tmp4 = (_tmp1 = vsc_symbol_completion_find_codenode (self, source, line, column, &_tmp0, &_tmp3, &inner_error), cl = (_tmp2 = _tmp0, (cl == NULL) ? NULL : (cl = (vala_code_node_unref (cl), NULL)), _tmp2), _tmp1), md = (_tmp5 = _tmp3, (md == NULL) ? NULL : (md = (vala_code_node_unref (md), NULL)), _tmp5), _tmp4);
	if (inner_error != NULL) {
		g_propagate_error (error, inner_error);
		(cl == NULL) ? NULL : (cl = (vala_code_node_unref (cl), NULL));
		(md == NULL) ? NULL : (md = (vala_code_node_unref (md), NULL));
		return NULL;
	}
	if (codenode != NULL) {
		ValaDataType* type;
		type = vsc_symbol_completion_get_datatype_for_name_in_code_node_with_context (self, codenode, context, symbolname, source, line, column, &inner_error);
		if (inner_error != NULL) {
			g_propagate_error (error, inner_error);
			(cl == NULL) ? NULL : (cl = (vala_code_node_unref (cl), NULL));
			(md == NULL) ? NULL : (md = (vala_code_node_unref (md), NULL));
			(codenode == NULL) ? NULL : (codenode = (vala_code_node_unref (codenode), NULL));
			return NULL;
		}
		if (type != NULL) {
			ValaDataType* _tmp6;
			_tmp6 = NULL;
			return (_tmp6 = type, (cl == NULL) ? NULL : (cl = (vala_code_node_unref (cl), NULL)), (md == NULL) ? NULL : (md = (vala_code_node_unref (md), NULL)), (codenode == NULL) ? NULL : (codenode = (vala_code_node_unref (codenode), NULL)), _tmp6);
		}
		if (cl != NULL) {
			if (_vala_strcmp0 (symbolname, "this") == 0) {
				ValaDataType* _tmp7;
				_tmp7 = NULL;
				return (_tmp7 = (ValaDataType*) vala_class_type_new (cl), (type == NULL) ? NULL : (type = (vala_code_node_unref (type), NULL)), (cl == NULL) ? NULL : (cl = (vala_code_node_unref (cl), NULL)), (md == NULL) ? NULL : (md = (vala_code_node_unref (md), NULL)), (codenode == NULL) ? NULL : (codenode = (vala_code_node_unref (codenode), NULL)), _tmp7);
			} else {
				if (_vala_strcmp0 (symbolname, "base") == 0) {
					if (vala_class_get_base_class (cl) != NULL) {
						ValaDataType* _tmp8;
						_tmp8 = NULL;
						return (_tmp8 = (ValaDataType*) vala_class_type_new (vala_class_get_base_class (cl)), (type == NULL) ? NULL : (type = (vala_code_node_unref (type), NULL)), (cl == NULL) ? NULL : (cl = (vala_code_node_unref (cl), NULL)), (md == NULL) ? NULL : (md = (vala_code_node_unref (md), NULL)), (codenode == NULL) ? NULL : (codenode = (vala_code_node_unref (codenode), NULL)), _tmp8);
					} else {
						ValaDataType* _tmp12;
						{
							GeeList* _tmp9;
							GeeIterator* _tmp10;
							GeeIterator* _item_it;
							_tmp9 = NULL;
							_tmp10 = NULL;
							_item_it = (_tmp10 = gee_iterable_iterator ((GeeIterable*) (_tmp9 = vala_class_get_base_types (cl))), (_tmp9 == NULL) ? NULL : (_tmp9 = (gee_collection_object_unref (_tmp9), NULL)), _tmp10);
							while (gee_iterator_next (_item_it)) {
								ValaDataType* item;
								item = (ValaDataType*) gee_iterator_get (_item_it);
								if (!(VALA_IS_INTERFACE (item))) {
									ValaDataType* _tmp11;
									/*this is a HACK!*/
									_tmp11 = NULL;
									return (_tmp11 = item, (_item_it == NULL) ? NULL : (_item_it = (gee_collection_object_unref (_item_it), NULL)), (type == NULL) ? NULL : (type = (vala_code_node_unref (type), NULL)), (cl == NULL) ? NULL : (cl = (vala_code_node_unref (cl), NULL)), (md == NULL) ? NULL : (md = (vala_code_node_unref (md), NULL)), (codenode == NULL) ? NULL : (codenode = (vala_code_node_unref (codenode), NULL)), _tmp11);
								}
								(item == NULL) ? NULL : (item = (vala_code_node_unref (item), NULL));
							}
							(_item_it == NULL) ? NULL : (_item_it = (gee_collection_object_unref (_item_it), NULL));
						}
						_tmp12 = NULL;
						return (_tmp12 = NULL, (type == NULL) ? NULL : (type = (vala_code_node_unref (type), NULL)), (cl == NULL) ? NULL : (cl = (vala_code_node_unref (cl), NULL)), (md == NULL) ? NULL : (md = (vala_code_node_unref (md), NULL)), (codenode == NULL) ? NULL : (codenode = (vala_code_node_unref (codenode), NULL)), _tmp12);
					}
				}
			}
			/*field class*/
			{
				GeeList* _tmp13;
				GeeIterator* _tmp14;
				GeeIterator* _field_it;
				/*field class*/
				_tmp13 = NULL;
				_tmp14 = NULL;
				_field_it = (_tmp14 = gee_iterable_iterator ((GeeIterable*) (_tmp13 = vala_class_get_fields (cl))), (_tmp13 == NULL) ? NULL : (_tmp13 = (gee_collection_object_unref (_tmp13), NULL)), _tmp14);
				/*field class*/
				while (gee_iterator_next (_field_it)) {
					ValaField* field;
					/*field class*/
					field = (ValaField*) gee_iterator_get (_field_it);
					if (_vala_strcmp0 (vala_symbol_get_name ((ValaSymbol*) field), symbolname) == 0) {
						ValaDataType* _tmp15;
						ValaDataType* _tmp16;
						_tmp15 = NULL;
						_tmp16 = NULL;
						return (_tmp16 = (_tmp15 = vala_field_get_field_type (field), (_tmp15 == NULL) ? NULL : vala_code_node_ref (_tmp15)), (field == NULL) ? NULL : (field = (vala_code_node_unref (field), NULL)), (_field_it == NULL) ? NULL : (_field_it = (gee_collection_object_unref (_field_it), NULL)), (type == NULL) ? NULL : (type = (vala_code_node_unref (type), NULL)), (cl == NULL) ? NULL : (cl = (vala_code_node_unref (cl), NULL)), (md == NULL) ? NULL : (md = (vala_code_node_unref (md), NULL)), (codenode == NULL) ? NULL : (codenode = (vala_code_node_unref (codenode), NULL)), _tmp16);
					}
					(field == NULL) ? NULL : (field = (vala_code_node_unref (field), NULL));
				}
				(_field_it == NULL) ? NULL : (_field_it = (gee_collection_object_unref (_field_it), NULL));
			}
			/*properties*/
			{
				GeeList* _tmp17;
				GeeIterator* _tmp18;
				GeeIterator* _prop_it;
				/*properties*/
				_tmp17 = NULL;
				_tmp18 = NULL;
				_prop_it = (_tmp18 = gee_iterable_iterator ((GeeIterable*) (_tmp17 = vala_object_type_symbol_get_properties ((ValaObjectTypeSymbol*) cl))), (_tmp17 == NULL) ? NULL : (_tmp17 = (gee_collection_object_unref (_tmp17), NULL)), _tmp18);
				/*properties*/
				while (gee_iterator_next (_prop_it)) {
					ValaProperty* prop;
					/*properties*/
					prop = (ValaProperty*) gee_iterator_get (_prop_it);
					if (_vala_strcmp0 (vala_symbol_get_name ((ValaSymbol*) prop), symbolname) == 0) {
						ValaDataType* _tmp19;
						ValaDataType* _tmp20;
						_tmp19 = NULL;
						_tmp20 = NULL;
						return (_tmp20 = (_tmp19 = vala_property_get_property_type (prop), (_tmp19 == NULL) ? NULL : vala_code_node_ref (_tmp19)), (prop == NULL) ? NULL : (prop = (vala_code_node_unref (prop), NULL)), (_prop_it == NULL) ? NULL : (_prop_it = (gee_collection_object_unref (_prop_it), NULL)), (type == NULL) ? NULL : (type = (vala_code_node_unref (type), NULL)), (cl == NULL) ? NULL : (cl = (vala_code_node_unref (cl), NULL)), (md == NULL) ? NULL : (md = (vala_code_node_unref (md), NULL)), (codenode == NULL) ? NULL : (codenode = (vala_code_node_unref (codenode), NULL)), _tmp20);
					}
					(prop == NULL) ? NULL : (prop = (vala_code_node_unref (prop), NULL));
				}
				(_prop_it == NULL) ? NULL : (_prop_it = (gee_collection_object_unref (_prop_it), NULL));
			}
			/*methods*/
			{
				GeeList* _tmp21;
				GeeIterator* _tmp22;
				GeeIterator* _clmt_it;
				/*methods*/
				_tmp21 = NULL;
				_tmp22 = NULL;
				_clmt_it = (_tmp22 = gee_iterable_iterator ((GeeIterable*) (_tmp21 = vala_object_type_symbol_get_methods ((ValaObjectTypeSymbol*) cl))), (_tmp21 == NULL) ? NULL : (_tmp21 = (gee_collection_object_unref (_tmp21), NULL)), _tmp22);
				/*methods*/
				while (gee_iterator_next (_clmt_it)) {
					ValaMethod* clmt;
					/*methods*/
					clmt = (ValaMethod*) gee_iterator_get (_clmt_it);
					if (_vala_strcmp0 (vala_symbol_get_name ((ValaSymbol*) clmt), symbolname) == 0) {
						ValaDataType* _tmp23;
						ValaDataType* _tmp24;
						_tmp23 = NULL;
						_tmp24 = NULL;
						return (_tmp24 = (_tmp23 = vala_method_get_return_type (clmt), (_tmp23 == NULL) ? NULL : vala_code_node_ref (_tmp23)), (clmt == NULL) ? NULL : (clmt = (vala_code_node_unref (clmt), NULL)), (_clmt_it == NULL) ? NULL : (_clmt_it = (gee_collection_object_unref (_clmt_it), NULL)), (type == NULL) ? NULL : (type = (vala_code_node_unref (type), NULL)), (cl == NULL) ? NULL : (cl = (vala_code_node_unref (cl), NULL)), (md == NULL) ? NULL : (md = (vala_code_node_unref (md), NULL)), (codenode == NULL) ? NULL : (codenode = (vala_code_node_unref (codenode), NULL)), _tmp24);
					}
					(clmt == NULL) ? NULL : (clmt = (vala_code_node_unref (clmt), NULL));
				}
				(_clmt_it == NULL) ? NULL : (_clmt_it = (gee_collection_object_unref (_clmt_it), NULL));
			}
		}
		(type == NULL) ? NULL : (type = (vala_code_node_unref (type), NULL));
	} else {
		ValaClass* _tmp25;
		ValaClass* _tmp26;
		_tmp25 = vsc_symbol_completion_get_class (self, source, line, column, &inner_error);
		if (inner_error != NULL) {
			g_propagate_error (error, inner_error);
			(cl == NULL) ? NULL : (cl = (vala_code_node_unref (cl), NULL));
			(md == NULL) ? NULL : (md = (vala_code_node_unref (md), NULL));
			(codenode == NULL) ? NULL : (codenode = (vala_code_node_unref (codenode), NULL));
			return NULL;
		}
		_tmp26 = NULL;
		cl = (_tmp26 = _tmp25, (cl == NULL) ? NULL : (cl = (vala_code_node_unref (cl), NULL)), _tmp26);
	}
	_tmp27 = NULL;
	return (_tmp27 = NULL, (cl == NULL) ? NULL : (cl = (vala_code_node_unref (cl), NULL)), (md == NULL) ? NULL : (md = (vala_code_node_unref (md), NULL)), (codenode == NULL) ? NULL : (codenode = (vala_code_node_unref (codenode), NULL)), _tmp27);
}


static ValaDataType* vsc_symbol_completion_datatype_for_localvariable (VscSymbolCompletion* self, ValaCodeContext* context, ValaSourceFile* source, gint line, gint column, ValaLocalVariable* lv) {
	ValaDataType* vt;
	gboolean _tmp0;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (context != NULL, NULL);
	g_return_val_if_fail (lv != NULL, NULL);
	g_warn_if_fail (self->priv->_parser != NULL);
	vt = NULL;
	_tmp0 = FALSE;
	if (vala_local_variable_get_variable_type (lv) == NULL) {
		_tmp0 = vala_local_variable_get_initializer (lv) != NULL;
	} else {
		_tmp0 = FALSE;
	}
	if (_tmp0) {
		ValaDataType* _tmp2;
		ValaDataType* _tmp1;
		_tmp2 = NULL;
		_tmp1 = NULL;
		vt = (_tmp2 = (_tmp1 = vala_expression_get_value_type (vala_local_variable_get_initializer (lv)), (_tmp1 == NULL) ? NULL : vala_code_node_ref (_tmp1)), (vt == NULL) ? NULL : (vt = (vala_code_node_unref (vt), NULL)), _tmp2);
		if (vt == NULL) {
			ValaExpression* _tmp3;
			ValaExpression* initializer;
			char* class_name;
			char* member_name;
			_tmp3 = NULL;
			initializer = (_tmp3 = vala_local_variable_get_initializer (lv), (_tmp3 == NULL) ? NULL : vala_code_node_ref (_tmp3));
			class_name = NULL;
			member_name = NULL;
			if (VALA_IS_OBJECT_CREATION_EXPRESSION (initializer)) {
				ValaObjectCreationExpression* _tmp4;
				ValaObjectCreationExpression* oce;
				_tmp4 = NULL;
				oce = (_tmp4 = ((VALA_OBJECT_CREATION_EXPRESSION ((vala_local_variable_get_initializer (lv))))), (_tmp4 == NULL) ? NULL : vala_code_node_ref (_tmp4));
				if (VALA_IS_MEMBER_ACCESS (vala_object_creation_expression_get_member_name (oce))) {
					ValaExpression* _tmp6;
					ValaExpression* _tmp5;
					_tmp6 = NULL;
					_tmp5 = NULL;
					initializer = (_tmp6 = (_tmp5 = (ValaExpression*) vala_object_creation_expression_get_member_name (oce), (_tmp5 == NULL) ? NULL : vala_code_node_ref (_tmp5)), (initializer == NULL) ? NULL : (initializer = (vala_code_node_unref (initializer), NULL)), _tmp6);
				}
				(oce == NULL) ? NULL : (oce = (vala_code_node_unref (oce), NULL));
			} else {
				if (VALA_IS_METHOD_CALL (initializer)) {
					ValaMethodCall* _tmp7;
					ValaMethodCall* invoc;
					ValaExpression* _tmp9;
					ValaExpression* _tmp8;
					_tmp7 = NULL;
					invoc = (_tmp7 = VALA_METHOD_CALL (initializer), (_tmp7 == NULL) ? NULL : vala_code_node_ref (_tmp7));
					_tmp9 = NULL;
					_tmp8 = NULL;
					initializer = (_tmp9 = (_tmp8 = vala_method_call_get_call (invoc), (_tmp8 == NULL) ? NULL : vala_code_node_ref (_tmp8)), (initializer == NULL) ? NULL : (initializer = (vala_code_node_unref (initializer), NULL)), _tmp9);
					(invoc == NULL) ? NULL : (invoc = (vala_code_node_unref (invoc), NULL));
				}
			}
			if (VALA_IS_MEMBER_ACCESS (initializer)) {
				ValaMemberAccess* _tmp10;
				ValaMemberAccess* ma;
				char* _tmp13;
				const char* _tmp12;
				_tmp10 = NULL;
				ma = (_tmp10 = VALA_MEMBER_ACCESS (initializer), (_tmp10 == NULL) ? NULL : vala_code_node_ref (_tmp10));
				if (vala_member_access_get_inner (ma) != NULL) {
					char* _tmp11;
					_tmp11 = NULL;
					class_name = (_tmp11 = vala_code_node_to_string ((ValaCodeNode*) vala_member_access_get_inner (ma)), class_name = (g_free (class_name), NULL), _tmp11);
				}
				_tmp13 = NULL;
				_tmp12 = NULL;
				member_name = (_tmp13 = (_tmp12 = vala_member_access_get_member_name (ma), (_tmp12 == NULL) ? NULL : g_strdup (_tmp12)), member_name = (g_free (member_name), NULL), _tmp13);
				(ma == NULL) ? NULL : (ma = (vala_code_node_unref (ma), NULL));
			} else {
				if (VALA_IS_CAST_EXPRESSION (initializer)) {
					ValaCastExpression* _tmp14;
					ValaCastExpression* ce;
					ValaDataType* _tmp16;
					ValaDataType* _tmp15;
					_tmp14 = NULL;
					ce = (_tmp14 = VALA_CAST_EXPRESSION (initializer), (_tmp14 == NULL) ? NULL : vala_code_node_ref (_tmp14));
					_tmp16 = NULL;
					_tmp15 = NULL;
					vt = (_tmp16 = (_tmp15 = vala_cast_expression_get_type_reference (ce), (_tmp15 == NULL) ? NULL : vala_code_node_ref (_tmp15)), (vt == NULL) ? NULL : (vt = (vala_code_node_unref (vt), NULL)), _tmp16);
					(ce == NULL) ? NULL : (ce = (vala_code_node_unref (ce), NULL));
				} else {
					g_warning ("vscsymbolcompletion.vala:657: (datatype_for_localvariable) initializer of type '%s' is not yet supported", g_type_name (G_TYPE_FROM_INSTANCE (initializer)));
				}
			}
			if (vt == NULL) {
				char* name;
				gboolean _tmp17;
				VscTypeFinderVisitor* finder;
				ValaDataType* _tmp23;
				name = NULL;
				_tmp17 = FALSE;
				if (member_name != NULL) {
					_tmp17 = class_name != NULL;
				} else {
					_tmp17 = FALSE;
				}
				if (_tmp17) {
					char* _tmp18;
					_tmp18 = NULL;
					name = (_tmp18 = g_strdup_printf ("%s.%s", class_name, member_name), name = (g_free (name), NULL), _tmp18);
				} else {
					if (class_name != NULL) {
						char* _tmp20;
						const char* _tmp19;
						_tmp20 = NULL;
						_tmp19 = NULL;
						name = (_tmp20 = (_tmp19 = class_name, (_tmp19 == NULL) ? NULL : g_strdup (_tmp19)), name = (g_free (name), NULL), _tmp20);
					} else {
						char* _tmp22;
						const char* _tmp21;
						_tmp22 = NULL;
						_tmp21 = NULL;
						name = (_tmp22 = (_tmp21 = member_name, (_tmp21 == NULL) ? NULL : g_strdup (_tmp21)), name = (g_free (name), NULL), _tmp22);
					}
				}
				finder = vsc_typefinder_visitor_new (NULL, NULL);
				vsc_typefinder_visitor_set_searched_typename (finder, name);
				vala_code_visitor_visit_namespace ((ValaCodeVisitor*) finder, vala_code_context_get_root (vsc_parser_manager_get_pri_context (self->priv->_parser)));
				_tmp23 = NULL;
				vt = (_tmp23 = vsc_symbol_completion_symbol_to_datatype (self, vsc_typefinder_visitor_get_result (finder)), (vt == NULL) ? NULL : (vt = (vala_code_node_unref (vt), NULL)), _tmp23);
				if (vt == NULL) {
					{
						GeeList* _tmp24;
						GeeIterator* _tmp25;
						GeeIterator* _item_it;
						_tmp24 = NULL;
						_tmp25 = NULL;
						_item_it = (_tmp25 = gee_iterable_iterator ((GeeIterable*) (_tmp24 = vala_source_file_get_using_directives (source))), (_tmp24 == NULL) ? NULL : (_tmp24 = (gee_collection_object_unref (_tmp24), NULL)), _tmp25);
						while (gee_iterator_next (_item_it)) {
							ValaUsingDirective* item;
							char* using_name;
							item = (ValaUsingDirective*) gee_iterator_get (_item_it);
							using_name = g_strdup_printf ("%s.%s", vala_symbol_get_name (vala_using_directive_get_namespace_symbol (item)), name);
							vsc_typefinder_visitor_set_searched_typename (finder, using_name);
							vala_code_visitor_visit_namespace ((ValaCodeVisitor*) finder, vala_code_context_get_root (vsc_parser_manager_get_pri_context (self->priv->_parser)));
							if (vsc_typefinder_visitor_get_result (finder) != NULL) {
								ValaDataType* _tmp26;
								_tmp26 = NULL;
								vt = (_tmp26 = vsc_symbol_completion_symbol_to_datatype (self, vsc_typefinder_visitor_get_result (finder)), (vt == NULL) ? NULL : (vt = (vala_code_node_unref (vt), NULL)), _tmp26);
								(item == NULL) ? NULL : (item = (vala_code_node_unref (item), NULL));
								using_name = (g_free (using_name), NULL);
								break;
							}
							(item == NULL) ? NULL : (item = (vala_code_node_unref (item), NULL));
							using_name = (g_free (using_name), NULL);
						}
						(_item_it == NULL) ? NULL : (_item_it = (gee_collection_object_unref (_item_it), NULL));
					}
					if (vt == NULL) {
						VscTypeFinderVisitor* _tmp27;
						ValaDataType* _tmp28;
						_tmp27 = NULL;
						finder = (_tmp27 = vsc_typefinder_visitor_new (source, vsc_parser_manager_get_pri_context (self->priv->_parser)), (finder == NULL) ? NULL : (finder = (vala_code_visitor_unref (finder), NULL)), _tmp27);
						vsc_typefinder_visitor_set_searched_typename (finder, name);
						vala_code_visitor_visit_namespace ((ValaCodeVisitor*) finder, vala_code_context_get_root (vsc_parser_manager_get_sec_context (self->priv->_parser)));
						_tmp28 = NULL;
						vt = (_tmp28 = vsc_symbol_completion_symbol_to_datatype (self, vsc_typefinder_visitor_get_result (finder)), (vt == NULL) ? NULL : (vt = (vala_code_node_unref (vt), NULL)), _tmp28);
					}
				}
				name = (g_free (name), NULL);
				(finder == NULL) ? NULL : (finder = (vala_code_visitor_unref (finder), NULL));
			}
			(initializer == NULL) ? NULL : (initializer = (vala_code_node_unref (initializer), NULL));
			class_name = (g_free (class_name), NULL);
			member_name = (g_free (member_name), NULL);
		}
	} else {
		if (vala_local_variable_get_variable_type (lv) != NULL) {
			ValaDataType* _tmp30;
			ValaDataType* _tmp29;
			_tmp30 = NULL;
			_tmp29 = NULL;
			vt = (_tmp30 = (_tmp29 = vala_local_variable_get_variable_type (lv), (_tmp29 == NULL) ? NULL : vala_code_node_ref (_tmp29)), (vt == NULL) ? NULL : (vt = (vala_code_node_unref (vt), NULL)), _tmp30);
			;
		}
	}
	return vt;
}


static gboolean vsc_symbol_completion_node_contains_position (VscSymbolCompletion* self, ValaCodeNode* node, gint line, gint column) {
	gboolean _tmp0;
	gboolean _tmp1;
	gboolean _tmp2;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (node != NULL, FALSE);
	_tmp0 = FALSE;
	_tmp1 = FALSE;
	_tmp2 = FALSE;
	if (vala_source_reference_get_first_line (vala_code_node_get_source_reference (node)) < line) {
		_tmp2 = vala_source_reference_get_last_line (vala_code_node_get_source_reference (node)) > line;
	} else {
		_tmp2 = FALSE;
	}
	if ((_tmp2)) {
		_tmp1 = TRUE;
	} else {
		gboolean _tmp3;
		_tmp3 = FALSE;
		if (vala_source_reference_get_first_line (vala_code_node_get_source_reference (node)) == line) {
			_tmp3 = vala_source_reference_get_first_column (vala_code_node_get_source_reference (node)) <= column;
		} else {
			_tmp3 = FALSE;
		}
		_tmp1 = (_tmp3);
	}
	if (_tmp1) {
		_tmp0 = TRUE;
	} else {
		gboolean _tmp4;
		_tmp4 = FALSE;
		if (vala_source_reference_get_last_line (vala_code_node_get_source_reference (node)) == line) {
			_tmp4 = vala_source_reference_get_last_column (vala_code_node_get_source_reference (node)) <= column;
		} else {
			_tmp4 = FALSE;
		}
		_tmp0 = (_tmp4);
	}
	/*
	debug ("search (%d,%d) vs (%d,%d) - (%d,%d)",
	    line, column,
	    node.source_reference.first_line,
	    node.source_reference.first_column,
	    node.source_reference.last_line,
	    node.source_reference.last_column);
	*/
	if (_tmp0) {
		return TRUE;
	}
	return FALSE;
}


static char* vsc_symbol_completion_normalize_typename (VscSymbolCompletion* self, const char* typename, const char* namespace_name) {
	char* _tmp0;
	gboolean _tmp1;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (typename != NULL, NULL);
	g_return_val_if_fail (namespace_name != NULL, NULL);
	_tmp0 = NULL;
	if ((_tmp1 = g_str_has_prefix (typename, _tmp0 = g_strdup_printf ("%s.", namespace_name)), _tmp0 = (g_free (_tmp0), NULL), _tmp1)) {
		const char* _tmp2;
		_tmp2 = NULL;
		return (_tmp2 = typename, (_tmp2 == NULL) ? NULL : g_strdup (_tmp2));
	} else {
		return g_strdup_printf ("%s.%s", namespace_name, typename);
	}
}


static char* vsc_symbol_completion_get_qualified_namespace_name (VscSymbolCompletion* self, ValaSymbol* namespace_symbol) {
	char* ns_name;
	ValaSymbol* _tmp0;
	ValaSymbol* ns;
	char* _tmp9;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (namespace_symbol != NULL, NULL);
	ns_name = NULL;
	_tmp0 = NULL;
	ns = (_tmp0 = namespace_symbol, (_tmp0 == NULL) ? NULL : vala_code_node_ref (_tmp0));
	while (TRUE) {
		gboolean _tmp1;
		_tmp1 = FALSE;
		if (ns != NULL) {
			_tmp1 = vala_symbol_get_name (ns) != NULL;
		} else {
			_tmp1 = FALSE;
		}
		if (!_tmp1) {
			break;
		}
		if (ns_name == NULL) {
			char* _tmp3;
			const char* _tmp2;
			_tmp3 = NULL;
			_tmp2 = NULL;
			ns_name = (_tmp3 = (_tmp2 = vala_symbol_get_name (ns), (_tmp2 == NULL) ? NULL : g_strdup (_tmp2)), ns_name = (g_free (ns_name), NULL), _tmp3);
		} else {
			char* _tmp4;
			_tmp4 = NULL;
			ns_name = (_tmp4 = g_strdup_printf ("%s.%s", vala_symbol_get_name (ns), ns_name), ns_name = (g_free (ns_name), NULL), _tmp4);
		}
		if (VALA_IS_UNRESOLVED_SYMBOL (ns)) {
			ValaSymbol* _tmp6;
			ValaSymbol* _tmp5;
			_tmp6 = NULL;
			_tmp5 = NULL;
			ns = (_tmp6 = (_tmp5 = (ValaSymbol*) vala_unresolved_symbol_get_inner ((VALA_UNRESOLVED_SYMBOL (ns))), (_tmp5 == NULL) ? NULL : vala_code_node_ref (_tmp5)), (ns == NULL) ? NULL : (ns = (vala_code_node_unref (ns), NULL)), _tmp6);
		} else {
			ValaSymbol* _tmp8;
			ValaSymbol* _tmp7;
			_tmp8 = NULL;
			_tmp7 = NULL;
			ns = (_tmp8 = (_tmp7 = vala_symbol_get_parent_symbol (ns), (_tmp7 == NULL) ? NULL : vala_code_node_ref (_tmp7)), (ns == NULL) ? NULL : (ns = (vala_code_node_unref (ns), NULL)), _tmp8);
		}
	}
	_tmp9 = NULL;
	return (_tmp9 = ns_name, (ns == NULL) ? NULL : (ns = (vala_code_node_unref (ns), NULL)), _tmp9);
}


VscSymbolCompletionResult* vsc_symbol_completion_get_completions_for_name (VscSymbolCompletion* self, VscSymbolCompletionFilterOptions* options, const char* symbolname, const char* sourcefile, gint line, gint column, GError** error) {
	GError * inner_error;
	VscSymbolCompletionResult* result;
	ValaSourceFile* source;
	gboolean _tmp1;
	VscTypeFinderVisitor* finder;
	VscCompletionVisitor* completion;
	gboolean _tmp26;
	gboolean _tmp30;
	gboolean _tmp31;
	VscSymbolCompletionResult* _tmp37;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (options != NULL, NULL);
	g_return_val_if_fail (symbolname != NULL, NULL);
	inner_error = NULL;
	g_warn_if_fail (self->priv->_parser != NULL);
	result = vsc_symbol_completion_result_new ();
	source = NULL;
	if (sourcefile != NULL) {
		ValaSourceFile* _tmp0;
		_tmp0 = NULL;
		source = (_tmp0 = vsc_symbol_completion_find_sourcefile (self, vsc_parser_manager_get_sec_context (self->priv->_parser), sourcefile), (source == NULL) ? NULL : (source = (vala_source_file_unref (source), NULL)), _tmp0);
	}
	/*first look in the namespaces defined in the source file*/
	vsc_parser_manager_lock_all_contexts (self->priv->_parser);
	_tmp1 = FALSE;
	if (options->local_variables) {
		_tmp1 = source != NULL;
	} else {
		_tmp1 = FALSE;
	}
	if (_tmp1) {
		ValaClass* cl;
		ValaMethod* md;
		ValaMethod* _tmp7;
		ValaCodeNode* _tmp6;
		ValaMethod* _tmp5;
		ValaClass* _tmp4;
		ValaCodeNode* _tmp3;
		ValaClass* _tmp2;
		ValaCodeNode* codenode;
		cl = NULL;
		md = NULL;
		_tmp7 = NULL;
		_tmp6 = NULL;
		_tmp5 = NULL;
		_tmp4 = NULL;
		_tmp3 = NULL;
		_tmp2 = NULL;
		codenode = (_tmp6 = (_tmp3 = vsc_symbol_completion_find_codenode (self, source, line, column, &_tmp2, &_tmp5, &inner_error), cl = (_tmp4 = _tmp2, (cl == NULL) ? NULL : (cl = (vala_code_node_unref (cl), NULL)), _tmp4), _tmp3), md = (_tmp7 = _tmp5, (md == NULL) ? NULL : (md = (vala_code_node_unref (md), NULL)), _tmp7), _tmp6);
		if (inner_error != NULL) {
			g_propagate_error (error, inner_error);
			(cl == NULL) ? NULL : (cl = (vala_code_node_unref (cl), NULL));
			(md == NULL) ? NULL : (md = (vala_code_node_unref (md), NULL));
			(result == NULL) ? NULL : (result = (g_object_unref (result), NULL));
			(source == NULL) ? NULL : (source = (vala_source_file_unref (source), NULL));
			return NULL;
		}
		if (codenode != NULL) {
			ValaCodeNode* _tmp8;
			ValaCodeNode* current;
			_tmp8 = NULL;
			current = (_tmp8 = codenode, (_tmp8 == NULL) ? NULL : vala_code_node_ref (_tmp8));
			while (current != NULL) {
				ValaBlock* body;
				body = vsc_symbol_completion_get_codenode_body (self, current);
				if (body != NULL) {
					/*method local vars*/
					{
						GeeList* _tmp9;
						GeeIterator* _tmp10;
						GeeIterator* _lvar_it;
						/*method local vars*/
						_tmp9 = NULL;
						_tmp10 = NULL;
						_lvar_it = (_tmp10 = gee_iterable_iterator ((GeeIterable*) (_tmp9 = vala_block_get_local_variables (body))), (_tmp9 == NULL) ? NULL : (_tmp9 = (gee_collection_object_unref (_tmp9), NULL)), _tmp10);
						/*method local vars*/
						while (gee_iterator_next (_lvar_it)) {
							ValaLocalVariable* lvar;
							VscSymbolCompletionItem* _tmp11;
							/*method local vars*/
							lvar = (ValaLocalVariable*) gee_iterator_get (_lvar_it);
							_tmp11 = NULL;
							gee_collection_add ((GeeCollection*) result->others, _tmp11 = vsc_symbol_completion_item_new (vala_symbol_get_name ((ValaSymbol*) lvar)));
							(_tmp11 == NULL) ? NULL : (_tmp11 = (g_object_unref (_tmp11), NULL));
							(lvar == NULL) ? NULL : (lvar = (vala_code_node_unref (lvar), NULL));
						}
						(_lvar_it == NULL) ? NULL : (_lvar_it = (gee_collection_object_unref (_lvar_it), NULL));
					}
					{
						GeeList* _tmp12;
						GeeIterator* _tmp13;
						GeeIterator* _st_it;
						_tmp12 = NULL;
						_tmp13 = NULL;
						_st_it = (_tmp13 = gee_iterable_iterator ((GeeIterable*) (_tmp12 = vala_block_get_statements (body))), (_tmp12 == NULL) ? NULL : (_tmp12 = (gee_collection_object_unref (_tmp12), NULL)), _tmp13);
						while (gee_iterator_next (_st_it)) {
							ValaStatement* st;
							st = (ValaStatement*) gee_iterator_get (_st_it);
							if (VALA_IS_DECLARATION_STATEMENT (st)) {
								ValaDeclarationStatement* _tmp14;
								ValaDeclarationStatement* decl;
								VscSymbolCompletionItem* _tmp15;
								_tmp14 = NULL;
								decl = (_tmp14 = VALA_DECLARATION_STATEMENT (st), (_tmp14 == NULL) ? NULL : vala_code_node_ref (_tmp14));
								_tmp15 = NULL;
								gee_collection_add ((GeeCollection*) result->others, _tmp15 = vsc_symbol_completion_item_new (vala_symbol_get_name (vala_declaration_statement_get_declaration (decl))));
								(_tmp15 == NULL) ? NULL : (_tmp15 = (g_object_unref (_tmp15), NULL));
								(decl == NULL) ? NULL : (decl = (vala_code_node_unref (decl), NULL));
							}
							(st == NULL) ? NULL : (st = (vala_code_node_unref (st), NULL));
						}
						(_st_it == NULL) ? NULL : (_st_it = (gee_collection_object_unref (_st_it), NULL));
					}
					if (VALA_IS_FOREACH_STATEMENT (current)) {
						ValaForeachStatement* _tmp16;
						ValaForeachStatement* fe;
						VscSymbolCompletionItem* _tmp17;
						_tmp16 = NULL;
						fe = (_tmp16 = VALA_FOREACH_STATEMENT (current), (_tmp16 == NULL) ? NULL : vala_code_node_ref (_tmp16));
						_tmp17 = NULL;
						gee_collection_add ((GeeCollection*) result->others, _tmp17 = vsc_symbol_completion_item_new (vala_foreach_statement_get_variable_name (fe)));
						(_tmp17 == NULL) ? NULL : (_tmp17 = (g_object_unref (_tmp17), NULL));
						(fe == NULL) ? NULL : (fe = (vala_code_node_unref (fe), NULL));
					}
				}
				if (VALA_IS_METHOD (current)) {
					/*method arguments*/
					{
						GeeList* _tmp18;
						GeeIterator* _tmp19;
						GeeIterator* _par_it;
						/*method arguments*/
						_tmp18 = NULL;
						_tmp19 = NULL;
						_par_it = (_tmp19 = gee_iterable_iterator ((GeeIterable*) (_tmp18 = vala_method_get_parameters ((VALA_METHOD (current))))), (_tmp18 == NULL) ? NULL : (_tmp18 = (gee_collection_object_unref (_tmp18), NULL)), _tmp19);
						/*method arguments*/
						while (gee_iterator_next (_par_it)) {
							ValaFormalParameter* par;
							VscSymbolCompletionItem* _tmp20;
							/*method arguments*/
							par = (ValaFormalParameter*) gee_iterator_get (_par_it);
							_tmp20 = NULL;
							gee_collection_add ((GeeCollection*) result->others, _tmp20 = vsc_symbol_completion_item_new (vala_symbol_get_name ((ValaSymbol*) par)));
							(_tmp20 == NULL) ? NULL : (_tmp20 = (g_object_unref (_tmp20), NULL));
							(par == NULL) ? NULL : (par = (vala_code_node_unref (par), NULL));
						}
						(_par_it == NULL) ? NULL : (_par_it = (gee_collection_object_unref (_par_it), NULL));
					}
					/*stop here*/
					(body == NULL) ? NULL : (body = (vala_code_node_unref (body), NULL));
					break;
				} else {
					ValaCodeNode* _tmp22;
					ValaCodeNode* _tmp21;
					_tmp22 = NULL;
					_tmp21 = NULL;
					current = (_tmp22 = (_tmp21 = vala_code_node_get_parent_node (current), (_tmp21 == NULL) ? NULL : vala_code_node_ref (_tmp21)), (current == NULL) ? NULL : (current = (vala_code_node_unref (current), NULL)), _tmp22);
				}
				(body == NULL) ? NULL : (body = (vala_code_node_unref (body), NULL));
			}
			(current == NULL) ? NULL : (current = (vala_code_node_unref (current), NULL));
		}
		(cl == NULL) ? NULL : (cl = (vala_code_node_unref (cl), NULL));
		(md == NULL) ? NULL : (md = (vala_code_node_unref (md), NULL));
		(codenode == NULL) ? NULL : (codenode = (vala_code_node_unref (codenode), NULL));
	}
	finder = vsc_typefinder_visitor_new (source, vsc_parser_manager_get_pri_context (self->priv->_parser));
	completion = vsc_completion_visitor_new (options, result, source, vsc_parser_manager_get_pri_context (self->priv->_parser));
	if (source != NULL) {
		{
			GeeList* _tmp23;
			GeeIterator* _tmp24;
			GeeIterator* _node_it;
			_tmp23 = NULL;
			_tmp24 = NULL;
			_node_it = (_tmp24 = gee_iterable_iterator ((GeeIterable*) (_tmp23 = vala_source_file_get_nodes (source))), (_tmp23 == NULL) ? NULL : (_tmp23 = (gee_collection_object_unref (_tmp23), NULL)), _tmp24);
			while (gee_iterator_next (_node_it)) {
				ValaCodeNode* node;
				node = (ValaCodeNode*) gee_iterator_get (_node_it);
				if (VALA_IS_NAMESPACE (node)) {
					ValaNamespace* _tmp25;
					ValaNamespace* ns;
					char* ns_name;
					char* name;
					_tmp25 = NULL;
					ns = (_tmp25 = VALA_NAMESPACE (node), (_tmp25 == NULL) ? NULL : vala_code_node_ref (_tmp25));
					ns_name = vsc_symbol_completion_get_qualified_namespace_name (self, (ValaSymbol*) ns);
					name = vsc_symbol_completion_normalize_typename (self, symbolname, ns_name);
					vsc_typefinder_visitor_set_searched_typename (finder, name);
					vala_code_visitor_visit_namespace ((ValaCodeVisitor*) finder, ns);
					if (vsc_typefinder_visitor_get_result (finder) != NULL) {
						vala_code_node_accept ((ValaCodeNode*) vsc_typefinder_visitor_get_result (finder), (ValaCodeVisitor*) completion);
						if (VALA_IS_NAMESPACE (vsc_typefinder_visitor_get_result (finder))) {
							vsc_symbol_completion_get_completion_for_name_in_namespace_with_context (self, ns_name, symbolname, finder, completion, vsc_parser_manager_get_pri_context (self->priv->_parser));
						}
						(ns == NULL) ? NULL : (ns = (vala_code_node_unref (ns), NULL));
						ns_name = (g_free (ns_name), NULL);
						name = (g_free (name), NULL);
						(node == NULL) ? NULL : (node = (vala_code_node_unref (node), NULL));
						break;
					} else {
						gint tmp;
						tmp = vsc_symbol_completion_result_get_count (result);
						vsc_symbol_completion_get_completion_for_name_in_namespace_with_context (self, ns_name, symbolname, finder, completion, vsc_parser_manager_get_pri_context (self->priv->_parser));
						if (tmp != vsc_symbol_completion_result_get_count (result)) {
							(ns == NULL) ? NULL : (ns = (vala_code_node_unref (ns), NULL));
							ns_name = (g_free (ns_name), NULL);
							name = (g_free (name), NULL);
							(node == NULL) ? NULL : (node = (vala_code_node_unref (node), NULL));
							break;
						}
					}
					(ns == NULL) ? NULL : (ns = (vala_code_node_unref (ns), NULL));
					ns_name = (g_free (ns_name), NULL);
					name = (g_free (name), NULL);
				}
				(node == NULL) ? NULL : (node = (vala_code_node_unref (node), NULL));
			}
			(_node_it == NULL) ? NULL : (_node_it = (gee_collection_object_unref (_node_it), NULL));
		}
	} else {
		/*found something in primary context*/
		g_warning ("vscsymbolcompletion.vala:828: no source file found");
	}
	if (vsc_typefinder_visitor_get_result (finder) == NULL) {
		/*search it in the root namespace, string and other base types are there*/
		vsc_typefinder_visitor_set_searched_typename (finder, symbolname);
		vala_code_visitor_visit_namespace ((ValaCodeVisitor*) finder, vala_code_context_get_root (vsc_parser_manager_get_pri_context (self->priv->_parser)));
		if (vsc_typefinder_visitor_get_result (finder) != NULL) {
			vala_code_node_accept ((ValaCodeNode*) vsc_typefinder_visitor_get_result (finder), (ValaCodeVisitor*) completion);
		}
	}
	_tmp26 = FALSE;
	if (source != NULL) {
		_tmp26 = options->imported_namespaces;
	} else {
		_tmp26 = FALSE;
	}
	/*import symbols from the imported namespaces*/
	if (_tmp26) {
		const char* _tmp27;
		char* prev_symbol;
		_tmp27 = NULL;
		prev_symbol = (_tmp27 = vsc_typefinder_visitor_get_searched_typename (finder), (_tmp27 == NULL) ? NULL : g_strdup (_tmp27));
		{
			GeeList* _tmp28;
			GeeIterator* _tmp29;
			GeeIterator* _item_it;
			_tmp28 = NULL;
			_tmp29 = NULL;
			_item_it = (_tmp29 = gee_iterable_iterator ((GeeIterable*) (_tmp28 = vala_source_file_get_using_directives (source))), (_tmp28 == NULL) ? NULL : (_tmp28 = (gee_collection_object_unref (_tmp28), NULL)), _tmp29);
			while (gee_iterator_next (_item_it)) {
				ValaUsingDirective* item;
				char* ns_name;
				item = (ValaUsingDirective*) gee_iterator_get (_item_it);
				ns_name = vsc_symbol_completion_get_qualified_namespace_name (self, vala_using_directive_get_namespace_symbol (item));
				vsc_typefinder_visitor_set_searched_typename (finder, ns_name);
				vsc_symbol_completion_get_completion_for_name_in_namespace_with_context (self, ns_name, ns_name, finder, completion, vsc_parser_manager_get_pri_context (self->priv->_parser));
				(item == NULL) ? NULL : (item = (vala_code_node_unref (item), NULL));
				ns_name = (g_free (ns_name), NULL);
			}
			(_item_it == NULL) ? NULL : (_item_it = (gee_collection_object_unref (_item_it), NULL));
		}
		vsc_typefinder_visitor_set_searched_typename (finder, prev_symbol);
		prev_symbol = (g_free (prev_symbol), NULL);
	}
	_tmp30 = FALSE;
	_tmp31 = FALSE;
	if (source != NULL) {
		_tmp31 = vsc_typefinder_visitor_get_result (finder) == NULL;
	} else {
		_tmp31 = FALSE;
	}
	if (_tmp31) {
		_tmp30 = !vsc_symbol_completion_symbol_has_known_namespace (symbolname);
	} else {
		_tmp30 = FALSE;
	}
	/*search it in referenced namespaces*/
	if (_tmp30) {
		gboolean force_exit;
		force_exit = FALSE;
		{
			GeeList* _tmp32;
			GeeIterator* _tmp33;
			GeeIterator* _item_it;
			_tmp32 = NULL;
			_tmp33 = NULL;
			_item_it = (_tmp33 = gee_iterable_iterator ((GeeIterable*) (_tmp32 = vala_source_file_get_using_directives (source))), (_tmp32 == NULL) ? NULL : (_tmp32 = (gee_collection_object_unref (_tmp32), NULL)), _tmp33);
			while (gee_iterator_next (_item_it)) {
				ValaUsingDirective* item;
				gint tmp;
				char* ns_name;
				char* _tmp34;
				gboolean _tmp35;
				gboolean _tmp36;
				item = (ValaUsingDirective*) gee_iterator_get (_item_it);
				tmp = vsc_symbol_completion_result_get_count (result);
				ns_name = vsc_symbol_completion_get_qualified_namespace_name (self, vala_using_directive_get_namespace_symbol (item));
				_tmp34 = NULL;
				if ((_tmp35 = g_str_has_prefix (symbolname, _tmp34 = g_strdup_printf ("%s.", ns_name)), _tmp34 = (g_free (_tmp34), NULL), _tmp35)) {
					force_exit = TRUE;
				}
				/*exit after this visit since symbolname is surely fully qualified*/
				vsc_symbol_completion_get_completion_for_name_in_namespace_with_context (self, ns_name, symbolname, finder, completion, vsc_parser_manager_get_pri_context (self->priv->_parser));
				_tmp36 = FALSE;
				if (tmp != vsc_symbol_completion_result_get_count (result)) {
					_tmp36 = TRUE;
				} else {
					_tmp36 = force_exit;
				}
				if (_tmp36) {
					(item == NULL) ? NULL : (item = (vala_code_node_unref (item), NULL));
					ns_name = (g_free (ns_name), NULL);
					break;
				}
				(item == NULL) ? NULL : (item = (vala_code_node_unref (item), NULL));
				ns_name = (g_free (ns_name), NULL);
			}
			(_item_it == NULL) ? NULL : (_item_it = (gee_collection_object_unref (_item_it), NULL));
		}
	}
	vsc_parser_manager_unlock_all_contexts (self->priv->_parser);
	_tmp37 = NULL;
	return (_tmp37 = result, (source == NULL) ? NULL : (source = (vala_source_file_unref (source), NULL)), (finder == NULL) ? NULL : (finder = (vala_code_visitor_unref (finder), NULL)), (completion == NULL) ? NULL : (completion = (vala_code_visitor_unref (completion), NULL)), _tmp37);
}


static void vsc_symbol_completion_get_completion_for_name_in_namespace_with_context (VscSymbolCompletion* self, const char* namespace_name, const char* typename, VscTypeFinderVisitor* finder, VscCompletionVisitor* completion, ValaCodeContext* context) {
	const char* _tmp2;
	gint _tmp1_length1;
	char** _tmp1;
	char** _tmp0;
	char* _tmp3;
	char* ns_name;
	g_return_if_fail (self != NULL);
	g_return_if_fail (namespace_name != NULL);
	g_return_if_fail (typename != NULL);
	g_return_if_fail (finder != NULL);
	g_return_if_fail (completion != NULL);
	g_return_if_fail (context != NULL);
	_tmp2 = NULL;
	_tmp1 = NULL;
	_tmp0 = NULL;
	_tmp3 = NULL;
	ns_name = (_tmp3 = (_tmp2 = (_tmp1 = _tmp0 = g_strsplit (namespace_name, ".", 2), _tmp1_length1 = _vala_array_length (_tmp0), _tmp1)[0], (_tmp2 == NULL) ? NULL : g_strdup (_tmp2)), _tmp1 = (_vala_array_free (_tmp1, _tmp1_length1, (GDestroyNotify) g_free), NULL), _tmp3);
	{
		GeeList* _tmp4;
		GeeIterator* _tmp5;
		GeeIterator* _ns_it;
		_tmp4 = NULL;
		_tmp5 = NULL;
		_ns_it = (_tmp5 = gee_iterable_iterator ((GeeIterable*) (_tmp4 = vala_namespace_get_namespaces (vala_code_context_get_root (context)))), (_tmp4 == NULL) ? NULL : (_tmp4 = (gee_collection_object_unref (_tmp4), NULL)), _tmp5);
		while (gee_iterator_next (_ns_it)) {
			ValaNamespace* ns;
			ns = (ValaNamespace*) gee_iterator_get (_ns_it);
			if (_vala_strcmp0 (vala_symbol_get_name ((ValaSymbol*) ns), ns_name) == 0) {
				gboolean same_namespace;
				same_namespace = FALSE;
				if (_vala_strcmp0 (typename, namespace_name) == 0) {
					/*search all symbols*/
					if (strstr (namespace_name, ".") == NULL) {
						same_namespace = TRUE;
					} else {
						/*don't waste time finding this exact namespace*/
						vsc_typefinder_visitor_set_searched_typename (finder, namespace_name);
					}
				} else {
					char* _tmp6;
					_tmp6 = NULL;
					vsc_typefinder_visitor_set_searched_typename (finder, _tmp6 = g_strdup_printf ("%s.%s", namespace_name, typename));
					_tmp6 = (g_free (_tmp6), NULL);
				}
				if (same_namespace) {
					vsc_completion_visitor_integrate_completion (completion, (ValaSymbol*) ns);
				} else {
					vala_code_visitor_visit_namespace ((ValaCodeVisitor*) finder, ns);
					if (vsc_typefinder_visitor_get_result (finder) != NULL) {
						vsc_completion_visitor_integrate_completion (completion, vsc_typefinder_visitor_get_result (finder));
					}
				}
				(ns == NULL) ? NULL : (ns = (vala_code_node_unref (ns), NULL));
				break;
			}
			(ns == NULL) ? NULL : (ns = (vala_code_node_unref (ns), NULL));
		}
		(_ns_it == NULL) ? NULL : (_ns_it = (gee_collection_object_unref (_ns_it), NULL));
	}
	ns_name = (g_free (ns_name), NULL);
}


gboolean vsc_symbol_completion_symbol_has_known_namespace (const char* name) {
	gboolean _tmp0;
	gboolean _tmp1;
	g_return_val_if_fail (name != NULL, FALSE);
	_tmp0 = FALSE;
	_tmp1 = FALSE;
	if (g_str_has_prefix (name, "GLib.")) {
		_tmp1 = TRUE;
	} else {
		_tmp1 = g_str_has_prefix (name, "Gtk.");
	}
	if (_tmp1) {
		_tmp0 = TRUE;
	} else {
		_tmp0 = g_str_has_prefix (name, "Gdk.");
	}
	return (_tmp0);
}


VscSymbolCompletion* vsc_symbol_completion_construct (GType object_type) {
	VscSymbolCompletion * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


VscSymbolCompletion* vsc_symbol_completion_new (void) {
	return vsc_symbol_completion_construct (VSC_TYPE_SYMBOL_COMPLETION);
}


VscParserManager* vsc_symbol_completion_get_parser (VscSymbolCompletion* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_parser;
}


static void vsc_symbol_completion_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	VscSymbolCompletion * self;
	gpointer boxed;
	self = VSC_SYMBOL_COMPLETION (object);
	switch (property_id) {
		case VSC_SYMBOL_COMPLETION_PARSER:
		g_value_set_object (value, vsc_symbol_completion_get_parser (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vsc_symbol_completion_class_init (VscSymbolCompletionClass * klass) {
	vsc_symbol_completion_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VscSymbolCompletionPrivate));
	G_OBJECT_CLASS (klass)->get_property = vsc_symbol_completion_get_property;
	G_OBJECT_CLASS (klass)->finalize = vsc_symbol_completion_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VSC_SYMBOL_COMPLETION_PARSER, g_param_spec_object ("parser", "parser", "parser", VSC_TYPE_PARSER_MANAGER, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void vsc_symbol_completion_instance_init (VscSymbolCompletion * self) {
	self->priv = VSC_SYMBOL_COMPLETION_GET_PRIVATE (self);
	self->priv->_parser = vsc_parser_manager_new ();
}


static void vsc_symbol_completion_finalize (GObject* obj) {
	VscSymbolCompletion * self;
	self = VSC_SYMBOL_COMPLETION (obj);
	(self->priv->_parser == NULL) ? NULL : (self->priv->_parser = (g_object_unref (self->priv->_parser), NULL));
	G_OBJECT_CLASS (vsc_symbol_completion_parent_class)->finalize (obj);
}


GType vsc_symbol_completion_get_type (void) {
	static GType vsc_symbol_completion_type_id = 0;
	if (vsc_symbol_completion_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (VscSymbolCompletionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vsc_symbol_completion_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VscSymbolCompletion), 0, (GInstanceInitFunc) vsc_symbol_completion_instance_init, NULL };
		vsc_symbol_completion_type_id = g_type_register_static (G_TYPE_OBJECT, "VscSymbolCompletion", &g_define_type_info, 0);
	}
	return vsc_symbol_completion_type_id;
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




