/*
 *  vtgbracketcompletion.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgbracketcompletion.h"
#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <float.h>
#include <math.h>
#include <gdk/gdkkeysyms.h>
#include <gedit/gedit-prefs-manager.h>




static glong string_get_length (const char* self);
struct _VtgBracketCompletionPrivate {
	VtgPlugin* _plugin;
	GeditView* _view;
	char* tab_chars;
};

#define VTG_BRACKET_COMPLETION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_TYPE_BRACKET_COMPLETION, VtgBracketCompletionPrivate))
enum  {
	VTG_BRACKET_COMPLETION_DUMMY_PROPERTY,
	VTG_BRACKET_COMPLETION_PLUGIN,
	VTG_BRACKET_COMPLETION_VIEW
};
static void vtg_bracket_completion_connect_view (VtgBracketCompletion* self, GeditView* view);
static void vtg_bracket_completion_disconnect_view (VtgBracketCompletion* self, GeditView* view);
static gboolean vtg_bracket_completion_enclose_selection_with_delimiters (VtgBracketCompletion* self, GtkTextBuffer* src, const char* start_delimiter, const char* end_delimiter);
static void vtg_bracket_completion_insert_chars (VtgBracketCompletion* self, GtkTextBuffer* src, const char* chars);
static void vtg_bracket_completion_move_backwards (VtgBracketCompletion* self, GtkTextBuffer* src, gint count);
static const char* vtg_bracket_completion_current_indentation_text (VtgBracketCompletion* self, GtkTextBuffer* src);
static gboolean vtg_bracket_completion_find_char (VtgBracketCompletion* self, const GtkTextIter* start, gunichar char_to_find, gunichar complementary_char, gunichar stop_to_char);
static gboolean vtg_bracket_completion_on_view_key_press (GeditView* sender, const GdkEventKey* evt, VtgBracketCompletion* instance);
static void vtg_bracket_completion_set_plugin (VtgBracketCompletion* self, VtgPlugin* value);
static void vtg_bracket_completion_set_view (VtgBracketCompletion* self, GeditView* value);
static GObject * vtg_bracket_completion_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer vtg_bracket_completion_parent_class = NULL;
static void vtg_bracket_completion_finalize (GObject* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);



static glong string_get_length (const char* self) {
	g_return_val_if_fail (self != NULL, 0L);
	return g_utf8_strlen (self, -1);
}


VtgBracketCompletion* vtg_bracket_completion_construct (GType object_type, VtgPlugin* plugin, GeditView* view) {
	GParameter * __params;
	GParameter * __params_it;
	VtgBracketCompletion * self;
	g_return_val_if_fail (plugin != NULL, NULL);
	g_return_val_if_fail (view != NULL, NULL);
	__params = g_new0 (GParameter, 2);
	__params_it = __params;
	__params_it->name = "plugin";
	g_value_init (&__params_it->value, VTG_TYPE_PLUGIN);
	g_value_set_object (&__params_it->value, plugin);
	__params_it++;
	__params_it->name = "view";
	g_value_init (&__params_it->value, GEDIT_TYPE_VIEW);
	g_value_set_object (&__params_it->value, view);
	__params_it++;
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


VtgBracketCompletion* vtg_bracket_completion_new (VtgPlugin* plugin, GeditView* view) {
	return vtg_bracket_completion_construct (VTG_TYPE_BRACKET_COMPLETION, plugin, view);
}


void vtg_bracket_completion_deactivate (VtgBracketCompletion* self) {
	g_return_if_fail (self != NULL);
	vtg_bracket_completion_disconnect_view (self, self->priv->_view);
}


static void vtg_bracket_completion_connect_view (VtgBracketCompletion* self, GeditView* view) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (view != NULL);
	g_signal_connect (view, "key-press-event", (GCallback) vtg_bracket_completion_on_view_key_press, self);
}


static void vtg_bracket_completion_disconnect_view (VtgBracketCompletion* self, GeditView* view) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (view != NULL);
	g_signal_handlers_disconnect_by_func (view, (void*) vtg_bracket_completion_on_view_key_press, self);
}


static gboolean vtg_bracket_completion_enclose_selection_with_delimiters (VtgBracketCompletion* self, GtkTextBuffer* src, const char* start_delimiter, const char* end_delimiter) {
	GtkTextIter sel_start = {0};
	GtkTextIter sel_end = {0};
	const char* _tmp0;
	char* text;
	gboolean _tmp1;
	gboolean _tmp4;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (src != NULL, FALSE);
	g_return_val_if_fail (start_delimiter != NULL, FALSE);
	gtk_text_buffer_get_selection_bounds (src, &sel_start, &sel_end);
	_tmp0 = NULL;
	text = (_tmp0 = gtk_text_buffer_get_text (src, &sel_start, &sel_end, TRUE), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
	if (end_delimiter == NULL) {
		end_delimiter = start_delimiter;
	}
	_tmp1 = FALSE;
	if (g_str_has_prefix (text, start_delimiter) == FALSE) {
		_tmp1 = g_str_has_suffix (text, end_delimiter) == FALSE;
	} else {
		_tmp1 = FALSE;
	}
	if (_tmp1) {
		GtkTextMark* mark;
		GtkTextIter pos = {0};
		char* _tmp2;
		gboolean _tmp3;
		mark = GTK_TEXT_MARK (gtk_text_buffer_get_insert (src));
		_tmp2 = NULL;
		text = (_tmp2 = g_strdup_printf ("%s%s%s", start_delimiter, text, end_delimiter), text = (g_free (text), NULL), _tmp2);
		gtk_text_buffer_begin_user_action (src);
		gtk_text_buffer_delete_selection (src, TRUE, TRUE);
		gtk_text_buffer_get_iter_at_mark (src, &pos, mark);
		gtk_text_buffer_insert (src, &pos, text, (gint) g_utf8_strlen (text, -1));
		gtk_text_buffer_end_user_action (src);
		return (_tmp3 = TRUE, text = (g_free (text), NULL), _tmp3);
	}
	return (_tmp4 = FALSE, text = (g_free (text), NULL), _tmp4);
}


static void vtg_bracket_completion_insert_chars (VtgBracketCompletion* self, GtkTextBuffer* src, const char* chars) {
	GtkTextMark* mark;
	GtkTextIter pos = {0};
	g_return_if_fail (self != NULL);
	g_return_if_fail (src != NULL);
	g_return_if_fail (chars != NULL);
	mark = GTK_TEXT_MARK (gtk_text_buffer_get_insert (src));
	gtk_text_buffer_get_iter_at_mark (src, &pos, mark);
	gtk_text_buffer_begin_user_action (src);
	gtk_text_buffer_insert (src, &pos, chars, (gint) g_utf8_strlen (chars, -1));
	gtk_text_buffer_end_user_action (src);
}


static void vtg_bracket_completion_move_backwards (VtgBracketCompletion* self, GtkTextBuffer* src, gint count) {
	GtkTextMark* mark;
	GtkTextIter pos = {0};
	g_return_if_fail (self != NULL);
	g_return_if_fail (src != NULL);
	mark = GTK_TEXT_MARK (gtk_text_buffer_get_insert (src));
	gtk_text_buffer_get_iter_at_mark (src, &pos, mark);
	gtk_text_iter_backward_chars (&pos, count);
	gtk_text_buffer_place_cursor (src, &pos);
}


static const char* vtg_bracket_completion_current_indentation_text (VtgBracketCompletion* self, GtkTextBuffer* src) {
	GtkTextMark* mark;
	GtkTextIter end = {0};
	GtkTextIter start = {0};
	const char* text;
	gint col;
	gunichar ch;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (src != NULL, NULL);
	mark = GTK_TEXT_MARK (gtk_text_buffer_get_insert (src));
	text = "";
	col = 0;
	gtk_text_buffer_get_iter_at_mark (src, &end, mark);
	col = gtk_text_iter_get_line_offset (&end);
	gtk_text_iter_set_line_offset (&end, 0);
	start = end;
	ch = gtk_text_iter_get_char (&end);
	if (g_unichar_isspace (ch)) {
		while (gtk_text_iter_forward_char (&end)) {
			if (gtk_text_iter_starts_word (&end)) {
				break;
			} else {
				if (gtk_text_iter_get_line_offset (&end) >= col) {
					break;
				} else {
					ch = gtk_text_iter_get_char (&end);
					if (g_unichar_isspace (ch) == FALSE) {
						break;
					}
				}
			}
		}
	}
	if (!gtk_text_iter_equal (&start, &end)) {
		text = gtk_text_iter_get_text (&start, &end);
	}
	return text;
}


static gboolean vtg_bracket_completion_find_char (VtgBracketCompletion* self, const GtkTextIter* start, gunichar char_to_find, gunichar complementary_char, gunichar stop_to_char) {
	gboolean result;
	gint level;
	GtkTextIter curr;
	g_return_val_if_fail (self != NULL, FALSE);
	result = FALSE;
	level = 0;
	curr = (*start);
	do {
		gunichar ch;
		ch = gtk_text_iter_get_char (&curr);
		if (ch == stop_to_char) {
			break;
		} else {
			if (ch == char_to_find) {
				if (level == 0) {
					result = TRUE;
					break;
				} else {
					level--;
				}
			} else {
				if (ch == complementary_char) {
					level++;
				}
			}
		}
	} while (gtk_text_iter_forward_char (&curr));
	return result;
}


static gboolean vtg_bracket_completion_on_view_key_press (GeditView* sender, const GdkEventKey* evt, VtgBracketCompletion* instance) {
	gboolean result;
	g_return_val_if_fail (sender != NULL, FALSE);
	g_return_val_if_fail (instance != NULL, FALSE);
	result = FALSE;
	if (((*evt).state & (GDK_MOD1_MASK)) == 0) {
		GtkTextBuffer* _tmp0;
		GtkTextBuffer* src;
		GtkTextMark* mark;
		GtkTextIter pos = {0};
		const char* indent;
		char* buffer;
		gunichar ch;
		_tmp0 = NULL;
		src = (_tmp0 = gtk_text_view_get_buffer ((GtkTextView*) sender), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
		mark = GTK_TEXT_MARK (gtk_text_buffer_get_insert (src));
		indent = NULL;
		buffer = NULL;
		ch = (gunichar) gdk_keyval_to_unicode ((*evt).keyval);
		gtk_text_buffer_get_iter_at_mark (src, &pos, mark);
		if (ch == '(') {
			if (gtk_text_buffer_get_has_selection (src)) {
				if (vtg_bracket_completion_enclose_selection_with_delimiters (instance, src, "(", ")")) {
					gtk_text_buffer_get_iter_at_mark (src, &pos, mark);
					gtk_text_buffer_place_cursor (src, &pos);
					result = TRUE;
				}
			} else {
				if (!vtg_bracket_completion_find_char (instance, &pos, (gunichar) ')', (gunichar) '(', (gunichar) ';')) {
					vtg_bracket_completion_insert_chars (instance, src, ")");
					vtg_bracket_completion_move_backwards (instance, src, 1);
				}
			}
		} else {
			if (ch == '[') {
				if (!vtg_bracket_completion_find_char (instance, &pos, (gunichar) ']', (gunichar) '[', (gunichar) ';')) {
					vtg_bracket_completion_insert_chars (instance, src, "]");
					vtg_bracket_completion_move_backwards (instance, src, 1);
				}
			} else {
				if (ch == '*') {
					gtk_text_iter_backward_char (&pos);
					if (gtk_text_iter_get_char (&pos) == '/') {
						char* _tmp1;
						indent = vtg_bracket_completion_current_indentation_text (instance, src);
						_tmp1 = NULL;
						buffer = (_tmp1 = g_strdup_printf ("*\n%s \n%s*/", indent, indent), buffer = (g_free (buffer), NULL), _tmp1);
						vtg_bracket_completion_insert_chars (instance, src, buffer);
						gtk_text_view_scroll_to_mark ((GtkTextView*) sender, mark, (double) 0, FALSE, (double) 0, (double) 0);
						vtg_bracket_completion_move_backwards (instance, src, (2 + ((gint) string_get_length (indent))) + 1);
						result = TRUE;
					}
				} else {
					if (ch == '{') {
						char* _tmp2;
						indent = vtg_bracket_completion_current_indentation_text (instance, src);
						_tmp2 = NULL;
						buffer = (_tmp2 = g_strdup_printf ("{\n%s%s\n%s}", indent, instance->priv->tab_chars, indent), buffer = (g_free (buffer), NULL), _tmp2);
						vtg_bracket_completion_insert_chars (instance, src, buffer);
						gtk_text_view_scroll_to_mark ((GtkTextView*) sender, mark, (double) 0, FALSE, (double) 0, (double) 0);
						vtg_bracket_completion_move_backwards (instance, src, 2 + ((gint) string_get_length (indent)));
						result = TRUE;
					} else {
						if ((*evt).keyval == GDK_Return) {
							indent = vtg_bracket_completion_current_indentation_text (instance, src);
							if (((*evt).state & GDK_SHIFT_MASK) != 0) {
								/*move to end line*/
								if (!gtk_text_iter_ends_line (&pos)) {
									gtk_text_iter_forward_to_line_end (&pos);
									gtk_text_buffer_place_cursor (src, &pos);
								}
								if (((*evt).state & GDK_CONTROL_MASK) == 0) {
									GtkTextIter tmp;
									/*move backward to first non blank char*/
									tmp = pos;
									while (gtk_text_iter_backward_char (&pos)) {
										ch = gtk_text_iter_get_char (&pos);
										if (!g_unichar_isspace (ch)) {
											gtk_text_iter_forward_char (&pos);
											break;
										}
									}
									if (gtk_text_iter_equal (&tmp, &pos)) {
										char* _tmp3;
										_tmp3 = NULL;
										vtg_bracket_completion_insert_chars (instance, src, _tmp3 = g_strdup_printf (";\n%s", indent));
										_tmp3 = (g_free (_tmp3), NULL);
									} else {
										char* _tmp4;
										gtk_text_buffer_place_cursor (src, &pos);
										vtg_bracket_completion_insert_chars (instance, src, ";");
										gtk_text_buffer_get_iter_at_mark (src, &pos, mark);
										gtk_text_iter_forward_to_line_end (&pos);
										gtk_text_buffer_place_cursor (src, &pos);
										_tmp4 = NULL;
										vtg_bracket_completion_insert_chars (instance, src, _tmp4 = g_strdup_printf ("\n%s", indent));
										_tmp4 = (g_free (_tmp4), NULL);
									}
									gtk_text_view_scroll_to_mark ((GtkTextView*) sender, mark, (double) 0, FALSE, (double) 0, (double) 0);
									/*place cursor to end*/
									gtk_text_buffer_get_iter_at_mark (src, &pos, mark);
									gtk_text_buffer_place_cursor (src, &pos);
								} else {
									char* _tmp5;
									_tmp5 = NULL;
									buffer = (_tmp5 = g_strdup_printf ("\n%s{\n%s%s\n%s}", indent, indent, instance->priv->tab_chars, indent), buffer = (g_free (buffer), NULL), _tmp5);
									vtg_bracket_completion_insert_chars (instance, src, buffer);
									gtk_text_view_scroll_to_mark ((GtkTextView*) sender, mark, (double) 0, FALSE, (double) 0, (double) 0);
									vtg_bracket_completion_move_backwards (instance, src, 2 + ((gint) string_get_length (indent)));
								}
								result = TRUE;
							} else {
								gboolean _tmp6;
								_tmp6 = FALSE;
								if (indent != NULL) {
									_tmp6 = _vala_strcmp0 (indent, "") != 0;
								} else {
									_tmp6 = FALSE;
								}
								if (_tmp6) {
									char* _tmp7;
									_tmp7 = NULL;
									vtg_bracket_completion_insert_chars (instance, src, _tmp7 = g_strdup_printf ("\n%s", indent));
									_tmp7 = (g_free (_tmp7), NULL);
									result = TRUE;
									gtk_text_view_scroll_to_mark ((GtkTextView*) sender, mark, (double) 0, FALSE, (double) 0, (double) 0);
								}
							}
						} else {
							if (ch == '\"') {
								if (gtk_text_buffer_get_has_selection (src)) {
									if (vtg_bracket_completion_enclose_selection_with_delimiters (instance, src, "\"", NULL)) {
										gtk_text_buffer_get_iter_at_mark (src, &pos, mark);
										gtk_text_buffer_place_cursor (src, &pos);
										result = TRUE;
									}
								} else {
									if (!vtg_bracket_completion_find_char (instance, &pos, (gunichar) '\"', (gunichar) '\"', (gunichar) ';')) {
										vtg_bracket_completion_insert_chars (instance, src, "\"");
										vtg_bracket_completion_move_backwards (instance, src, 1);
									}
								}
							}
						}
					}
				}
			}
		}
		(src == NULL) ? NULL : (src = (g_object_unref (src), NULL));
		buffer = (g_free (buffer), NULL);
	}
	return result;
}


VtgPlugin* vtg_bracket_completion_get_plugin (VtgBracketCompletion* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_plugin;
}


static void vtg_bracket_completion_set_plugin (VtgBracketCompletion* self, VtgPlugin* value) {
	VtgPlugin* _tmp2;
	VtgPlugin* _tmp1;
	g_return_if_fail (self != NULL);
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_plugin = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), (self->priv->_plugin == NULL) ? NULL : (self->priv->_plugin = (g_object_unref (self->priv->_plugin), NULL)), _tmp2);
	g_object_notify ((GObject *) self, "plugin");
}


GeditView* vtg_bracket_completion_get_view (VtgBracketCompletion* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_view;
}


static void vtg_bracket_completion_set_view (VtgBracketCompletion* self, GeditView* value) {
	GeditView* _tmp2;
	GeditView* _tmp1;
	g_return_if_fail (self != NULL);
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_view = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), (self->priv->_view == NULL) ? NULL : (self->priv->_view = (g_object_unref (self->priv->_view), NULL)), _tmp2);
	g_object_notify ((GObject *) self, "view");
}


static GObject * vtg_bracket_completion_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	VtgBracketCompletionClass * klass;
	GObjectClass * parent_class;
	VtgBracketCompletion * self;
	klass = VTG_BRACKET_COMPLETION_CLASS (g_type_class_peek (VTG_TYPE_BRACKET_COMPLETION));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VTG_BRACKET_COMPLETION (obj);
	{
		if (gedit_prefs_manager_get_insert_spaces ()) {
			char* _tmp0;
			_tmp0 = NULL;
			self->priv->tab_chars = (_tmp0 = g_strnfill ((gulong) gedit_prefs_manager_get_tabs_size (), ' '), self->priv->tab_chars = (g_free (self->priv->tab_chars), NULL), _tmp0);
		} else {
			char* _tmp1;
			_tmp1 = NULL;
			self->priv->tab_chars = (_tmp1 = g_strdup ("\t"), self->priv->tab_chars = (g_free (self->priv->tab_chars), NULL), _tmp1);
		}
		vtg_bracket_completion_connect_view (self, self->priv->_view);
	}
	return obj;
}


static void vtg_bracket_completion_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	VtgBracketCompletion * self;
	gpointer boxed;
	self = VTG_BRACKET_COMPLETION (object);
	switch (property_id) {
		case VTG_BRACKET_COMPLETION_PLUGIN:
		g_value_set_object (value, vtg_bracket_completion_get_plugin (self));
		break;
		case VTG_BRACKET_COMPLETION_VIEW:
		g_value_set_object (value, vtg_bracket_completion_get_view (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_bracket_completion_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	VtgBracketCompletion * self;
	self = VTG_BRACKET_COMPLETION (object);
	switch (property_id) {
		case VTG_BRACKET_COMPLETION_PLUGIN:
		vtg_bracket_completion_set_plugin (self, g_value_get_object (value));
		break;
		case VTG_BRACKET_COMPLETION_VIEW:
		vtg_bracket_completion_set_view (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_bracket_completion_class_init (VtgBracketCompletionClass * klass) {
	vtg_bracket_completion_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VtgBracketCompletionPrivate));
	G_OBJECT_CLASS (klass)->get_property = vtg_bracket_completion_get_property;
	G_OBJECT_CLASS (klass)->set_property = vtg_bracket_completion_set_property;
	G_OBJECT_CLASS (klass)->constructor = vtg_bracket_completion_constructor;
	G_OBJECT_CLASS (klass)->finalize = vtg_bracket_completion_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_BRACKET_COMPLETION_PLUGIN, g_param_spec_object ("plugin", "plugin", "plugin", VTG_TYPE_PLUGIN, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_BRACKET_COMPLETION_VIEW, g_param_spec_object ("view", "view", "view", GEDIT_TYPE_VIEW, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void vtg_bracket_completion_instance_init (VtgBracketCompletion * self) {
	self->priv = VTG_BRACKET_COMPLETION_GET_PRIVATE (self);
	self->priv->tab_chars = g_strdup ("");
}


static void vtg_bracket_completion_finalize (GObject* obj) {
	VtgBracketCompletion * self;
	self = VTG_BRACKET_COMPLETION (obj);
	(self->priv->_plugin == NULL) ? NULL : (self->priv->_plugin = (g_object_unref (self->priv->_plugin), NULL));
	(self->priv->_view == NULL) ? NULL : (self->priv->_view = (g_object_unref (self->priv->_view), NULL));
	self->priv->tab_chars = (g_free (self->priv->tab_chars), NULL);
	G_OBJECT_CLASS (vtg_bracket_completion_parent_class)->finalize (obj);
}


static GType vtg_bracket_completion_type_id = 0;
GType vtg_bracket_completion_get_type (void) {
	return vtg_bracket_completion_type_id;
}


GType vtg_bracket_completion_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgBracketCompletionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_bracket_completion_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgBracketCompletion), 0, (GInstanceInitFunc) vtg_bracket_completion_instance_init, NULL };
	vtg_bracket_completion_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgBracketCompletion", &g_define_type_info, 0);
	return vtg_bracket_completion_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




