/*
 *  vtgprojectmanagerview.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgprojectmanagerview.h"
#include <stdlib.h>
#include <string.h>
#include <glib/gi18n-lib.h>
#include <gee/list.h>
#include <glib/gstdio.h>
#include <gee/iterable.h>
#include <gee/iterator.h>
#include <gedit/gedit-window.h>
#include <gedit/gedit-panel.h>
#include <config.h>
#include "vtgprojectmanagerprojectmodule.h"
#include "vtgprojectmanagerprojecttarget.h"
#include "vtgprojectmanagerpackagesdialog.h"
#include "vtgprojectmanagerprojectgroup.h"
#include "vtgutils.h"




struct _VtgProjectManagerViewPrivate {
	VtgPlugin* _plugin;
	GtkComboBox* _prjs_combo;
	GtkTreeView* _prj_view;
	gint _project_count;
	VtgProjectManagerProjectModule* _last_selected_module;
	VtgProjectManagerProjectTarget* _last_selected_target;
	GtkMenu* _popup_modules;
	guint _popup_modules_ui_id;
	char* _popup_modules_ui_def;
	GtkMenu* _popup_targets;
	guint _popup_targets_ui_id;
	char* _popup_targets_ui_def;
	GtkActionGroup* _actions;
	GtkVBox* _side_panel;
	VtgProjectManagerProject* _current_project;
};

#define VTG_PROJECT_MANAGER_VIEW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_PROJECT_MANAGER_TYPE_VIEW, VtgProjectManagerViewPrivate))
enum  {
	VTG_PROJECT_MANAGER_VIEW_DUMMY_PROPERTY,
	VTG_PROJECT_MANAGER_VIEW_CURRENT_PROJECT,
	VTG_PROJECT_MANAGER_VIEW_PLUGIN
};
static void _vtg_project_manager_view_on_packages_edit_gtk_action_callback (GtkAction* action, gpointer self);
static void _vtg_project_manager_view_on_packages_open_configure_gtk_action_callback (GtkAction* action, gpointer self);
static void _vtg_project_manager_view_on_target_open_makefile_gtk_action_callback (GtkAction* action, gpointer self);
static void _g_list_free_gtk_tree_path_free (GList* self);
static void vtg_project_manager_view_show_packages_dialog (VtgProjectManagerView* self, VtgProjectManagerProjectModule* module);
static void vtg_project_manager_view_update_view (VtgProjectManagerView* self, const char* project_name);
static void vtg_project_manager_view_on_current_project_updated (VtgProjectManagerView* self, VtgProjectManagerProject* sender);
static void vtg_project_manager_view_on_packages_edit (VtgProjectManagerView* self, GtkAction* action);
static void vtg_project_manager_view_on_packages_open_configure (VtgProjectManagerView* self, GtkAction* action);
static void vtg_project_manager_view_on_target_open_makefile (VtgProjectManagerView* self, GtkAction* action);
static void _vtg_project_manager_view_on_current_project_updated_vtg_project_manager_project_updated (VtgProjectManagerProject* _sender, gpointer self);
static void vtg_project_manager_view_set_plugin (VtgProjectManagerView* self, VtgPlugin* value);
static void _vtg_project_manager_view_on_project_combobox_changed_gtk_combo_box_changed (GtkComboBox* _sender, gpointer self);
static void _vtg_project_manager_view_on_project_view_row_activated_gtk_tree_view_row_activated (GtkTreeView* _sender, const GtkTreePath* path, GtkTreeViewColumn* column, gpointer self);
static gboolean _vtg_project_manager_view_on_project_view_button_press_gtk_widget_button_press_event (GtkTreeView* _sender, const GdkEventButton* event, gpointer self);
static GObject * vtg_project_manager_view_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer vtg_project_manager_view_parent_class = NULL;
static void vtg_project_manager_view_finalize (GObject* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);

static const GtkActionEntry VTG_PROJECT_MANAGER_VIEW__action_entries[] = {{"packages-edit", GTK_STOCK_ADD, N_ ("Add/Remove Packages..."), "<control><shift>P", N_ ("Manage packages references"), (GCallback) _vtg_project_manager_view_on_packages_edit_gtk_action_callback}, {"packages-open-configure", GTK_STOCK_OPEN, N_ ("Open configure file..."), "<control><shift>C", N_ ("Open configure.ac file"), (GCallback) _vtg_project_manager_view_on_packages_open_configure_gtk_action_callback}, {"target-open-makefile", GTK_STOCK_OPEN, N_ ("Open makefile"), "<control><shift>M", N_ ("Open makefile.am file"), (GCallback) _vtg_project_manager_view_on_target_open_makefile_gtk_action_callback}};


static void _vtg_project_manager_view_on_packages_edit_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_view_on_packages_edit (self, action);
}


static void _vtg_project_manager_view_on_packages_open_configure_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_view_on_packages_open_configure (self, action);
}


static void _vtg_project_manager_view_on_target_open_makefile_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_view_on_target_open_makefile (self, action);
}


VtgProjectManagerView* vtg_project_manager_view_construct (GType object_type, VtgPlugin* plugin) {
	GParameter * __params;
	GParameter * __params_it;
	VtgProjectManagerView * self;
	g_return_val_if_fail (plugin != NULL, NULL);
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
	__params_it->name = "plugin";
	g_value_init (&__params_it->value, VTG_TYPE_PLUGIN);
	g_value_set_object (&__params_it->value, plugin);
	__params_it++;
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


VtgProjectManagerView* vtg_project_manager_view_new (VtgPlugin* plugin) {
	return vtg_project_manager_view_construct (VTG_PROJECT_MANAGER_TYPE_VIEW, plugin);
}


void vtg_project_manager_view_add_project (VtgProjectManagerView* self, VtgProjectManagerProject* project) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (project != NULL);
	gtk_combo_box_append_text (self->priv->_prjs_combo, project->name);
	gtk_combo_box_set_active (self->priv->_prjs_combo, self->priv->_project_count);
	self->priv->_project_count++;
}


void vtg_project_manager_view_remove_project (VtgProjectManagerView* self, VtgProjectManagerProject* project) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (project != NULL);
	gtk_combo_box_remove_text (self->priv->_prjs_combo, self->priv->_project_count - 1);
	self->priv->_project_count--;
	if (self->priv->_project_count > 0) {
		gtk_combo_box_set_active (self->priv->_prjs_combo, self->priv->_project_count - 1);
	} else {
		vtg_project_manager_view_update_view (self, NULL);
	}
}


void vtg_project_manager_view_on_project_view_row_activated (VtgProjectManagerView* self, GtkWidget* sender, const GtkTreePath* path, GtkTreeViewColumn* column) {
	GtkTreeView* _tmp0;
	GtkTreeView* tw;
	GtkTreeModel* _tmp1;
	GtkTreeModel* model;
	GtkTreeIter iter = {0};
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	g_return_if_fail (path != NULL);
	g_return_if_fail (column != NULL);
	_tmp0 = NULL;
	tw = (_tmp0 = GTK_TREE_VIEW (sender), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	_tmp1 = NULL;
	model = (_tmp1 = gtk_tree_view_get_model (tw), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
	if (gtk_tree_model_get_iter (model, &iter, path)) {
		char* name;
		char* id;
		char* file;
		gboolean _tmp2;
		name = NULL;
		id = NULL;
		gtk_tree_model_get (model, &iter, 1, &name, 2, &id, -1);
		file = vtg_string_utils_replace (id, "file://", "");
		_tmp2 = FALSE;
		if (name != NULL) {
			_tmp2 = g_file_test (file, G_FILE_TEST_EXISTS);
		} else {
			_tmp2 = FALSE;
		}
		/*HACK*/
		if (_tmp2) {
			GeditTab* _tmp3;
			_tmp3 = NULL;
			_tmp3 = vtg_plugin_activate_uri (self->priv->_plugin, id, 0, 0);
			(_tmp3 == NULL) ? NULL : (_tmp3 = (g_object_unref (_tmp3), NULL));
		}
		name = (g_free (name), NULL);
		id = (g_free (id), NULL);
		file = (g_free (file), NULL);
	}
	(tw == NULL) ? NULL : (tw = (g_object_unref (tw), NULL));
	(model == NULL) ? NULL : (model = (g_object_unref (model), NULL));
}


static void _g_list_free_gtk_tree_path_free (GList* self) {
	g_list_foreach (self, (GFunc) gtk_tree_path_free, NULL);
	g_list_free (self);
}


gboolean vtg_project_manager_view_on_project_view_button_press (VtgProjectManagerView* self, GtkWidget* sender, const GdkEventButton* event) {
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (sender != NULL, FALSE);
	if ((*event).button == 3) {
		GtkTreeModel* model;
		GList* rows;
		model = NULL;
		rows = gtk_tree_selection_get_selected_rows (gtk_tree_view_get_selection (self->priv->_prj_view), &model);
		if (g_list_length (rows) == 1) {
			GtkTreeIter iter = {0};
			GObject* obj;
			const GtkTreePath* path;
			char* name;
			obj = NULL;
			path = (const GtkTreePath*) g_list_nth_data (rows, (guint) 0);
			name = NULL;
			gtk_tree_model_get_iter (model, &iter, path);
			gtk_tree_model_get (model, &iter, 1, &name, 3, &obj, -1);
			if (VTG_PROJECT_MANAGER_IS_PROJECT_MODULE (obj)) {
				VtgProjectManagerProjectModule* _tmp1;
				VtgProjectManagerProjectModule* _tmp0;
				_tmp1 = NULL;
				_tmp0 = NULL;
				self->priv->_last_selected_module = (_tmp1 = (_tmp0 = VTG_PROJECT_MANAGER_PROJECT_MODULE (obj), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (self->priv->_last_selected_module == NULL) ? NULL : (self->priv->_last_selected_module = (g_object_unref (self->priv->_last_selected_module), NULL)), _tmp1);
				gtk_menu_popup (self->priv->_popup_modules, NULL, NULL, NULL, NULL, (*event).button, (*event).time);
			} else {
				if (VTG_PROJECT_MANAGER_IS_PROJECT_TARGET (obj)) {
					VtgProjectManagerProjectTarget* _tmp3;
					VtgProjectManagerProjectTarget* _tmp2;
					_tmp3 = NULL;
					_tmp2 = NULL;
					self->priv->_last_selected_target = (_tmp3 = (_tmp2 = VTG_PROJECT_MANAGER_PROJECT_TARGET (obj), (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2)), (self->priv->_last_selected_target == NULL) ? NULL : (self->priv->_last_selected_target = (g_object_unref (self->priv->_last_selected_target), NULL)), _tmp3);
					gtk_menu_popup (self->priv->_popup_targets, NULL, NULL, NULL, NULL, (*event).button, (*event).time);
				}
			}
			(obj == NULL) ? NULL : (obj = (g_object_unref (obj), NULL));
			name = (g_free (name), NULL);
		}
		(rows == NULL) ? NULL : (rows = (_g_list_free_gtk_tree_path_free (rows), NULL));
	}
	return FALSE;
}


void vtg_project_manager_view_on_project_combobox_changed (VtgProjectManagerView* self, GtkWidget* sender) {
	const char* _tmp0;
	char* project_name;
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	_tmp0 = NULL;
	project_name = (_tmp0 = gtk_combo_box_get_active_text ((GTK_COMBO_BOX (sender))), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
	vtg_project_manager_view_update_view (self, project_name);
	project_name = (g_free (project_name), NULL);
}


static void vtg_project_manager_view_show_packages_dialog (VtgProjectManagerView* self, VtgProjectManagerProjectModule* module) {
	VtgProjectManagerPackagesDialog* dialog;
	g_return_if_fail (self != NULL);
	g_return_if_fail (module != NULL);
	dialog = vtg_project_manager_packages_dialog_new (module, vtg_project_manager_view_get_current_project (self)->modules);
	vtg_project_manager_packages_dialog_show (dialog);
	(dialog == NULL) ? NULL : (dialog = (vtg_project_manager_packages_dialog_unref (dialog), NULL));
}


static void vtg_project_manager_view_update_view (VtgProjectManagerView* self, const char* project_name) {
	VtgProjectManagerProject* prj;
	g_return_if_fail (self != NULL);
	prj = NULL;
	if (project_name != NULL) {
		/*find project*/
		{
			GeeIterator* _item_it;
			/*find project*/
			_item_it = gee_iterable_iterator ((GeeIterable*) vtg_plugin_get_projects (self->priv->_plugin));
			/*find project*/
			while (gee_iterator_next (_item_it)) {
				VtgProjectDescriptor* item;
				/*find project*/
				item = (VtgProjectDescriptor*) gee_iterator_get (_item_it);
				if (_vala_strcmp0 (item->project->name, project_name) == 0) {
					VtgProjectManagerProject* _tmp1;
					VtgProjectManagerProject* _tmp0;
					_tmp1 = NULL;
					_tmp0 = NULL;
					prj = (_tmp1 = (_tmp0 = item->project, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (prj == NULL) ? NULL : (prj = (g_object_unref (prj), NULL)), _tmp1);
					(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
					break;
				}
				(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
			}
			(_item_it == NULL) ? NULL : (_item_it = (gee_collection_object_unref (_item_it), NULL));
		}
	}
	vtg_project_manager_view_set_current_project (self, prj);
	(prj == NULL) ? NULL : (prj = (g_object_unref (prj), NULL));
}


static void vtg_project_manager_view_on_current_project_updated (VtgProjectManagerView* self, VtgProjectManagerProject* sender) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	gtk_tree_view_set_model (self->priv->_prj_view, vtg_project_manager_project_get_model (sender));
	gtk_tree_view_expand_all (self->priv->_prj_view);
}


static void vtg_project_manager_view_on_packages_edit (VtgProjectManagerView* self, GtkAction* action) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	vtg_project_manager_view_show_packages_dialog (self, self->priv->_last_selected_module);
}


static void vtg_project_manager_view_on_packages_open_configure (VtgProjectManagerView* self, GtkAction* action) {
	char* file;
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	g_return_if_fail (self->priv->_last_selected_module != NULL);
	file = g_build_filename (vtg_project_manager_project_module_get_project (self->priv->_last_selected_module)->filename, "configure.ac", NULL);
	if (g_file_test (file, G_FILE_TEST_EXISTS)) {
		GeditTab* _tmp1;
		char* _tmp0;
		_tmp1 = NULL;
		_tmp0 = NULL;
		_tmp1 = vtg_plugin_activate_uri (self->priv->_plugin, _tmp0 = g_strdup_printf ("file://%s", file), 0, 0);
		(_tmp1 == NULL) ? NULL : (_tmp1 = (g_object_unref (_tmp1), NULL));
		_tmp0 = (g_free (_tmp0), NULL);
	}
	file = (g_free (file), NULL);
}


static void vtg_project_manager_view_on_target_open_makefile (VtgProjectManagerView* self, GtkAction* action) {
	char* file;
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	g_return_if_fail (self->priv->_last_selected_target != NULL);
	file = g_build_filename (self->priv->_last_selected_target->group->project->filename, self->priv->_last_selected_target->group->id, "Makefile.am", NULL);
	if (g_file_test (file, G_FILE_TEST_EXISTS)) {
		GeditTab* _tmp1;
		char* _tmp0;
		_tmp1 = NULL;
		_tmp0 = NULL;
		_tmp1 = vtg_plugin_activate_uri (self->priv->_plugin, _tmp0 = g_strdup_printf ("file://%s", file), 0, 0);
		(_tmp1 == NULL) ? NULL : (_tmp1 = (g_object_unref (_tmp1), NULL));
		_tmp0 = (g_free (_tmp0), NULL);
	}
	file = (g_free (file), NULL);
}


VtgProjectManagerProject* vtg_project_manager_view_get_current_project (VtgProjectManagerView* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_current_project;
}


static void _vtg_project_manager_view_on_current_project_updated_vtg_project_manager_project_updated (VtgProjectManagerProject* _sender, gpointer self) {
	vtg_project_manager_view_on_current_project_updated (self, _sender);
}


void vtg_project_manager_view_set_current_project (VtgProjectManagerView* self, VtgProjectManagerProject* value) {
	VtgProjectManagerProject* _tmp3;
	VtgProjectManagerProject* _tmp2;
	g_return_if_fail (self != NULL);
	if (self->priv->_current_project != NULL) {
		guint _tmp1;
		g_signal_handlers_disconnect_matched (self->priv->_current_project, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("updated", VTG_PROJECT_MANAGER_TYPE_PROJECT, &_tmp1, NULL, FALSE), _tmp1), 0, NULL, (GCallback) _vtg_project_manager_view_on_current_project_updated_vtg_project_manager_project_updated, self);
	}
	_tmp3 = NULL;
	_tmp2 = NULL;
	self->priv->_current_project = (_tmp3 = (_tmp2 = value, (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2)), (self->priv->_current_project == NULL) ? NULL : (self->priv->_current_project = (g_object_unref (self->priv->_current_project), NULL)), _tmp3);
	if (self->priv->_current_project != NULL) {
		g_signal_connect_object (self->priv->_current_project, "updated", (GCallback) _vtg_project_manager_view_on_current_project_updated_vtg_project_manager_project_updated, self, 0);
		gtk_tree_view_set_model (self->priv->_prj_view, vtg_project_manager_project_get_model (self->priv->_current_project));
		gtk_tree_view_expand_all (self->priv->_prj_view);
	} else {
		gtk_tree_view_set_model (self->priv->_prj_view, NULL);
	}
	g_object_notify ((GObject *) self, "current-project");
}


static void vtg_project_manager_view_set_plugin (VtgProjectManagerView* self, VtgPlugin* value) {
	VtgPlugin* _tmp1;
	VtgPlugin* _tmp0;
	g_return_if_fail (self != NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->priv->_plugin = (_tmp1 = (_tmp0 = value, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (self->priv->_plugin == NULL) ? NULL : (self->priv->_plugin = (g_object_unref (self->priv->_plugin), NULL)), _tmp1);
	g_object_notify ((GObject *) self, "plugin");
}


static void _vtg_project_manager_view_on_project_combobox_changed_gtk_combo_box_changed (GtkComboBox* _sender, gpointer self) {
	vtg_project_manager_view_on_project_combobox_changed (self, _sender);
}


static void _vtg_project_manager_view_on_project_view_row_activated_gtk_tree_view_row_activated (GtkTreeView* _sender, const GtkTreePath* path, GtkTreeViewColumn* column, gpointer self) {
	vtg_project_manager_view_on_project_view_row_activated (self, _sender, path, column);
}


static gboolean _vtg_project_manager_view_on_project_view_button_press_gtk_widget_button_press_event (GtkTreeView* _sender, const GdkEventButton* event, gpointer self) {
	return vtg_project_manager_view_on_project_view_button_press (self, _sender, event);
}


static GObject * vtg_project_manager_view_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	VtgProjectManagerViewClass * klass;
	GObjectClass * parent_class;
	VtgProjectManagerView * self;
	GError * inner_error;
	klass = VTG_PROJECT_MANAGER_VIEW_CLASS (g_type_class_peek (VTG_PROJECT_MANAGER_TYPE_VIEW));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VTG_PROJECT_MANAGER_VIEW (obj);
	inner_error = NULL;
	{
		GeditPanel* _tmp0;
		GeditPanel* panel;
		GtkVBox* _tmp1;
		GtkComboBox* _tmp2;
		GtkTreeView* _tmp3;
		GtkCellRenderer* renderer;
		GtkTreeViewColumn* column;
		GtkCellRenderer* _tmp4;
		GtkScrolledWindow* scroll;
		GtkActionGroup* _tmp5;
		GtkUIManager* _tmp6;
		GtkUIManager* manager;
		_tmp0 = NULL;
		panel = (_tmp0 = gedit_window_get_side_panel (vtg_plugin_get_gedit_window (self->priv->_plugin)), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
		_tmp1 = NULL;
		self->priv->_side_panel = (_tmp1 = g_object_ref_sink ((GtkVBox*) gtk_vbox_new (FALSE, 8)), (self->priv->_side_panel == NULL) ? NULL : (self->priv->_side_panel = (g_object_unref (self->priv->_side_panel), NULL)), _tmp1);
		_tmp2 = NULL;
		self->priv->_prjs_combo = (_tmp2 = g_object_ref_sink ((GtkComboBox*) gtk_combo_box_new_text ()), (self->priv->_prjs_combo == NULL) ? NULL : (self->priv->_prjs_combo = (g_object_unref (self->priv->_prjs_combo), NULL)), _tmp2);
		g_signal_connect_object (self->priv->_prjs_combo, "changed", (GCallback) _vtg_project_manager_view_on_project_combobox_changed_gtk_combo_box_changed, self, 0);
		_tmp3 = NULL;
		self->priv->_prj_view = (_tmp3 = g_object_ref_sink ((GtkTreeView*) gtk_tree_view_new ()), (self->priv->_prj_view == NULL) ? NULL : (self->priv->_prj_view = (g_object_unref (self->priv->_prj_view), NULL)), _tmp3);
		renderer = (GtkCellRenderer*) g_object_ref_sink ((GtkCellRendererPixbuf*) gtk_cell_renderer_pixbuf_new ());
		column = g_object_ref_sink (gtk_tree_view_column_new ());
		gtk_cell_layout_pack_start ((GtkCellLayout*) column, renderer, FALSE);
		gtk_cell_layout_add_attribute ((GtkCellLayout*) column, renderer, "stock-id", 0);
		_tmp4 = NULL;
		renderer = (_tmp4 = (GtkCellRenderer*) g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ()), (renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL)), _tmp4);
		gtk_cell_layout_pack_start ((GtkCellLayout*) column, renderer, TRUE);
		gtk_cell_layout_add_attribute ((GtkCellLayout*) column, renderer, "text", 1);
		gtk_tree_view_append_column (self->priv->_prj_view, column);
		gtk_tree_view_set_headers_visible (self->priv->_prj_view, FALSE);
		g_signal_connect_object (self->priv->_prj_view, "row-activated", (GCallback) _vtg_project_manager_view_on_project_view_row_activated_gtk_tree_view_row_activated, self, 0);
		g_signal_connect_object ((GtkWidget*) self->priv->_prj_view, "button-press-event", (GCallback) _vtg_project_manager_view_on_project_view_button_press_gtk_widget_button_press_event, self, 0);
		scroll = g_object_ref_sink ((GtkScrolledWindow*) gtk_scrolled_window_new (NULL, NULL));
		gtk_container_add ((GtkContainer*) scroll, (GtkWidget*) self->priv->_prj_view);
		gtk_box_pack_start ((GtkBox*) self->priv->_side_panel, (GtkWidget*) self->priv->_prjs_combo, FALSE, FALSE, (guint) 4);
		gtk_box_pack_start ((GtkBox*) self->priv->_side_panel, (GtkWidget*) scroll, TRUE, TRUE, (guint) 4);
		gtk_widget_show_all ((GtkWidget*) self->priv->_side_panel);
		gedit_panel_add_item (panel, (GtkWidget*) self->priv->_side_panel, _ ("Projects"), NULL);
		gedit_panel_activate_item (panel, (GtkWidget*) self->priv->_side_panel);
		self->priv->_project_count = 0;
		_tmp5 = NULL;
		self->priv->_actions = (_tmp5 = gtk_action_group_new ("ProjectManagerActionGroup"), (self->priv->_actions == NULL) ? NULL : (self->priv->_actions = (g_object_unref (self->priv->_actions), NULL)), _tmp5);
		gtk_action_group_set_translation_domain (self->priv->_actions, GETTEXT_PACKAGE);
		gtk_action_group_add_actions (self->priv->_actions, VTG_PROJECT_MANAGER_VIEW__action_entries, G_N_ELEMENTS (VTG_PROJECT_MANAGER_VIEW__action_entries), self);
		_tmp6 = NULL;
		manager = (_tmp6 = gedit_window_get_ui_manager (vtg_plugin_get_gedit_window (self->priv->_plugin)), (_tmp6 == NULL) ? NULL : g_object_ref (_tmp6));
		gtk_ui_manager_insert_action_group (manager, self->priv->_actions, -1);
		{
			guint _tmp7;
			GtkMenu* _tmp9;
			GtkMenu* _tmp8;
			guint _tmp10;
			GtkMenu* _tmp12;
			GtkMenu* _tmp11;
			_tmp7 = gtk_ui_manager_add_ui_from_string (manager, self->priv->_popup_modules_ui_def, (gssize) (-1), &inner_error);
			if (inner_error != NULL) {
				goto __catch10_g_error;
				goto __finally10;
			}
			self->priv->_popup_modules_ui_id = _tmp7;
			_tmp9 = NULL;
			_tmp8 = NULL;
			self->priv->_popup_modules = (_tmp9 = (_tmp8 = GTK_MENU (gtk_ui_manager_get_widget (manager, "/ProjectManagerPopupPackagesEdit")), (_tmp8 == NULL) ? NULL : g_object_ref (_tmp8)), (self->priv->_popup_modules == NULL) ? NULL : (self->priv->_popup_modules = (g_object_unref (self->priv->_popup_modules), NULL)), _tmp9);
			g_assert (self->priv->_popup_modules != NULL);
			_tmp10 = gtk_ui_manager_add_ui_from_string (manager, self->priv->_popup_targets_ui_def, (gssize) (-1), &inner_error);
			if (inner_error != NULL) {
				goto __catch10_g_error;
				goto __finally10;
			}
			self->priv->_popup_targets_ui_id = _tmp10;
			_tmp12 = NULL;
			_tmp11 = NULL;
			self->priv->_popup_targets = (_tmp12 = (_tmp11 = GTK_MENU (gtk_ui_manager_get_widget (manager, "/ProjectManagerPopupTargets")), (_tmp11 == NULL) ? NULL : g_object_ref (_tmp11)), (self->priv->_popup_targets == NULL) ? NULL : (self->priv->_popup_targets = (g_object_unref (self->priv->_popup_targets), NULL)), _tmp12);
			g_assert (self->priv->_popup_targets != NULL);
		}
		goto __finally10;
		__catch10_g_error:
		{
			GError * err;
			err = inner_error;
			inner_error = NULL;
			{
				g_warning ("vtgprojectmanagerview.vala:145: Error %s", err->message);
				(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
			}
		}
		__finally10:
		if (inner_error != NULL) {
			(panel == NULL) ? NULL : (panel = (g_object_unref (panel), NULL));
			(renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL));
			(column == NULL) ? NULL : (column = (g_object_unref (column), NULL));
			(scroll == NULL) ? NULL : (scroll = (g_object_unref (scroll), NULL));
			(manager == NULL) ? NULL : (manager = (g_object_unref (manager), NULL));
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
			g_clear_error (&inner_error);
		}
		/*default empty project*/
		gtk_combo_box_append_text (self->priv->_prjs_combo, _ ("(no project opened - default)"));
		gtk_combo_box_set_active (self->priv->_prjs_combo, 0);
		self->priv->_project_count++;
		(panel == NULL) ? NULL : (panel = (g_object_unref (panel), NULL));
		(renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL));
		(column == NULL) ? NULL : (column = (g_object_unref (column), NULL));
		(scroll == NULL) ? NULL : (scroll = (g_object_unref (scroll), NULL));
		(manager == NULL) ? NULL : (manager = (g_object_unref (manager), NULL));
	}
	return obj;
}


static void vtg_project_manager_view_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	VtgProjectManagerView * self;
	gpointer boxed;
	self = VTG_PROJECT_MANAGER_VIEW (object);
	switch (property_id) {
		case VTG_PROJECT_MANAGER_VIEW_CURRENT_PROJECT:
		g_value_set_object (value, vtg_project_manager_view_get_current_project (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_project_manager_view_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	VtgProjectManagerView * self;
	self = VTG_PROJECT_MANAGER_VIEW (object);
	switch (property_id) {
		case VTG_PROJECT_MANAGER_VIEW_CURRENT_PROJECT:
		vtg_project_manager_view_set_current_project (self, g_value_get_object (value));
		break;
		case VTG_PROJECT_MANAGER_VIEW_PLUGIN:
		vtg_project_manager_view_set_plugin (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_project_manager_view_class_init (VtgProjectManagerViewClass * klass) {
	vtg_project_manager_view_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VtgProjectManagerViewPrivate));
	G_OBJECT_CLASS (klass)->get_property = vtg_project_manager_view_get_property;
	G_OBJECT_CLASS (klass)->set_property = vtg_project_manager_view_set_property;
	G_OBJECT_CLASS (klass)->constructor = vtg_project_manager_view_constructor;
	G_OBJECT_CLASS (klass)->finalize = vtg_project_manager_view_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PROJECT_MANAGER_VIEW_CURRENT_PROJECT, g_param_spec_object ("current-project", "current-project", "current-project", VTG_PROJECT_MANAGER_TYPE_PROJECT, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PROJECT_MANAGER_VIEW_PLUGIN, g_param_spec_object ("plugin", "plugin", "plugin", VTG_TYPE_PLUGIN, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void vtg_project_manager_view_instance_init (VtgProjectManagerView * self) {
	self->priv = VTG_PROJECT_MANAGER_VIEW_GET_PRIVATE (self);
	self->priv->_plugin = NULL;
	self->priv->_project_count = 0;
	self->priv->_popup_modules_ui_def = g_strdup ("\n                                        <ui>\n                                        <popup name='ProjectManagerPopupPackagesEdit'>\n                                            <menuitem action='packages-edit'/>\n                                            <menuitem action='packages-open-configure'/>\n                                        </popup>\n                                        </ui>");
	self->priv->_popup_targets_ui_def = g_strdup ("\n                                        <ui>\n                                        <popup name='ProjectManagerPopupTargets'>\n                                            <menuitem action='target-open-makefile'/>\n                                        </popup>\n                                        </ui>");
	self->priv->_current_project = NULL;
}


static void vtg_project_manager_view_finalize (GObject* obj) {
	VtgProjectManagerView * self;
	self = VTG_PROJECT_MANAGER_VIEW (obj);
	{
		GtkUIManager* _tmp13;
		GtkUIManager* manager;
		GeditPanel* _tmp14;
		GeditPanel* panel;
		_tmp13 = NULL;
		manager = (_tmp13 = gedit_window_get_ui_manager (vtg_plugin_get_gedit_window (self->priv->_plugin)), (_tmp13 == NULL) ? NULL : g_object_ref (_tmp13));
		gtk_ui_manager_remove_action_group (manager, self->priv->_actions);
		_tmp14 = NULL;
		panel = (_tmp14 = gedit_window_get_side_panel (vtg_plugin_get_gedit_window (self->priv->_plugin)), (_tmp14 == NULL) ? NULL : g_object_ref (_tmp14));
		gedit_panel_remove_item (panel, (GtkWidget*) self->priv->_side_panel);
		(manager == NULL) ? NULL : (manager = (g_object_unref (manager), NULL));
		(panel == NULL) ? NULL : (panel = (g_object_unref (panel), NULL));
	}
	(self->priv->_plugin == NULL) ? NULL : (self->priv->_plugin = (g_object_unref (self->priv->_plugin), NULL));
	(self->priv->_prjs_combo == NULL) ? NULL : (self->priv->_prjs_combo = (g_object_unref (self->priv->_prjs_combo), NULL));
	(self->priv->_prj_view == NULL) ? NULL : (self->priv->_prj_view = (g_object_unref (self->priv->_prj_view), NULL));
	(self->priv->_last_selected_module == NULL) ? NULL : (self->priv->_last_selected_module = (g_object_unref (self->priv->_last_selected_module), NULL));
	(self->priv->_last_selected_target == NULL) ? NULL : (self->priv->_last_selected_target = (g_object_unref (self->priv->_last_selected_target), NULL));
	(self->priv->_popup_modules == NULL) ? NULL : (self->priv->_popup_modules = (g_object_unref (self->priv->_popup_modules), NULL));
	self->priv->_popup_modules_ui_def = (g_free (self->priv->_popup_modules_ui_def), NULL);
	(self->priv->_popup_targets == NULL) ? NULL : (self->priv->_popup_targets = (g_object_unref (self->priv->_popup_targets), NULL));
	self->priv->_popup_targets_ui_def = (g_free (self->priv->_popup_targets_ui_def), NULL);
	(self->priv->_actions == NULL) ? NULL : (self->priv->_actions = (g_object_unref (self->priv->_actions), NULL));
	(self->priv->_side_panel == NULL) ? NULL : (self->priv->_side_panel = (g_object_unref (self->priv->_side_panel), NULL));
	(self->priv->_current_project == NULL) ? NULL : (self->priv->_current_project = (g_object_unref (self->priv->_current_project), NULL));
	G_OBJECT_CLASS (vtg_project_manager_view_parent_class)->finalize (obj);
}


static GType vtg_project_manager_view_type_id = 0;
GType vtg_project_manager_view_get_type (void) {
	return vtg_project_manager_view_type_id;
}


GType vtg_project_manager_view_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgProjectManagerViewClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_project_manager_view_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgProjectManagerView), 0, (GInstanceInitFunc) vtg_project_manager_view_instance_init, NULL };
	vtg_project_manager_view_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgProjectManagerView", &g_define_type_info, 0);
	return vtg_project_manager_view_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




