/*
 *  vtgutils.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgutils.h"
#include <config.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <gee/arraylist.h>
#include <gee/collection.h>
#include "vtgprojectmanagerprojectpackage.h"
#include <gobject/gvaluecollector.h>




static glong string_get_length (const char* self);
static char* string_substring (const char* self, glong offset, glong len);
enum  {
	VTG_CACHES_DUMMY_PROPERTY
};
static GtkListStore* vtg_caches__build_cache = NULL;
static GtkListStore* vtg_caches__compile_cache = NULL;
static GtkListStore* vtg_caches__configure_cache = NULL;
static GtkListStore* vtg_caches__executer_cache = NULL;
#define VTG_CACHES_CACHE_LIMIT 20
static gpointer vtg_caches_parent_class = NULL;
static void vtg_caches_finalize (VtgCaches* obj);
enum  {
	VTG_UTILS_DUMMY_PROPERTY
};
static gboolean vtg_utils__initialized = FALSE;
static GscProposal** vtg_utils__proposals = NULL;
static gint vtg_utils__proposals_length1 = 0;
static gint vtg_utils__proposals_size = 0;
static GeeList* vtg_utils__available_packages = NULL;
static GtkBuilder* vtg_utils__builder = NULL;
static void vtg_utils_initialize (void);
static void _g_list_free_g_free (GList* self);
static void vtg_utils_initialize_packages_cache (void);
static gpointer vtg_utils_parent_class = NULL;
static void vtg_utils_finalize (GObject* obj);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);



static glong string_get_length (const char* self) {
	g_return_val_if_fail (self != NULL, 0L);
	return g_utf8_strlen (self, -1);
}


static char* string_substring (const char* self, glong offset, glong len) {
	glong string_length;
	const char* start;
	g_return_val_if_fail (self != NULL, NULL);
	string_length = g_utf8_strlen (self, -1);
	if (offset < 0) {
		offset = string_length + offset;
		g_return_val_if_fail (offset >= 0, NULL);
	} else {
		g_return_val_if_fail (offset <= string_length, NULL);
	}
	if (len < 0) {
		len = string_length - offset;
	}
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
	start = g_utf8_offset_to_pointer (self, offset);
	return g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
}


void vtg_interaction_error_message (const char* message, GError* err) {
	GtkMessageDialog* dialog;
	g_return_if_fail (message != NULL);
	dialog = g_object_ref_sink ((GtkMessageDialog*) gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, message, NULL));
	g_object_set (dialog, "secondary-text", err->message, NULL);
	gtk_dialog_run ((GtkDialog*) dialog);
	gtk_object_destroy ((GtkObject*) dialog);
	(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
}


GtkListStore* vtg_caches_get_build_cache (void) {
	GtkListStore* _tmp1;
	if (vtg_caches__build_cache == NULL) {
		GtkListStore* _tmp0;
		_tmp0 = NULL;
		vtg_caches__build_cache = (_tmp0 = gtk_list_store_new (1, G_TYPE_STRING, NULL), (vtg_caches__build_cache == NULL) ? NULL : (vtg_caches__build_cache = (g_object_unref (vtg_caches__build_cache), NULL)), _tmp0);
	}
	_tmp1 = NULL;
	return (_tmp1 = vtg_caches__build_cache, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
}


GtkListStore* vtg_caches_get_compile_cache (void) {
	GtkListStore* _tmp1;
	if (vtg_caches__compile_cache == NULL) {
		GtkListStore* _tmp0;
		_tmp0 = NULL;
		vtg_caches__compile_cache = (_tmp0 = gtk_list_store_new (1, G_TYPE_STRING, NULL), (vtg_caches__compile_cache == NULL) ? NULL : (vtg_caches__compile_cache = (g_object_unref (vtg_caches__compile_cache), NULL)), _tmp0);
	}
	_tmp1 = NULL;
	return (_tmp1 = vtg_caches__compile_cache, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
}


GtkListStore* vtg_caches_get_configure_cache (void) {
	GtkListStore* _tmp1;
	if (vtg_caches__configure_cache == NULL) {
		GtkListStore* _tmp0;
		_tmp0 = NULL;
		vtg_caches__configure_cache = (_tmp0 = gtk_list_store_new (1, G_TYPE_STRING, NULL), (vtg_caches__configure_cache == NULL) ? NULL : (vtg_caches__configure_cache = (g_object_unref (vtg_caches__configure_cache), NULL)), _tmp0);
	}
	_tmp1 = NULL;
	return (_tmp1 = vtg_caches__configure_cache, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
}


GtkListStore* vtg_caches_get_executer_cache (void) {
	GtkListStore* _tmp1;
	if (vtg_caches__executer_cache == NULL) {
		GtkListStore* _tmp0;
		_tmp0 = NULL;
		vtg_caches__executer_cache = (_tmp0 = gtk_list_store_new (1, G_TYPE_STRING, NULL), (vtg_caches__executer_cache == NULL) ? NULL : (vtg_caches__executer_cache = (g_object_unref (vtg_caches__executer_cache), NULL)), _tmp0);
	}
	_tmp1 = NULL;
	return (_tmp1 = vtg_caches__executer_cache, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
}


gboolean vtg_caches_cache_contains (GtkListStore* cache, const char* data) {
	GtkTreeIter iter = {0};
	gboolean found;
	g_return_val_if_fail (cache != NULL, FALSE);
	g_return_val_if_fail (data != NULL, FALSE);
	found = FALSE;
	if (gtk_tree_model_get_iter_first ((GtkTreeModel*) cache, &iter)) {
		do {
			char* tmp;
			tmp = NULL;
			gtk_tree_model_get ((GtkTreeModel*) cache, &iter, 0, &tmp, -1);
			if (_vala_strcmp0 (tmp, data) == 0) {
				found = TRUE;
				tmp = (g_free (tmp), NULL);
				break;
			}
			tmp = (g_free (tmp), NULL);
		} while (gtk_tree_model_iter_next ((GtkTreeModel*) cache, &iter));
	}
	return found;
}


void vtg_caches_cache_add (GtkListStore* cache, const char* data) {
	GtkTreeIter iter = {0};
	g_return_if_fail (cache != NULL);
	g_return_if_fail (data != NULL);
	if (vtg_caches_cache_count (cache) > VTG_CACHES_CACHE_LIMIT) {
		if (gtk_tree_model_get_iter_first ((GtkTreeModel*) cache, &iter)) {
			GtkTreeIter target;
			target = iter;
			while (gtk_tree_model_iter_next ((GtkTreeModel*) cache, &iter)) {
				target = iter;
			}
			gtk_list_store_remove (cache, &target);
		}
	}
	gtk_list_store_insert (cache, &iter, 0);
	gtk_list_store_set (cache, &iter, 0, data, -1);
}


gint vtg_caches_cache_count (GtkListStore* cache) {
	gint count;
	GtkTreeIter iter = {0};
	g_return_val_if_fail (cache != NULL, 0);
	count = 0;
	if (gtk_tree_model_get_iter_first ((GtkTreeModel*) cache, &iter)) {
		do {
			count++;
		} while (gtk_tree_model_iter_next ((GtkTreeModel*) cache, &iter));
	}
	return count;
}


VtgCaches* vtg_caches_construct (GType object_type) {
	VtgCaches* self;
	self = (VtgCaches*) g_type_create_instance (object_type);
	return self;
}


VtgCaches* vtg_caches_new (void) {
	return vtg_caches_construct (VTG_TYPE_CACHES);
}


static void vtg_value_caches_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void vtg_value_caches_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		vtg_caches_unref (value->data[0].v_pointer);
	}
}


static void vtg_value_caches_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = vtg_caches_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer vtg_value_caches_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* vtg_value_caches_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		VtgCaches* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = vtg_caches_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* vtg_value_caches_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	VtgCaches** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = vtg_caches_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* vtg_param_spec_caches (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	VtgParamSpecCaches* spec;
	g_return_val_if_fail (g_type_is_a (object_type, VTG_TYPE_CACHES), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer vtg_value_get_caches (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VTG_TYPE_CACHES), NULL);
	return value->data[0].v_pointer;
}


void vtg_value_set_caches (GValue* value, gpointer v_object) {
	VtgCaches* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VTG_TYPE_CACHES));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, VTG_TYPE_CACHES));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		vtg_caches_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		vtg_caches_unref (old);
	}
}


static void vtg_caches_class_init (VtgCachesClass * klass) {
	vtg_caches_parent_class = g_type_class_peek_parent (klass);
	VTG_CACHES_CLASS (klass)->finalize = vtg_caches_finalize;
}


static void vtg_caches_instance_init (VtgCaches * self) {
	self->ref_count = 1;
}


static void vtg_caches_finalize (VtgCaches* obj) {
	VtgCaches * self;
	self = VTG_CACHES (obj);
}


static GType vtg_caches_type_id = 0;
GType vtg_caches_get_type (void) {
	return vtg_caches_type_id;
}


GType vtg_caches_register_type (GTypeModule * module) {
	static const GTypeValueTable g_define_type_value_table = { vtg_value_caches_init, vtg_value_caches_free_value, vtg_value_caches_copy_value, vtg_value_caches_peek_pointer, "p", vtg_value_caches_collect_value, "p", vtg_value_caches_lcopy_value };
	static const GTypeInfo g_define_type_info = { sizeof (VtgCachesClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_caches_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgCaches), 0, (GInstanceInitFunc) vtg_caches_instance_init, &g_define_type_value_table };
	static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
	vtg_caches_type_id = g_type_register_fundamental (g_type_fundamental_next (), "VtgCaches", &g_define_type_info, &g_define_type_fundamental_info, 0);
	return vtg_caches_type_id;
}


gpointer vtg_caches_ref (gpointer instance) {
	VtgCaches* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void vtg_caches_unref (gpointer instance) {
	VtgCaches* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		VTG_CACHES_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


gboolean vtg_string_utils_is_null_or_empty (const char* data) {
	gboolean _tmp0;
	_tmp0 = FALSE;
	if (data == NULL) {
		_tmp0 = TRUE;
	} else {
		_tmp0 = _vala_strcmp0 (data, "") == 0;
	}
	return _tmp0;
}


char* vtg_string_utils_replace (const char* data, const char* search, const char* replace) {
	GError * inner_error;
	g_return_val_if_fail (data != NULL, NULL);
	g_return_val_if_fail (search != NULL, NULL);
	g_return_val_if_fail (replace != NULL, NULL);
	inner_error = NULL;
	{
		char* _tmp0;
		GRegex* _tmp1;
		GRegex* regex;
		char* _tmp2;
		char* _tmp3;
		_tmp0 = NULL;
		_tmp1 = NULL;
		regex = (_tmp1 = g_regex_new (_tmp0 = g_regex_escape_string (search, -1), 0, 0, &inner_error), _tmp0 = (g_free (_tmp0), NULL), _tmp1);
		if (inner_error != NULL) {
			if (inner_error->domain == G_REGEX_ERROR) {
				goto __catch35_g_regex_error;
			}
			goto __finally35;
		}
		_tmp2 = g_regex_replace_literal (regex, data, (glong) (-1), 0, replace, 0, &inner_error);
		if (inner_error != NULL) {
			(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
			if (inner_error->domain == G_REGEX_ERROR) {
				goto __catch35_g_regex_error;
			}
			goto __finally35;
		}
		_tmp3 = NULL;
		return (_tmp3 = _tmp2, (regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL)), _tmp3);
	}
	goto __finally35;
	__catch35_g_regex_error:
	{
		GError * e;
		e = inner_error;
		inner_error = NULL;
		{
			g_assert_not_reached ();
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally35:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return NULL;
	}
}


char* vtg_path_utils_normalize_path (const char* name) {
	gboolean _tmp0;
	char** _tmp6;
	gint name_parts_size;
	gint name_parts_length1;
	char** _tmp4;
	char* _tmp3;
	char** _tmp5;
	char** name_parts;
	char* last_item;
	char* target_name;
	gboolean _tmp13;
	char* _tmp16;
	g_return_val_if_fail (name != NULL, NULL);
	_tmp0 = FALSE;
	if (name == NULL) {
		_tmp0 = TRUE;
	} else {
		_tmp0 = string_get_length (name) < 2;
	}
	if (_tmp0) {
		const char* _tmp1;
		_tmp1 = NULL;
		return (_tmp1 = name, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1));
	}
	_tmp6 = NULL;
	_tmp4 = NULL;
	_tmp3 = NULL;
	_tmp5 = NULL;
	name_parts = (_tmp6 = (_tmp5 = _tmp4 = g_strsplit (_tmp3 = string_substring (name, (glong) 1, string_get_length (name) - 1), "/", 0), _tmp3 = (g_free (_tmp3), NULL), _tmp5), name_parts_length1 = _vala_array_length (_tmp4), name_parts_size = name_parts_length1, _tmp6);
	last_item = NULL;
	target_name = g_strdup ("");
	{
		char** item_collection;
		int item_collection_length1;
		int item_it;
		item_collection = name_parts;
		item_collection_length1 = name_parts_length1;
		for (item_it = 0; item_it < name_parts_length1; item_it = item_it + 1) {
			const char* _tmp12;
			char* item;
			_tmp12 = NULL;
			item = (_tmp12 = item_collection[item_it], (_tmp12 == NULL) ? NULL : g_strdup (_tmp12));
			{
				if (_vala_strcmp0 (item, "..") != 0) {
					char* _tmp10;
					const char* _tmp9;
					if (last_item != NULL) {
						char* _tmp8;
						char* _tmp7;
						_tmp8 = NULL;
						_tmp7 = NULL;
						target_name = (_tmp8 = g_strconcat (target_name, _tmp7 = (g_strconcat ("/", last_item, NULL)), NULL), target_name = (g_free (target_name), NULL), _tmp8);
						_tmp7 = (g_free (_tmp7), NULL);
					}
					_tmp10 = NULL;
					_tmp9 = NULL;
					last_item = (_tmp10 = (_tmp9 = item, (_tmp9 == NULL) ? NULL : g_strdup (_tmp9)), last_item = (g_free (last_item), NULL), _tmp10);
				} else {
					char* _tmp11;
					_tmp11 = NULL;
					last_item = (_tmp11 = NULL, last_item = (g_free (last_item), NULL), _tmp11);
				}
				item = (g_free (item), NULL);
			}
		}
	}
	_tmp13 = FALSE;
	if (last_item != NULL) {
		_tmp13 = _vala_strcmp0 (last_item, "..") != 0;
	} else {
		_tmp13 = FALSE;
	}
	if (_tmp13) {
		char* _tmp15;
		char* _tmp14;
		_tmp15 = NULL;
		_tmp14 = NULL;
		target_name = (_tmp15 = g_strconcat (target_name, _tmp14 = (g_strconcat ("/", last_item, NULL)), NULL), target_name = (g_free (target_name), NULL), _tmp15);
		_tmp14 = (g_free (_tmp14), NULL);
	}
	_tmp16 = NULL;
	return (_tmp16 = target_name, name_parts = (_vala_array_free (name_parts, name_parts_length1, (GDestroyNotify) g_free), NULL), last_item = (g_free (last_item), NULL), _tmp16);
}


gint vtg_path_utils_compare_vala_filenames (const char* filea, const char* fileb) {
	const char* _tmp0;
	char* dataa;
	const char* _tmp1;
	char* datab;
	gboolean _tmp2;
	gboolean _tmp4;
	gint _tmp6;
	g_return_val_if_fail (filea != NULL, 0);
	g_return_val_if_fail (fileb != NULL, 0);
	_tmp0 = NULL;
	dataa = (_tmp0 = filea, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
	_tmp1 = NULL;
	datab = (_tmp1 = fileb, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1));
	_tmp2 = FALSE;
	if (g_str_has_suffix (dataa, ".vala")) {
		_tmp2 = TRUE;
	} else {
		_tmp2 = g_str_has_suffix (dataa, ".vapi");
	}
	if (_tmp2) {
		char* _tmp3;
		_tmp3 = NULL;
		dataa = (_tmp3 = string_substring (dataa, (glong) 0, string_get_length (dataa) - 5), dataa = (g_free (dataa), NULL), _tmp3);
	}
	_tmp4 = FALSE;
	if (g_str_has_suffix (datab, ".vala")) {
		_tmp4 = TRUE;
	} else {
		_tmp4 = g_str_has_suffix (datab, ".vapi");
	}
	if (_tmp4) {
		char* _tmp5;
		_tmp5 = NULL;
		datab = (_tmp5 = string_substring (datab, (glong) 0, string_get_length (datab) - 5), datab = (g_free (datab), NULL), _tmp5);
	}
	return (_tmp6 = strcmp (dataa, datab), dataa = (g_free (dataa), NULL), datab = (g_free (datab), NULL), _tmp6);
}


GtkBuilder* vtg_utils_get_builder (void) {
	GError * inner_error;
	GtkBuilder* _tmp2;
	inner_error = NULL;
	if (vtg_utils__builder == NULL) {
		GtkBuilder* _tmp0;
		_tmp0 = NULL;
		vtg_utils__builder = (_tmp0 = gtk_builder_new (), (vtg_utils__builder == NULL) ? NULL : (vtg_utils__builder = (g_object_unref (vtg_utils__builder), NULL)), _tmp0);
		{
			char* _tmp1;
			_tmp1 = NULL;
			gtk_builder_add_from_file (vtg_utils__builder, _tmp1 = vtg_utils_get_ui_path ("vtg.ui"), &inner_error);
			if (inner_error != NULL) {
				goto __catch36_g_error;
				goto __finally36;
			}
			_tmp1 = (g_free (_tmp1), NULL);
		}
		goto __finally36;
		__catch36_g_error:
		{
			GError * err;
			err = inner_error;
			inner_error = NULL;
			{
				g_warning ("vtgutils.vala:220: initialize_ui: %s", err->message);
				(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
			}
		}
		__finally36:
		if (inner_error != NULL) {
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
			g_clear_error (&inner_error);
			return NULL;
		}
	}
	_tmp2 = NULL;
	return (_tmp2 = vtg_utils__builder, (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2));
}


GscProposal** vtg_utils_get_proposal_cache (int* result_length1) {
	GscProposal** _tmp0;
	if (!vtg_utils__initialized) {
		vtg_utils_initialize ();
	}
	_tmp0 = NULL;
	return (_tmp0 = vtg_utils__proposals, *result_length1 = vtg_utils__proposals_length1, _tmp0);
}


char* vtg_utils_get_image_path (const char* id) {
	char* result;
	g_return_val_if_fail (id != NULL, NULL);
	result = g_build_filename (PACKAGE_DATADIR, "images", id, NULL);
	return result;
}


char* vtg_utils_get_ui_path (const char* id) {
	char* result;
	g_return_val_if_fail (id != NULL, NULL);
	result = g_build_filename (PACKAGE_DATADIR, "ui", id, NULL);
	return result;
}


static void vtg_utils_initialize (void) {
	GError * inner_error;
	inner_error = NULL;
	{
		GscProposal** _tmp0;
		GdkPixbuf* _tmp1;
		GdkPixbuf* _icon_generic;
		_tmp0 = NULL;
		vtg_utils__proposals = (_tmp0 = g_new0 (GscProposal*, VTG_UTILS_prealloc_count + 1), vtg_utils__proposals = (_vala_array_free (vtg_utils__proposals, vtg_utils__proposals_length1, (GDestroyNotify) g_object_unref), NULL), vtg_utils__proposals_length1 = VTG_UTILS_prealloc_count, vtg_utils__proposals_size = vtg_utils__proposals_length1, _tmp0);
		_tmp1 = NULL;
		_icon_generic = (_tmp1 = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), GTK_STOCK_FILE, 16, GTK_ICON_LOOKUP_GENERIC_FALLBACK, &inner_error), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
		if (inner_error != NULL) {
			goto __catch37_g_error;
			goto __finally37;
		}
		{
			gint idx;
			idx = 0;
			for (; idx < VTG_UTILS_prealloc_count; idx++) {
				GscProposal* obj;
				GscProposal* _tmp3;
				GscProposal* _tmp2;
				obj = gsc_proposal_new ("", "", _icon_generic);
				_tmp3 = NULL;
				_tmp2 = NULL;
				vtg_utils__proposals[idx] = (_tmp3 = (_tmp2 = obj, (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2)), (vtg_utils__proposals[idx] == NULL) ? NULL : (vtg_utils__proposals[idx] = (g_object_unref (vtg_utils__proposals[idx]), NULL)), _tmp3);
				(obj == NULL) ? NULL : (obj = (g_object_unref (obj), NULL));
			}
		}
		vtg_utils__initialized = TRUE;
		(_icon_generic == NULL) ? NULL : (_icon_generic = (g_object_unref (_icon_generic), NULL));
	}
	goto __finally37;
	__catch37_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			g_warning (err->message);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally37:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
}


GeeList* vtg_utils_get_available_packages (void) {
	GeeList* _tmp0;
	if (vtg_utils__available_packages == NULL) {
		vtg_utils_initialize_packages_cache ();
	}
	_tmp0 = NULL;
	return (_tmp0 = vtg_utils__available_packages, (_tmp0 == NULL) ? NULL : gee_collection_object_ref (_tmp0));
}


static void _g_list_free_g_free (GList* self) {
	g_list_foreach (self, (GFunc) g_free, NULL);
	g_list_free (self);
}


static void vtg_utils_initialize_packages_cache (void) {
	GError * inner_error;
	GList* vapidirs;
	GeeList* _tmp0;
	inner_error = NULL;
	vapidirs = NULL;
	vapidirs = g_list_append (vapidirs, g_strdup ("/usr/share/vala/vapi"));
	vapidirs = g_list_append (vapidirs, g_strdup ("/usr/local/share/vala/vapi"));
	_tmp0 = NULL;
	vtg_utils__available_packages = (_tmp0 = (GeeList*) gee_array_list_new (VTG_PROJECT_MANAGER_TYPE_PROJECT_PACKAGE, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal), (vtg_utils__available_packages == NULL) ? NULL : (vtg_utils__available_packages = (gee_collection_object_unref (vtg_utils__available_packages), NULL)), _tmp0);
	{
		GList* vapidir_collection;
		GList* vapidir_it;
		vapidir_collection = vapidirs;
		for (vapidir_it = vapidir_collection; vapidir_it != NULL; vapidir_it = vapidir_it->next) {
			const char* _tmp9;
			char* vapidir;
			_tmp9 = NULL;
			vapidir = (_tmp9 = (const char*) vapidir_it->data, (_tmp9 == NULL) ? NULL : g_strdup (_tmp9));
			{
				GDir* dir;
				const char* _tmp3;
				char* filename;
				dir = NULL;
				{
					GDir* _tmp1;
					GDir* _tmp2;
					_tmp1 = g_dir_open (vapidir, 0, &inner_error);
					if (inner_error != NULL) {
						if (inner_error->domain == G_FILE_ERROR) {
							goto __catch38_g_file_error;
						}
						goto __finally38;
					}
					_tmp2 = NULL;
					dir = (_tmp2 = _tmp1, (dir == NULL) ? NULL : (dir = (g_dir_close (dir), NULL)), _tmp2);
				}
				goto __finally38;
				__catch38_g_file_error:
				{
					GError * err;
					err = inner_error;
					inner_error = NULL;
					{
						/*do nothing*/
						(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
						vapidir = (g_free (vapidir), NULL);
						(dir == NULL) ? NULL : (dir = (g_dir_close (dir), NULL));
						continue;
					}
				}
				__finally38:
				if (inner_error != NULL) {
					vapidir = (g_free (vapidir), NULL);
					(dir == NULL) ? NULL : (dir = (g_dir_close (dir), NULL));
					(vapidirs == NULL) ? NULL : (vapidirs = (_g_list_free_g_free (vapidirs), NULL));
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
					g_clear_error (&inner_error);
					return;
				}
				_tmp3 = NULL;
				filename = (_tmp3 = g_dir_read_name (dir), (_tmp3 == NULL) ? NULL : g_strdup (_tmp3));
				while (filename != NULL) {
					char* _tmp8;
					const char* _tmp7;
					if (g_str_has_suffix (filename, ".vapi")) {
						char* _tmp4;
						VtgProjectManagerProjectPackage* _tmp6;
						char* _tmp5;
						_tmp4 = NULL;
						filename = (_tmp4 = g_utf8_strdown (filename, -1), filename = (g_free (filename), NULL), _tmp4);
						_tmp6 = NULL;
						_tmp5 = NULL;
						gee_collection_add ((GeeCollection*) vtg_utils__available_packages, _tmp6 = vtg_project_manager_project_package_new (_tmp5 = string_substring (filename, (glong) 0, string_get_length (filename) - 5)));
						(_tmp6 == NULL) ? NULL : (_tmp6 = (g_object_unref (_tmp6), NULL));
						_tmp5 = (g_free (_tmp5), NULL);
					}
					_tmp8 = NULL;
					_tmp7 = NULL;
					filename = (_tmp8 = (_tmp7 = g_dir_read_name (dir), (_tmp7 == NULL) ? NULL : g_strdup (_tmp7)), filename = (g_free (filename), NULL), _tmp8);
				}
				vapidir = (g_free (vapidir), NULL);
				(dir == NULL) ? NULL : (dir = (g_dir_close (dir), NULL));
				filename = (g_free (filename), NULL);
			}
		}
	}
	(vapidirs == NULL) ? NULL : (vapidirs = (_g_list_free_g_free (vapidirs), NULL));
}


VtgUtils* vtg_utils_construct (GType object_type) {
	VtgUtils * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


VtgUtils* vtg_utils_new (void) {
	return vtg_utils_construct (VTG_TYPE_UTILS);
}


static void vtg_utils_class_init (VtgUtilsClass * klass) {
	vtg_utils_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->finalize = vtg_utils_finalize;
}


static void vtg_utils_instance_init (VtgUtils * self) {
}


static void vtg_utils_finalize (GObject* obj) {
	VtgUtils * self;
	self = VTG_UTILS (obj);
	G_OBJECT_CLASS (vtg_utils_parent_class)->finalize (obj);
}


static GType vtg_utils_type_id = 0;
GType vtg_utils_get_type (void) {
	return vtg_utils_type_id;
}


GType vtg_utils_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgUtilsClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_utils_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgUtils), 0, (GInstanceInitFunc) vtg_utils_instance_init, NULL };
	vtg_utils_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgUtils", &g_define_type_info, 0);
	return vtg_utils_type_id;
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




